@@ -21,6 +21,7 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 
+#include <pcmcia/config.h>
 #include <pcmcia/cs_types.h>
 #include <pcmcia/cs.h>
 #include <pcmcia/cistpl.h>
@@ -28,15 +29,15 @@
 
 #if 0
 static const char *version =
-"cardinfo.c 1.16 1997/03/21 07:24:30 (David Hinds)";
+"cardinfo.c 1.20 1997/05/12 16:19:23 (David Hinds)";
 #endif
 
 /*====================================================================*/
 
 #ifdef ETC
-static char *config = ETC;
+static char *configpath = ETC;
 #else
-static char *config = "/etc/pcmcia";
+static char *configpath = "/etc/pcmcia";
 #endif
 
 static char *scheme = "/var/run/pcmcia-scheme";
@@ -53,7 +54,7 @@ static int lookup_dev(char *name)
     
     f = fopen("/proc/devices", "r");
     if (f == NULL)
-	return -1;
+	return -errno;
     while (fgets(s, 32, f) != NULL) {
 	if (sscanf(s, "%d %s", &n, t) == 2)
 	    if (strcmp(name, t) == 0)
@@ -63,7 +64,7 @@ static int lookup_dev(char *name)
     if (strcmp(name, t) == 0)
 	return n;
     else
-	return -1;
+	return -ENODEV;
 } /* lookup_dev */
 
 /*====================================================================*/
@@ -276,22 +277,81 @@ static int do_cmd(int fd, int cmd)
     
 ======================================================================*/
 
-static int execute(FILE *stab, char *action)
+typedef struct stab_t {
+    int		status;
+    char	class[33];
+    char	dev[33];
+} stab_t;
+
+static stab_t stab[256];
+static int nstab;
+
+static int fetch_stab(void)
 {
-    char s[133], class[33], dev[33];
-    int ret;
-    
-    while (fgets(s, 132, stab))
+    char s[133];
+    FILE *f;
+
+    f = fopen("/var/run/stab", "r");
+    if (f == NULL)
+	return -1;
+    for (nstab = 0; fgets(s, 132, f); ) {
 	if (s[0] != 'S') {
-	    sscanf(s, "%*d\t%s\t%*s\t%*d\t%s", class, dev);
-	    sprintf(s, "%s/%s %s %s", config, class, action, dev);
-	    ret = system(s);
-	    if (!WIFEXITED(ret) || WEXITSTATUS(ret))
-		return -1;
+	    sscanf(s, "%*d\t%s\t%*s\t%*d\t%s",
+		   stab[nstab].class, stab[nstab].dev);
+	    stab[nstab].status = 0;
+	    nstab++;
 	}
+    }
+    fclose(f);
     return 0;
 }
 
+static int execute(stab_t *s, char *action, char *scheme)
+{
+    int ret;
+    char cmd[133];
+    
+    if (scheme)
+	sprintf(cmd, "./%s %s %s %s", s->class, action, s->dev, scheme);
+    else
+	sprintf(cmd, "./%s %s %s", s->class, action, s->dev);
+    ret = system(cmd);
+    if (!WIFEXITED(ret) || WEXITSTATUS(ret))
+	return -1;
+    return 0;
+}
+
+static int stop_scheme(char *new)
+{
+    int i;
+    
+    fprintf(stderr, "checking:");
+    for (i = 0; i < nstab; i++) {
+	fprintf(stderr, " %s", stab[i].dev);
+	stab[i].status = execute(stab+i, "cksum", new);
+	if (stab[i].status &&
+	    (execute(stab+i, "check", NULL) != 0)) break;
+    }
+    fprintf(stderr, "\n");
+    if (i < nstab) {
+	fprintf(stderr, "Device '%s' busy: scheme unchanged.\n",
+		stab[i].dev);
+	return -1;
+    }
+    for (i = 0; i < nstab; i++)
+	if (stab[i].status) execute(stab+i, "stop", NULL);
+    return 0;
+}
+
+static int start_scheme(void)
+{
+    int i, j = 0;
+    
+    for (i = 0; i < nstab; i++)
+	if (stab[i].status) j |= execute(stab+i, "start", NULL);
+    return j;
+}
+
 /*======================================================================
 
     do_scheme() is in charge of checking and updating the current 
@@ -303,7 +363,7 @@ static int execute(FILE *stab, char *action)
 
 static int do_scheme(char *new)
 {
-    FILE *f, *stab;
+    FILE *f;
     char old[33];
     
     f = fopen(scheme, "r");
@@ -315,10 +375,12 @@ static int do_scheme(char *new)
     
     if (new) {
 
+#ifndef UNSAFE_TOOLS
 	if (getuid() != 0) {
 	    fprintf(stderr, "Only root can select a new scheme.\n");
 	    return -1;
 	}
+#endif
 	
 	/* Sanity checks... */
 	if ((strlen(new) < 1) || (strlen(new) > 32) ||
@@ -330,17 +392,15 @@ static int do_scheme(char *new)
 	    fprintf(stderr, "Scheme unchanged.\n");
 	    return 0;
 	}
+
+	if (chdir(configpath) != 0) {
+	    fprintf(stderr, "Could not change to %s.\n", configpath);
+	    return -1;
+	}
 	
 	/* Shut down devices in old scheme */
-	stab = fopen("/var/run/stab", "r");
-	if (stab) {
-	    if (execute(stab, "check") != 0) {
-		fprintf(stderr, "Device(s) busy: scheme unchanged.\n");
-		return -1;
-	    }
-	    rewind(stab);
-	    execute(stab, "stop");
-	}
+	if ((fetch_stab() == 0) && (stop_scheme(new) != 0))
+	    return -1;
 
 	/* Update scheme state */
 	if (old[0])
@@ -353,17 +413,12 @@ static int do_scheme(char *new)
 	if (f) {
 	    fprintf(f, "%s\n", new);
 	    fclose(f);
-	}
-	else
+	} else
 	    perror("Could not set scheme.");
 
 	/* Start up devices in new scheme */
-	if (stab) {
-	    rewind(stab);
-	    if (execute(stab, "start") != 0)
-		fprintf(stderr, "Some devices did not start cleanly.\n");
-	    fclose(stab);
-	}
+	if (start_scheme() != 0)
+	    fprintf(stderr, "Some devices did not start cleanly.\n");
 	
     }
     
@@ -382,7 +437,7 @@ void usage(char *name)
 {
     int i;
     fprintf(stderr, "usage: %s command [socket #]\n", name);
-    fprintf(stderr, "    or %s [-c config] [-f scheme]"
+    fprintf(stderr, "    or %s [-c configpath] [-f scheme]"
 	    " [-s stab] scheme [name]\n", name);
     fprintf(stderr, "  commands:");
     for (i = 0; i < NCMD; i++)
@@ -405,7 +460,7 @@ void main(int argc, char *argv[])
     while ((optch = getopt(argc, argv, "c:f:s:")) != -1) {
 	switch (optch) {
 	case 'c':
-	    config = strdup(optarg); break;
+	    configpath = strdup(optarg); break;
 	case 'f':
 	    scheme = strdup(optarg); break;
 	case 's':
@@ -425,12 +480,17 @@ void main(int argc, char *argv[])
     
     major = lookup_dev("pcmcia");
     if (major < 0) {
-	fprintf(stderr, "no pcmcia driver in /proc/devices\n");
+	if (major == -ENODEV)
+	    fprintf(stderr, "no pcmcia driver in /proc/devices\n");
+	else
+	    perror("could not open /proc/devices");
 	exit(EXIT_FAILURE);
     }
 
     if (strcmp(argv[1], "scheme") == 0) {
+#ifndef UNSAFE_TOOLS
 	setuid(getuid());
+#endif
 	if (do_scheme((argc == 2) ? NULL : argv[2]) == 0)
 	    exit(EXIT_SUCCESS);
 	else
@@ -450,7 +510,9 @@ void main(int argc, char *argv[])
 	    perror("open_dev()");
 	    exit(EXIT_FAILURE);
 	}
+#ifndef UNSAFE_TOOLS
 	setuid(getuid());
+#endif
 	ret = do_cmd(fd[0], cmd);
 	if (ret != 0)
 	    perror("ioctl()");
@@ -460,7 +522,9 @@ void main(int argc, char *argv[])
 	    fd[ns] = open_dev((major<<8)+ns);
 	    if (fd[ns] < 0) break;
 	}
+#ifndef UNSAFE_TOOLS
 	setuid(getuid());
+#endif
 	if (ns == 0) {
 	    perror("open_dev()");
 	    exit(EXIT_FAILURE);
