@@ -33,7 +33,7 @@
 
 #if 0
 static const char *version =
-"cardinfo.c 1.13 1997/03/21 07:24:40 (David Hinds)";
+"cardinfo.c 1.15 1997/04/30 16:34:33 (David Hinds)";
 #endif
 
 /*====================================================================*/
@@ -100,7 +100,7 @@ static int lookup_dev(char *name)
     
     f = fopen("/proc/devices", "r");
     if (f == NULL)
-	return -1;
+	return -errno;
     while (fgets(s, 32, f) != NULL) {
 	if (sscanf(s, "%d %s", &n, t) == 2)
 	    if (strcmp(name, t) == 0)
@@ -110,7 +110,7 @@ static int lookup_dev(char *name)
     if (strcmp(name, t) == 0)
 	return n;
     else
-	return -1;
+	return -ENODEV;
 } /* lookup_dev */
 
 /*====================================================================*/
@@ -428,7 +428,10 @@ void main(int argc, char *argv[])
     
     major = lookup_dev("pcmcia");
     if (major < 0) {
-	fprintf(stderr, "no pcmcia driver in /proc/devices\n");
+	if (major == -ENODEV)
+	    fprintf(stderr, "no pcmcia driver in /proc/devices\n");
+	else
+	    perror("could not open /proc/devices");
 	exit(EXIT_FAILURE);
     }
     
@@ -453,7 +456,9 @@ void main(int argc, char *argv[])
     }
     
     /* Switch back to real user privileges, to be safe */
+#ifndef UNSAFE_TOOLS
     setuid(getuid());
+#endif
 
     if ((ret = fork()) > 0) exit(0);
     if (ret == -1)
