@@ -44,9 +44,6 @@
 
 #include "cardmgr.h"
 
-static const char *version =
-    "cardmgr.c 1.87 1997/03/21 17:08:22 (David Hinds)";
-
 /*====================================================================*/
 
 typedef struct socket_info_t {
@@ -115,7 +112,7 @@ static int caught_signal = 0;
 
 /*====================================================================*/
 
-int lookup_dev(char *name)
+static int lookup_dev(char *name)
 {
     FILE *f;
     int n;
@@ -123,7 +120,7 @@ int lookup_dev(char *name)
     
     f = fopen("/proc/devices", "r");
     if (f == NULL)
-	return -1;
+	return -errno;
     while (fgets(s, 32, f) != NULL) {
 	if (sscanf(s, "%d %s", &n, t) == 2)
 	    if (strcmp(name, t) == 0)
@@ -133,7 +130,7 @@ int lookup_dev(char *name)
     if (strcmp(name, t) == 0)
 	return n;
     else
-	return -1;
+	return -ENODEV;
 } /* lookup_dev */
 
 /*====================================================================*/
@@ -600,8 +597,8 @@ static void remove_module(char *mod)
 		s = mod;
 	    else
 		s++;
-	    sprintf(cmd, "rmmod %s", s);
-	    execute("rmmod", cmd);
+	    sprintf(cmd, do_modprobe ? "modprobe -r %s" : "rmmod %s", s);
+	    execute(do_modprobe ? "modprobe" : "rmmod", cmd);
 	}
     }
 }
@@ -1029,7 +1026,10 @@ static int init_sockets(void)
     
     major = lookup_dev("pcmcia");
     if (major < 0) {
-	syslog(LOG_INFO, "no pcmcia driver in /proc/devices");
+	if (major == -ENODEV)
+	    syslog(LOG_INFO, "no pcmcia driver in /proc/devices");
+	else
+	    syslog(LOG_INFO, "could not open /proc/devices: %m");
 	exit(EXIT_FAILURE);
     }
     for (i = 0; i < MAX_SOCKS; i++) {
@@ -1122,10 +1122,10 @@ void main(int argc, char *argv[])
 	fork_now();
 #endif
     
-    syslog(LOG_INFO, "starting");
-    syslog(LOG_INFO, "%s", version);
+    syslog(LOG_INFO, "starting, version is " CS_RELEASE);
     atexit(&done);
-
+    setenv("PATH", "/bin:/sbin:/usr/bin:/usr/sbin", 1);
+    
     if (modpath == NULL) {
 	struct utsname utsname;
 	if (uname(&utsname) != 0) {
