@@ -23,7 +23,7 @@
 #include "tcic.h"
 
 static const char *version =
-"probe.c 1.20 1996/12/06 16:51:34 (David Hinds)\n";
+"probe.c 1.21 1997/03/21 07:25:23 (David Hinds)\n";
 
 static int i365_base = 0x03e0;
 
@@ -38,6 +38,7 @@ typedef struct {
 
 pci_id_t pci_id[] = {
     { "Cirrus Logic CL 6729", "Cirrus PD6729" },
+    { "Vendor id=119b. Device id=1221.", "Omega Micro 82C092G" },
     { "Vendor id=1180. Device id=466.", "Ricoh RL5C466" },
     { "Vendor id=10b3. Device id=b106.", "SMC 34C90" },
     { "Vendor id=1013. Device id=1110.", "Cirrus PD6832" },
@@ -85,17 +86,15 @@ static int pci_probe(int verbose, int module)
 
 static u_char i365_get(u_short sock, u_short reg)
 {
-    u_short port = I365_PORT(sock);
     u_char val = I365_REG(sock, reg);
-    outb_p(val, port); val = inb_p(port+1);
+    outb_p(val, i365_base); val = inb_p(i365_base+1);
     return val;
 }
 
 static void i365_set(u_short sock, u_short reg, u_char data)
 {
-    u_short port = I365_PORT(sock);
     u_char val = I365_REG(sock, reg);
-    outb_p(val, port); outb_p(data, port+1);
+    outb_p(val, i365_base); outb_p(data, i365_base+1);
 }
 
 static void i365_bset(u_short sock, u_short reg, u_char mask)
@@ -162,8 +161,8 @@ int i365_probe(int verbose, int module)
 	name = "i82365sl DF";
 
     /* Check for Vadem chips */
-    outb_p(0x0e, I365_PORT(0));
-    outb_p(0x37, I365_PORT(0));
+    outb_p(0x0e, i365_base);
+    outb_p(0x37, i365_base);
     i365_bset(0, VG468_MISC, VG468_MISC_VADEMREV);
     val = i365_get(0, I365_IDENT);
     if (val & I365_IDENT_VADEM) {
