@@ -4,7 +4,7 @@
 
     Written by David Hinds, dhinds@allegro.stanford.edu
 
-    scsi_info.c 1.5 1996/11/26 07:25:29
+    scsi_info.c 1.6 1997/05/07 16:10:48
     
 ======================================================================*/
 
@@ -103,7 +103,7 @@ void main(int argc, char *argv[])
     
     f = fopen("/proc/scsi/scsi", "r");
     if (f == NULL) {
-	printf("MODEL=\"Unknown\"\n");
+	printf("MODEL=\"Unknown\"\nFW_REV=\"Unknown\"\n");
 	exit(0);
     }
 
@@ -119,6 +119,6 @@ void main(int argc, char *argv[])
     strncpy(rev, s+48, 4); rev[4] = '\0';
     for (i = 3; (i >= 0) && (rev[i] == ' '); i--)
 	rev[i] = '\0';
-    printf("MODEL=\"%s %s %s\"\n", vendor, model, rev);
+    printf("MODEL=\"%s %s\"\nFW_REV=\"%s\"\n", vendor, model, rev);
     exit(0);
 }
