@@ -26,7 +26,7 @@
 #define NUMBER 282
 typedef union {
     char *str;
-    int num;
+    long num;
     struct device_info_t *device;
     struct card_info_t *card;
     struct mtd_ident_t *mtd;
