@@ -10,7 +10,7 @@ static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
 #define YYPREFIX "yy"
 #line 2 "yacc_config.y"
 /*
- * yacc_config.y 1.29 1997/03/21 07:25:01 (David Hinds)\n"
+ * yacc_config.y 1.30 1997/04/15 16:00:00 (David Hinds)\n"
  */
     
 #include <stdlib.h>
@@ -37,7 +37,7 @@ static int add_module(device_info_t *card, char *name);
 #line 35 "yacc_config.y"
 typedef union {
     char *str;
-    int num;
+    long num;
     struct device_info_t *device;
     struct card_info_t *card;
     struct mtd_ident_t *mtd;
