@@ -4,7 +4,7 @@
 
     Written by David Hinds, dhinds@allegro.stanford.edu
 
-    ftl_format.c 1.3 1996/11/26 07:24:56
+    ftl_format.c 1.4 1997/05/23 15:56:58
 
 ======================================================================*/
 
@@ -110,7 +110,7 @@ static int format_partition(int fd, int quiet, int interrogate,
 	print_size(region.RegionSize);
 	printf(", erase unit size = ");
 	print_size(region.BlockSize);
-	printf(", %ld tranfer units\n", spare);
+	printf(", %ld transfer units\n", spare);
 	if (bootsize != 0) {
 	    print_size(hdr.FirstPhysicalEUN << hdr.EraseUnitSize);
 	    printf(" allocated for boot image\n");
@@ -231,7 +231,7 @@ void main(int argc, char *argv[])
     errflg = 0;
     bootsize = 0;
     
-    while ((optch = getopt(argc, argv, "qis:b:")) != -1) {
+    while ((optch = getopt(argc, argv, "qirs:b:")) != -1) {
 	switch (optch) {
 	case 'q':
 	    quiet = 1; break;
