@@ -1,5 +1,5 @@
 /*
- * ciscode.h 1.8 1997/01/26 06:27:54 (David Hinds)
+ * ciscode.h 1.14 1997/06/04 14:58:43 (David Hinds)
  */
 
 #ifndef _LINUX_CISCODE_H
@@ -16,10 +16,15 @@
 #define MANFID_ATT			0xffff
 #define PRODID_ATT_KIT			0x0100
 
+#define MANFID_CONTEC			0xc001
+
+#define MANFID_FUJITSU			0x0004
+
 #define MANFID_IBM			0x00a4
 #define PRODID_IBM_HOME_AND_AWAY	0x002e
 
 #define MANFID_INTEL			0x0089
+#define PRODID_INTEL_DUAL_RS232		0x0301
 #define PRODID_INTEL_2PLUS		0x8422
 
 #define MANFID_LINKSYS			0x0143
@@ -31,12 +36,16 @@
 
 #define MANFID_MACNICA			0xc00b
 
+#define MANFID_NEW_MEDIA		0x0057
+
 #define MANFID_OSITECH			0x0140
 #define PRODID_OSITECH_JACK_144		0x0001
 #define PRODID_OSITECH_JACK_288		0x0002
+#define PRODID_OSITECH_JACK_336		0x0007
 
 #define MANFID_QUATECH			0x0137
 #define PRODID_QUATECH_DUAL_RS232	0x0012
+#define PRODID_QUATECH_QUAD_RS232	0x001b
 
 #define MANFID_SMC			0x0108
 #define PRODID_SMC_ETHER		0x0105
@@ -46,4 +55,6 @@
 
 #define MANFID_SUNDISK			0x0045
 
+#define MANFID_TDK			0x0105
+
 #endif /* _LINUX_CISCODE_H */
