@@ -1,5 +1,5 @@
 /*
- * k_compat.h 1.28 1997/03/12 19:23:31 (David Hinds)
+ * k_compat.h 1.30 1997/05/21 16:31:39 (David Hinds)
  */
 
 #ifndef _LINUX_K_COMPAT_H
@@ -15,6 +15,12 @@
 #define U_FS_SIZE_T unsigned long
 #endif
 
+#if (LINUX_VERSION_CODE < VERSION(2,1,31))
+#define FS_RELEASE_T void
+#else
+#define FS_RELEASE_T int
+#endif
+
 #if (LINUX_VERSION_CODE < VERSION(2,1,25))
 #define net_device_stats enet_statistics
 #endif
@@ -143,4 +149,8 @@ char kernel_version[] = UTS_RELEASE;
 #define init_waitqueue(p) (*(p) = NULL)
 #endif
 
+#if (LINUX_VERSION_CODE < VERSION(2,1,38))
+#define test_and_set_bit set_bit
+#endif
+
 #endif /* _LINUX_K_COMPAT_H */
