@@ -45,9 +45,6 @@
 #include <pcmcia/ciscode.h>
 #include <pcmcia/ds.h>
 
-/* Include code for transceiver autodetection? */
-/*#define AUTODETECT*/
-
 /* To minimize the size of the driver source I only define operating
    constants if they are used several times.  You'll need the manual
    if you want to understand driver details. */
@@ -96,11 +93,8 @@ enum RxFilter {
 struct el3_private {
     dev_node_t node;
     struct net_device_stats stats;
-#ifdef AUTODETECT
     int probe_port;
-    struct timer_list check;
     int flipped;
-#endif
 };
 
 static char *if_names[] = { "Auto", "10baseT", "10base2", "AUI" };
@@ -108,24 +102,19 @@ static char *if_names[] = { "Auto", "10baseT", "10base2", "AUI" };
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"3c589_cs.c $Revision: 1.88 $ $Date: 1997/02/08 07:57:48 $ (David Hinds)";
-#endif
-
-#ifdef TC589_DEBUG
-static int tc589_debug = TC589_DEBUG;
+"3c589_cs.c 1.96 1997/06/01 05:03:16 (David Hinds)";
 #else
-static int tc589_debug = 3;
+#define DEBUG(n, args...)
 #endif
 
-MODULE_PARM(tc589_debug, "i");
-
 /*====================================================================*/
 
 /* Parameters that can be set with 'insmod' */
 
 /* Special hook for setting if_port when module is loaded */
-static int if_port = 1;
+static int if_port = 0;
 
 /* Bit map of interrupts to choose from */
 static u_long irq_mask = 0xdeb8;
@@ -142,9 +131,7 @@ static int tc589_event(event_t event, int priority,
 
 static ushort read_eeprom(short ioaddr, int index);
 static void tc589_reset(struct device *dev);
-#ifdef AUTODETECT
-static void check_if_port(u_long i);
-#endif
+static void check_if_port(struct device *dev);
 static int el3_config(struct device *dev, struct ifmap *map);
 static int el3_open(struct device *dev);
 static int el3_start_xmit(struct sk_buff *skb, struct device *dev);
@@ -200,11 +187,8 @@ static dev_link_t *tc589_attach(void)
     dev_link_t *link;
     struct device *dev;
     int ret;
-    
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "3c589_attach()\n");
-#endif
+
+    DEBUG(0, "3c589_attach()\n");
 
     /* Create new ethernet device */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -279,10 +263,7 @@ static void tc589_detach(dev_link_t *link)
 {
     dev_link_t **linkp;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "3c589_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "3c589_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -335,10 +316,7 @@ static void tc589_config(dev_link_t *link)
     dev = link->priv;
     phys_addr = (short *)dev->dev_addr;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "3c589_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "3c589_config(0x%p)\n", link);
 
     do {
 	tuple.Attributes = 0;
@@ -369,8 +347,7 @@ static void tc589_config(dev_link_t *link)
 	if (buf[0] != MANFID_3COM)
 	    printk(KERN_INFO "3c589_cs: hmmm, is this really a "
 		   "3Com card??\n");
-	if (buf[1] == PRODID_3COM_3C562)
-	    multi = 1;
+	multi = (buf[1] == PRODID_3COM_3C562);
     }
     
     /* Configure card */
@@ -378,8 +355,9 @@ static void tc589_config(dev_link_t *link)
 
     do {
 	/* For the 3c562, the base address must be xx00-xx7f */
-	for (j = 0x300; j < (multi ? 0x380 : 0x400); j += 0x10) {
-	    link->io.BasePort1 = j;
+	for (j = 0; j < 0x400; j += 0x10) {
+	    if (multi && (j & 0x80)) continue;
+	    link->io.BasePort1 = j ^ 0x300;
 	    i = CardServices(RequestIO, link->handle, &link->io);
 	    if (i == CS_SUCCESS) break;
 	}
@@ -414,30 +392,35 @@ static void tc589_config(dev_link_t *link)
 	return;
     }
 
-    link->dev = &((struct el3_private *)dev->priv)->node;
-    
     ioaddr = dev->base_addr;
     EL3WINDOW(0);
 
     /* The 3c589 has an extra EEPROM for configuration info, including
        the hardware address.  The 3c562 puts the address in the CIS. */
-    if (multi) {
-	tuple.DesiredTuple = 0x88;
-	CardServices(GetFirstTuple, handle, &tuple);
+    tuple.DesiredTuple = 0x88;
+    if (CardServices(GetFirstTuple, handle, &tuple) == CS_SUCCESS) {
 	CardServices(GetTupleData, handle, &tuple);
 	for (i = 0; i < 3; i++)
 	    phys_addr[i] = htons(buf[i]);
-    }
-    else
+    } else {
 	for (i = 0; i < 3; i++)
 	    phys_addr[i] = htons(read_eeprom(ioaddr, i));
+	if (phys_addr[0] == 0x6060) {
+	    printk(KERN_NOTICE "3c589_cs: IO port conflict at 0x%03lx"
+		   "-0x%03lx\n", dev->base_addr, dev->base_addr+15);
+	    tc589_release((u_long)link);
+	    return;
+	}
+    }
+    
+    link->dev = &((struct el3_private *)dev->priv)->node;
     
     /* The address and resource configuration register aren't loaded from
        the EEPROM and *must* be set to 0 and IRQ3 for the PCMCIA version. */
     outw(0x3f00, ioaddr + 8);
 
     /* The if_port symbol can be set when the module is loaded */
-    if ((if_port >= 1) && (if_port <= 3))
+    if ((if_port >= 0) && (if_port <= 3))
 	dev->if_port = if_port;
     else
 	printk(KERN_NOTICE "3c589_cs: invalid if_port requested\n");
@@ -445,7 +428,7 @@ static void tc589_config(dev_link_t *link)
     /* This is vital: the transceiver used must be set in the resource
        configuration register.  It took me many hours to discover this. */
     switch (dev->if_port) {
-    case 1: outw(0, ioaddr + 6); break;
+    case 0: case 1: outw(0, ioaddr + 6); break;
     case 2: outw(3<<14, ioaddr + 6); break;
     case 3: outw(1<<14, ioaddr + 6); break;
     }
@@ -472,17 +455,11 @@ static void tc589_release(u_long arg)
     dev_link_t *link = (dev_link_t *)arg;
     struct device *dev = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "3c589_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "3c589_release(0x%p)\n", link);
     
     if (link->open) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "3c589_cs: release postponed, '%s' "
-		   "still open\n", link->dev->dev_name);
-#endif
+	DEBUG(1, "3c589_cs: release postponed, '%s' still open\n",
+	      link->dev->dev_name);
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
@@ -517,18 +494,9 @@ static int tc589_event(event_t event, int priority,
     dev_link_t *link = args->client_data;
     struct device *dev = link->priv;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "3c589_event()\n");
-#endif
+    DEBUG(1, "3c589_event(0x%06x)\n", event);
     
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "3c589_cs: registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -590,7 +558,7 @@ static void tc589_reset(struct device *dev)
     EL3WINDOW(0);
     outw(0x0001, ioaddr + 4);			/* Activate board. */ 
     switch (dev->if_port) {			/* Set the xcvr */
-    case 1: outw(0, ioaddr + 6); break;
+    case 0: case 1: outw(0, ioaddr + 6); break;
     case 2: outw(3<<14, ioaddr + 6); break;
     case 3: outw(1<<14, ioaddr + 6); break;
     }
@@ -604,7 +572,7 @@ static void tc589_reset(struct device *dev)
     if (dev->if_port == 2)
 	/* Start the thinnet transceiver. We should really wait 50ms...*/
 	outw(StartCoax, ioaddr + EL3_CMD);
-    else if (dev->if_port == 1) {
+    else if (dev->if_port < 2) {
 	/* 10baseT interface, enabled link beat and jabber check. */
 	EL3WINDOW(4);
 	outw(inw(ioaddr + WN4_MEDIA) | MEDIA_TP, ioaddr + WN4_MEDIA);
@@ -637,13 +605,14 @@ static void tc589_reset(struct device *dev)
 
 static int el3_config(struct device *dev, struct ifmap *map)
 {
+    struct el3_private *lp = (struct el3_private *)dev->priv;
     if ((map->port != (u_char)(-1)) && (map->port != dev->if_port)) {
-	if ((map->port >= 1) && (map->port <= 3)) {
+	if (map->port <= 3) {
+	    lp->probe_port = 0;
 	    dev->if_port = map->port;
 	    printk(KERN_INFO "%s: switched to %s port\n",
 		   dev->name, if_names[dev->if_port]);
-	}
-	else
+	} else
 	    return -EINVAL;
     }
     return 0;
@@ -651,9 +620,7 @@ static int el3_config(struct device *dev, struct ifmap *map)
 
 static int el3_open(struct device *dev)
 {
-#ifdef AUTODETECT
     struct el3_private *lp = (struct el3_private *)dev->priv;
-#endif
     int ioaddr = dev->base_addr;
     dev_link_t *link;
     
@@ -670,16 +637,13 @@ static int el3_open(struct device *dev)
     irq2dev_map[dev->irq] = dev;
     
     dev->interrupt = 0; dev->tbusy = 0; dev->start = 1;
-#ifdef AUTODETECT
-    lp->probe_port = 15; lp->flipped = 0;
-    lp->check.function = &check_if_port;
-    lp->check.data = (u_long)dev;
-#endif
+    if (dev->if_port == 0) {
+	lp->probe_port = 15; lp->flipped = 0;
+    }
     tc589_reset(dev);
-    
-    if (tc589_debug > 3)
-	printk(KERN_DEBUG "%s: opened, status %4.4x.\n",
-	       dev->name, inw(ioaddr + EL3_STATUS));
+
+    DEBUG(2, "%s: opened, status %4.4x.\n",
+	  dev->name, inw(ioaddr + EL3_STATUS));
     
     return 0;					/* Always succeed */
 }
@@ -689,11 +653,6 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
     struct el3_private *lp = (struct el3_private *)dev->priv;
     int ioaddr = dev->base_addr;
 
-#ifdef AUTODETECT
-    if (lp->probe_port)
-	check_if_port((u_long)dev);
-#endif
-    
     /* Transmitter timeout, serious problems. */
     if (dev->tbusy) {
 	int tickssofar = jiffies - dev->trans_start;
@@ -719,17 +678,16 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
     if (skb->len <= 0)
 	return 0;
 #endif
-    
-    if (tc589_debug > 4) {
-	printk(KERN_DEBUG "%s: el3_start_xmit(length = %ld) called, "
-	       "status %4.4x.\n", dev->name, (long)skb->len,
-	       inw(ioaddr + EL3_STATUS));
-    }
+
+    DEBUG(3, "%s: el3_start_xmit(length = %ld) called, "
+	  "status %4.4x.\n", dev->name, (long)skb->len,
+	  inw(ioaddr + EL3_STATUS));
+
 #ifdef PCMCIA_DEBUG
     {	/* Error-checking code, delete someday. */
 	ushort status = inw(ioaddr + EL3_STATUS);
 	if (status & 0x0003 		/* IRQ line active, missed one. */
-	    && inw(ioaddr + EL3_STATUS) & 3) { 			/* Make sure. */
+	    && inw(ioaddr + EL3_STATUS) & 3) { 		/* Make sure. */
 	    printk(KERN_NOTICE "%s: missed interrupt, status then %04x"
 		   " now %04x  Tx %2.2x Rx %4.4x.\n", dev->name, status,
 		   inw(ioaddr + EL3_STATUS), inb(ioaddr + TX_STATUS),
@@ -746,7 +704,7 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
 #endif
     
     /* Avoid timer-based retransmission conflicts. */
-    if (set_bit(0, (void*)&dev->tbusy) != 0)
+    if (test_and_set_bit(0, (void*)&dev->tbusy) != 0)
 	printk(KERN_NOTICE "%s: transmitter access conflict.\n",
 	       dev->name);
     else {
@@ -782,12 +740,8 @@ static int el3_start_xmit(struct sk_buff *skb, struct device *dev)
 	}
     }
 
-#ifdef AUTODETECT
-    if (lp->probe_port && (lp->check.next == NULL)) {
-	lp->check.expires = RUN_AT(HZ/20);
-	add_timer(&lp->check);
-    }
-#endif
+    if (lp->probe_port)
+	check_if_port(dev);
     
     return 0;
 }
@@ -812,17 +766,14 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 	return;
     }
     dev->interrupt = 1;
-    if (tc589_debug > 4)
-	printk(KERN_DEBUG "%s: interrupt, status %4.4x.\n",
-	       dev->name, inw(ioaddr + EL3_STATUS));
+    DEBUG(3, "%s: interrupt, status %4.4x.\n",
+	  dev->name, inw(ioaddr + EL3_STATUS));
 #endif
     
     while ((status = inw(ioaddr + EL3_STATUS)) &
 	(IntLatch | RxComplete | StatsFull)) {
 	if ((dev->start == 0) || ((status & 0xe000) != 0x2000)) {
-	    if (tc589_debug > 3)
-		printk(KERN_DEBUG "%s: interrupt from dead card\n",
-		       dev->name);
+	    DEBUG(1, "%s: interrupt from dead card\n", dev->name);
 	    break;
 	}
 	
@@ -830,8 +781,7 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 	    el3_rx(dev);
 	
 	if (status & TxAvailable) {
-	    if (tc589_debug > 5)
-		printk(KERN_DEBUG "    TX room bit was handled.\n");
+	    DEBUG(3, "    TX room bit was handled.\n");
 	    /* There's room in the FIFO for a full-sized packet. */
 	    outw(AckIntr | TxAvailable, ioaddr + EL3_CMD);
 	    dev->tbusy = 0;
@@ -867,17 +817,13 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 	/* Acknowledge the IRQ. */
 	outw(AckIntr | IntReq | IntLatch, ioaddr + EL3_CMD);
 
-#ifdef AUTODETECT
 	if (lp->probe_port)
-	    check_if_port((u_long)dev);
-#endif
+	    check_if_port(dev);
     }
 
 #ifdef PCMCIA_DEBUG
-    if (tc589_debug > 4)
-	printk(KERN_DEBUG "%s: exiting interrupt, status %4.4x.\n",
-	       dev->name, inw(ioaddr + EL3_STATUS));
-    
+    DEBUG(3, "%s: exiting interrupt, status %4.4x.\n",
+	  dev->name, inw(ioaddr + EL3_STATUS));
     dev->interrupt = 0;
 #endif
     return;
@@ -886,14 +832,12 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 /* Check for a transceiver mismatch.  We should get carrier errors when
    there is a mismatch. */
 
-#ifdef AUTODETECT
-static void check_if_port(u_long i)
+static void check_if_port(struct device *dev)
 {
-    struct device *dev = (struct device *)i;
     struct el3_private *lp = (struct el3_private *)dev->priv;
     int ioaddr = dev->base_addr;
-    static int in_check = 0;
-    int flip, carrier;
+    static volatile int in_check = 0;
+    int carrier, tx_packets, rx_packets;
     long flags;
 
     /* We don't want this to get interrupted! */
@@ -909,34 +853,33 @@ static void check_if_port(u_long i)
     /* Switch to the stats window, and read carrier errors. */
     EL3WINDOW(6);
     carrier = inb(ioaddr + 0);
+    tx_packets = inb(ioaddr + 6);
+    rx_packets = inb(ioaddr + 7);
     lp->stats.tx_carrier_errors += carrier;
+    lp->stats.rx_packets += rx_packets;
+    lp->stats.tx_packets += tx_packets;
     /* Back to window 1, and turn statistics back on. */
     EL3WINDOW(1);
     outw(StatsEnable, ioaddr + EL3_CMD);
     restore_flags(flags);
     
-    if (carrier && ((dev->if_port == 1) || (dev->if_port == 2))) {
-	flip = lp->flipped + 1;
-	if (flip == 4)
-	    printk(KERN_NOTICE "%s: network cable problem?\n",
+    if (carrier) {
+	dev->if_port = 2;
+	tc589_reset(dev);
+    } else {
+	if (rx_packets + tx_packets > 0) {
+	    if (dev->if_port == 0) dev->if_port = 1;
+	    printk(KERN_INFO "%s: autodetected %s\n",
+		   dev->name, if_names[dev->if_port]);
+	    lp->probe_port = 1;
+	} else if (lp->probe_port == 1) {
+	    printk(KERN_INFO "%s: network cable problem?\n",
 		   dev->name);
-	el3_close(dev);
-	dev->if_port = (3 - dev->if_port);
-	el3_open(dev);
-	lp->flipped = flip;
-	dev_tint(dev);
-    }
-    else {
-	if (lp->probe_port) {
-	    lp->probe_port--;
-	    if ((lp->probe_port == 0) && lp->flipped)
-		printk(KERN_INFO "%s: flipped port to %s\n",
-		       dev->name, if_names[dev->if_port]);
 	}
     }
+    lp->probe_port--;
     in_check = 0;
 }
-#endif /* AUTODETECT */
 
 static struct net_device_stats *el3_get_stats(struct device *dev)
 {
@@ -959,8 +902,7 @@ static void update_stats(int ioaddr, struct device *dev)
 {
     struct el3_private *lp = (struct el3_private *)dev->priv;
     
-    if (tc589_debug > 5)
-	printk(KERN_DEBUG "%s: updating the statistics.\n", dev->name);
+    DEBUG(2, "%s: updating the statistics.\n", dev->name);
     /* Turn off statistics updates while reading. */
     outw(StatsDisable, ioaddr + EL3_CMD);
     /* Switch to the stats window, and read everything. */
@@ -989,10 +931,8 @@ static int el3_rx(struct device *dev)
     int boguscnt = 10;
     short rx_status;
     
-    if (tc589_debug > 5)
-	printk(KERN_DEBUG "%s: in rx_packet(), status %4.4x, "
-	       "rx_status %4.4x.\n", dev->name,
-	       inw(ioaddr+EL3_STATUS), inw(ioaddr+RX_STATUS));
+    DEBUG(3, "%s: in rx_packet(), status %4.4x, rx_status %4.4x.\n",
+	  dev->name, inw(ioaddr+EL3_STATUS), inw(ioaddr+RX_STATUS));
     while ((rx_status = inw(ioaddr + RX_STATUS)) > 0) {
 	if (rx_status & 0x4000) { /* Error, update stats. */
 	    short error = rx_status & 0x3800;
@@ -1011,9 +951,8 @@ static int el3_rx(struct device *dev)
 	    
 	    skb = ALLOC_SKB(pkt_len+3);
 	    
-	    if (tc589_debug > 4)
-		printk(KERN_DEBUG "    Receiving packet size %d "
-		       "status %4.4x.\n", pkt_len, rx_status);
+	    DEBUG(3, "    Receiving packet size %d status %4.4x.\n",
+		  pkt_len, rx_status);
 	    if (skb != NULL) {
 		skb->dev = dev;
 		
@@ -1027,9 +966,9 @@ static int el3_rx(struct device *dev)
 		lp->stats.rx_bytes += skb->len;
 #endif
 		continue;
-	    } else if (tc589_debug)
-		printk(KERN_DEBUG "%s: couldn't allocate a sk_buff of"
-		       " size %d.\n", dev->name, pkt_len);
+	    } else
+		DEBUG(1, "%s: couldn't allocate a sk_buff of"
+		      " size %d.\n", dev->name, pkt_len);
 	}
 	lp->stats.rx_dropped++;
 	outw(RxDiscard, ioaddr + EL3_CMD); /* Rx discard */
@@ -1053,14 +992,16 @@ static int el3_rx(struct device *dev)
 static void set_multicast_list(struct device *dev)
 {
     short ioaddr = dev->base_addr;
-    if (tc589_debug > 3) {
+#ifdef PCMCIA_DEBUG
+    if (pc_debug > 2) {
 	static int old = 0;
 	if (old != dev->mc_count) {
 	    old = dev->mc_count;
-	    printk(KERN_DEBUG "%s: setting Rx mode to %d addresses.\n",
-		   dev->name, old);
+	    DEBUG(0, "%s: setting Rx mode to %d addresses.\n",
+		  dev->name, old);
 	}
     }
+#endif
     if (dev->flags & IFF_PROMISC)
 	outw(SetRxFilter | RxStation | RxMulticast | RxBroadcast | RxProm,
 	     ioaddr + EL3_CMD);
@@ -1074,14 +1015,16 @@ static void
 set_multicast_list(struct device *dev, int num_addrs, void *addrs)
 {
     short ioaddr = dev->base_addr;
-    if (tc589_debug > 3) {
+#ifdef PCMCIA_DEBUG
+    if (pc_debug > 2) {
 	static int old = 0;
 	if (old != num_addrs) {
 	    old = num_addrs;
-	    printk(KERN_DEBUG "%s: setting Rx mode to %d addresses.\n",
-		   dev->name, num_addrs);
+	    DEBUG(0, "%s: setting Rx mode to %d addresses.\n",
+		  dev->name, num_addrs);
 	}
     }
+#endif
     if ((num_addrs > 0) || (num_addrs == -2))
 	outw(SetRxFilter|RxStation|RxMulticast|RxBroadcast, ioaddr + EL3_CMD);
     else if (num_addrs < 0)
@@ -1102,8 +1045,7 @@ static int el3_close(struct device *dev)
     if (link == NULL)
 	return -ENODEV;
     
-    if (tc589_debug > 3)
-	printk(KERN_DEBUG "%s: shutting down ethercard.\n", dev->name);
+    DEBUG(2, "%s: shutting down ethercard.\n", dev->name);
     
     /* Turn off statistics ASAP.  We update lp->stats below. */
     outw(StatsDisable, ioaddr + EL3_CMD);
@@ -1148,10 +1090,7 @@ static int el3_close(struct device *dev)
 int init_module(void)
 {
     servinfo_t serv;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
 	printk(KERN_NOTICE "serial_cs: Card Services release "
@@ -1164,10 +1103,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "3c589_cs: unloading\n");
-#endif
+    DEBUG(0, "3c589_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL)
 	tc589_detach(dev_list);
