@@ -51,8 +51,11 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"aha152x_cs.c 1.27 1997/03/16 20:03:23 (David Hinds)";
+"aha152x_cs.c 1.29 1997/06/01 05:03:16 (David Hinds)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -67,9 +70,8 @@ static int host_id = 7;
 static int reconnect = 1;
 static int parity = 1;
 static int synchronous = 0;
-
-/* Time to pause during bus reset, in jiffies */
 static int reset_delay = 100;
+static int ext_trans = 0;
 
 MODULE_PARM(irq_mask, "i");
 MODULE_PARM(host_id, "i");
@@ -77,6 +79,7 @@ MODULE_PARM(reconnect, "i");
 MODULE_PARM(parity, "i");
 MODULE_PARM(synchronous, "i");
 MODULE_PARM(reset_delay, "i");
+MODULE_PARM(ext_trans, "i");
 
 /*====================================================================*/
 
@@ -116,10 +119,7 @@ static dev_link_t *aha152x_attach(void)
     dev_link_t *link;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "aha152x_attach()\n");
-#endif
+    DEBUG(0, "aha152x_attach()\n");
 
     /* Create new SCSI device */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -168,10 +168,7 @@ static void aha152x_detach(dev_link_t *link)
 {
     dev_link_t **linkp;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "aha152x_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "aha152x_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -218,10 +215,7 @@ static void aha152x_config_x(dev_link_t *link)
     handle = link->handle;
     info = link->priv;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "aha152x_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "aha152x_config(0x%p)\n", link);
 
     do {
 	tuple.DesiredTuple = CISTPL_CONFIG;
@@ -312,6 +306,9 @@ static void aha152x_config_x(dev_link_t *link)
     ints[5] = parity;
     ints[6] = synchronous;
     ints[7] = reset_delay;
+    if (ext_trans) {
+	ints[8] = ext_trans; ints[0] = 8;
+    }
     aha152x_setup("PCMCIA setup", ints);
     
     scsi_register_module(MODULE_SCSI_HA, &driver_template);
@@ -369,21 +366,15 @@ static void aha152x_release(u_long arg)
 {
     dev_link_t *link = (dev_link_t *)arg;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "aha152x_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "aha152x_release(0x%p)\n", link);
 
 #if (LINUX_VERSION_CODE < VERSION(2,1,23))
     if (*driver_template.usage_count != 0) {
 #else
     if (driver_template.module->usecount != 0) {
 #endif
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "aha152x_cs: release postponed, "
-		   "device still open\n");
-#endif
+	DEBUG(1, "aha152x_cs: release postponed, "
+	      "device still open\n");
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
@@ -409,18 +400,9 @@ static int aha152x_event(event_t event, int priority,
     dev_link_t *link = args->client_data;
     scsi_info_t *info = link->priv;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "aha152x_event()\n");
-#endif
+    DEBUG(0, "aha152x_event(0x%06x)\n", event);
     
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "aha152x_cs: registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -458,10 +440,7 @@ static int aha152x_event(event_t event, int priority,
 
 int init_module(void) {
     servinfo_t serv;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
 	printk(KERN_NOTICE "aha152x_cs: Card Services release "
@@ -473,10 +452,7 @@ int init_module(void) {
 }
 
 void cleanup_module(void) {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "aha152x_cs: unloading\n");
-#endif
+    DEBUG(0, "aha152x_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL)
 	aha152x_detach(dev_list);
