@@ -92,10 +92,7 @@ static int do_mtd_request(memory_handle_t handle, mtd_request_t *req,
 
 static void insert_queue(erase_busy_t *head, erase_busy_t *entry)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "Adding 0x%p to queue 0x%p\n", entry, head);
-#endif
+    DEBUG(2, "cs: adding 0x%p to queue 0x%p\n", entry, head);
     entry->next = head;
     entry->prev = head->prev;
     head->prev->next = entry;
@@ -104,10 +101,7 @@ static void insert_queue(erase_busy_t *head, erase_busy_t *entry)
 
 static void remove_queue(erase_busy_t *entry)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "Unqueueing 0x%p\n", entry);
-#endif
+    DEBUG(2, "cs: unqueueing 0x%p\n", entry);
     entry->next->prev = entry->prev;
     entry->prev->next = entry->next;
 }
@@ -120,10 +114,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
     socket_info_t *s;
     int ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "Trying erase request 0x%p...\n", busy);
-#endif
+    DEBUG(2, "cs: trying erase request 0x%p...\n", busy);
     if (busy->next)
 	remove_queue(busy);
     req.Function = MTD_REQ_ERASE | cause;
@@ -135,10 +126,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
     mtd->event_callback_args.mtdrequest = &req;
     ret = EVENT(mtd, CS_EVENT_MTD_REQUEST, CS_EVENT_PRI_LOW);
     if (ret == CS_BUSY) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug > 1)
-	    printk(KERN_DEBUG "  Status = %ld, requeueing.\n", req.Status);
-#endif
+	DEBUG(2, "  Status = %ld, requeueing.\n", req.Status);
 	switch (req.Status) {
 	case MTD_WAITREQ:
 	case MTD_WAITPOWER:
@@ -155,10 +143,7 @@ static void retry_erase(erase_busy_t *busy, u_long cause)
     }
     else {
 	/* update erase queue status */
-#ifdef PCMCIA_DEBUG
-	if (pc_debug > 1)
-	    printk(KERN_DEBUG "  Ret = %d\n", ret);
-#endif
+	DEBUG(2, "  Ret = %d\n", ret);
 	switch (ret) {
 	case CS_SUCCESS:
 	    erase->State = ERASE_PASSED; break;
@@ -186,10 +171,7 @@ void retry_erase_list(erase_busy_t *list, u_long cause)
 {
     erase_busy_t tmp = *list;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "Rescanning erase queue list 0x%p\n", list);
-#endif
+    DEBUG(2, "cs: rescanning erase queue list 0x%p\n", list);
     if (list->next == list)
 	return;
     /* First, truncate the original list */
@@ -206,10 +188,7 @@ void retry_erase_list(erase_busy_t *list, u_long cause)
 
 static void handle_erase_timeout(u_long arg)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "Erase timeout for entry 0x%lx\n", arg);
-#endif
+    DEBUG(0, "cs: erase timeout for entry 0x%lx\n", arg);
     retry_erase((erase_busy_t *)arg, MTD_REQ_TIMEOUT);
 }
 
@@ -327,11 +306,9 @@ static void setup_regions(client_handle_t handle, int attr,
     cistpl_device_geo_t geo;
     memory_handle_t r;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: setup_regions(0x%p, %d, 0x%p)\n",
-	       handle, attr, list);
-#endif
+    DEBUG(1, "cs: setup_regions(0x%p, %d, 0x%p)\n",
+	  handle, attr, list);
+
     code = (attr) ? CISTPL_DEVICE_A : CISTPL_DEVICE;
     if (read_tuple(handle, code, &device) != CS_SUCCESS)
 	return;
@@ -452,11 +429,8 @@ int register_mtd(client_handle_t handle, mtd_reg_t *reg)
 	list = s->a_region;
     else
 	list = s->c_region;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: register_mtd(0x%p, '%s', 0x%lx)\n",
-	       handle, (char *)handle->dev_info, reg->Offset);
-#endif
+    DEBUG(1, "cs: register_mtd(0x%p, '%s', 0x%lx)\n",
+	  handle, (char *)handle->dev_info, reg->Offset);
     while (list) {
 	if (list->info.CardOffset == reg->Offset) break;
 	list = list->info.next;
@@ -544,11 +518,8 @@ int open_memory(client_handle_t *handle, open_mem_t *open)
     }
     if (region && region->mtd) {
 	*handle = (client_handle_t)region;
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "cs: open_memory(0x%p, 0x%lx) = 0x%p\n",
-		   handle, open->Offset, region);
-#endif
+	DEBUG(1, "cs: open_memory(0x%p, 0x%lx) = 0x%p\n",
+	      handle, open->Offset, region);
 	return CS_SUCCESS;
     }
     else
@@ -565,10 +536,7 @@ int open_memory(client_handle_t *handle, open_mem_t *open)
 
 int close_memory(memory_handle_t handle)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: close_memory(0x%p)\n", handle);
-#endif
+    DEBUG(1, "cs: close_memory(0x%p)\n", handle);
     if (CHECK_REGION(handle))
 	return CS_BAD_HANDLE;
     return CS_SUCCESS;
