@@ -47,7 +47,8 @@ static const u_long exponent[] = {
     (mantissa[((v)>>3)&15] * exponent[(v)&7] / 10)
 #define POWER_SCALE(v) (exponent[(v)&7])
 
-static int strict_cis = 0;
+/* Upper limit on reasonable # of tuples */
+#define MAX_TUPLES 200
 
 /*======================================================================
 
@@ -63,22 +64,15 @@ void read_cis_mem(socket_info_t *s, int attr,
     u_char *sys = (u_char *)(mem->sys_start + (addr & 0x0fff));
     int inc = 1;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 3)
-	printk(KERN_DEBUG "cs: read_cis_mem(%d, %#lx, %ld)\n",
-	       attr, addr, len);
-#endif
+    DEBUG(3, "cs: read_cis_mem(%d, %#lx, %ld)\n", attr, addr, len);
     mem->card_start = addr & ~0x0fff;
     mem->flags &= ~MAP_ATTRIB;
     if (attr) { mem->flags |= MAP_ATTRIB; inc++; }
     
     for (; len > 0; mem->card_start += 0x1000, sys -= 0x1000) {
 	s->ss_entry(s->sock, SS_SetMemMap, mem);
-#ifdef PCMCIA_DEBUG
-	if (pc_debug > 3)
-	    printk(KERN_DEBUG "cs:  %#2.2x %#2.2x %#2.2x %#2.2x %#2.2x ...\n",
-		   sys[0], sys[inc], sys[2*inc], sys[3*inc], sys[4*inc]);
-#endif
+	DEBUG(3, "cs:  %#2.2x %#2.2x %#2.2x %#2.2x %#2.2x ...\n",
+	      sys[0], sys[inc], sys[2*inc], sys[3*inc], sys[4*inc]);
 	for ( ; len > 0; len--, ptr++, sys += inc) {
 	    if (sys > (u_char *)mem->sys_stop) break;
 	    *ptr = readb(sys);
@@ -108,7 +102,45 @@ void write_cis_mem(socket_info_t *s, int attr,
 
 int setup_cis_mem(socket_info_t *s)
 {
+    /* Validation function for cards with a valid CIS */
+    int cis_readable(u_long base)
+    {
+	cisinfo_t info;
+	int ret;
+	s->cis_mem.sys_start = base;
+	s->cis_mem.sys_stop = base+0x0fff;
+	ret = validate_cis(s->clients, &info);
+	s->cis_used = 0; /* invalidate CIS cache */
+	return ((ret == 0) && (info.Chains > 0));
+    }
+
+    /* Validation function for simple memory cards */
+    int checksum(u_long base)
+    {
+	int i, a;
+	s->cis_mem.sys_start = base;
+	s->cis_mem.sys_stop = base+0xfff;
+	s->cis_mem.card_start = 0;
+	s->cis_mem.flags = MAP_ACTIVE;
+	s->ss_entry(s->sock, SS_SetMemMap, &s->cis_mem);
+	/* Don't bother checking every word... */
+	for (i = a = 0; i < 0x1000; i += 53)
+	    a += (int)readl(base+i);
+	return a;
+    }
+
+    int checksum_match(u_long base)
+    {
+	return (checksum(base) == checksum(base+0x1000));
+    }
+    
+    static int first = 1;
     if (s->cis_mem.sys_start == 0) {
+	if (first == 1) {
+	    first = 0;
+	    validate_mem(cis_readable, checksum_match);
+	    s->cis_mem.sys_start = 0;
+	}
 	if (find_mem_region(&s->cis_mem.sys_start,
 			    0x1000, "card services") != 0)
 	    return CS_OUT_OF_RESOURCE;
@@ -201,7 +233,11 @@ int get_first_tuple(client_handle_t handle, tuple_t *tuple)
 	!(tuple->Attributes & TUPLE_RETURN_COMMON)) {
 	cisdata_t req = tuple->DesiredTuple;
 	tuple->DesiredTuple = CISTPL_LONGLINK_MFC;
-	if (get_next_tuple(handle, tuple) != CS_SUCCESS)
+	if (get_next_tuple(handle, tuple) == CS_SUCCESS) {
+	    tuple->DesiredTuple = CISTPL_LINKTARGET;
+	    if (get_next_tuple(handle, tuple) != CS_SUCCESS)
+		return CS_NO_MORE_ITEMS;
+	} else
 	    tuple->CISOffset = -1;
 	tuple->DesiredTuple = req;
     }
@@ -289,7 +325,7 @@ int get_next_tuple(client_handle_t handle, tuple_t *tuple)
     }
     attr = tuple->Flags & IS_ATTR;
     
-    for (i = 0; i < 1000; i++) {
+    for (i = 0; i < MAX_TUPLES; i++) {
 	read_cis_cache(s, attr, ofs, 2, link);
 
 	if (link[0] == CISTPL_NULL) {
@@ -355,9 +391,11 @@ int get_next_tuple(client_handle_t handle, tuple_t *tuple)
 	    break;
 	ofs += (attr ? 2 : 1) * (link[1]+2);
     }
-    if (i == 1000) {
-	printk(KERN_NOTICE "cs: overrun in get_next_tuple for "
+    if (i == MAX_TUPLES) {
+#ifdef PCMCIA_DEBUG
+	printk(KERN_DEBUG "cs: overrun in get_next_tuple for "
 	       "socket %d\n", handle->Socket);
+#endif
 	return CS_NO_MORE_ITEMS;
     }
     
@@ -396,40 +434,6 @@ int get_tuple_data(client_handle_t handle, tuple_t *tuple)
     return CS_SUCCESS;
 } /* get_tuple_data */
 
-/*====================================================================*/
-
-int validate_cis(client_handle_t handle, cisinfo_t *info)
-{
-    tuple_t tuple;
-    int ret;
-    
-    if (CHECK_HANDLE(handle))
-	return CS_BAD_HANDLE;
-    
-    info->Chains = 0;
-    tuple.DesiredTuple = RETURN_FIRST_TUPLE;
-    tuple.Attributes = 0;
-    ret = get_first_tuple(handle, &tuple);
-    if (ret != CS_SUCCESS)
-	return CS_SUCCESS;
-
-    /* First tuple should be CISTPL_DEVICE */
-    if (strict_cis && (tuple.TupleCode != CISTPL_DEVICE))
-	return CS_SUCCESS;
-
-    for (info->Chains = 1; info->Chains < 500; info->Chains++) {
-	ret = get_next_tuple(handle, &tuple);
-	if (ret != CS_SUCCESS) break;
-    }
-    if (info->Chains == 500) {
-	printk(KERN_NOTICE "cs: too many tuples for socket %d\n",
-	       handle->Socket);
-	info->Chains = 0;
-    }
-    
-    return CS_SUCCESS;
-} /* validate_cis */
-
 /*======================================================================
 
     Parsing routines for individual tuples
@@ -1111,3 +1115,57 @@ int read_tuple(client_handle_t handle, cisdata_t code, void *parse)
     ret = CardServices(ParseTuple, handle, &tuple, parse);
     return ret;
 }
+
+/*======================================================================
+
+    This tries to determine if a card has a sensible CIS.  It returns
+    the number of tuples in the CIS, or 0 if the CIS looks bad.  The
+    checks include making sure several critical tuples are present and
+    valid; seeing if the total number of tuples is reasonable; and
+    looking for tuples that use reserved codes.
+    
+======================================================================*/
+
+int validate_cis(client_handle_t handle, cisinfo_t *info)
+{
+    tuple_t tuple;
+    cisparse_t parse;
+    int ret, reserved, errors;
+    
+    if (CHECK_HANDLE(handle))
+	return CS_BAD_HANDLE;
+    
+    info->Chains = reserved = errors = 0;
+    tuple.DesiredTuple = RETURN_FIRST_TUPLE;
+    tuple.Attributes = 0;
+    ret = get_first_tuple(handle, &tuple);
+    if (ret != CS_SUCCESS)
+	return CS_SUCCESS;
+
+    /* First tuple should be DEVICE */
+    if (tuple.TupleCode != CISTPL_DEVICE)
+	errors++;
+    /* All cards should have a MANFID tuple */
+    if (read_tuple(handle, CISTPL_MANFID, &parse) != CS_SUCCESS)
+	errors++;
+    /* All cards should have either a VERS_1 or a VERS_2 tuple */
+    if ((read_tuple(handle, CISTPL_VERS_1, &parse) != CS_SUCCESS) &&
+	(read_tuple(handle, CISTPL_VERS_2, &parse) != CS_SUCCESS))
+	errors++;
+    if (errors > 1)
+	return CS_SUCCESS;
+    
+    for (info->Chains = 1; info->Chains < MAX_TUPLES; info->Chains++) {
+	ret = get_next_tuple(handle, &tuple);
+	if (ret != CS_SUCCESS) break;
+	if (((tuple.TupleCode > 0x23) && (tuple.TupleCode < 0x40)) ||
+	    ((tuple.TupleCode > 0x47) && (tuple.TupleCode < 0x80)) ||
+	    ((tuple.TupleCode > 0x90) && (tuple.TupleCode < 0xff)))
+	    reserved++;
+    }
+    if ((info->Chains == MAX_TUPLES) || (reserved > 5))
+	info->Chains = 0;
+    
+    return CS_SUCCESS;
+} /* validate_cis */
+
