@@ -41,7 +41,7 @@ static int handle_apm_event(apm_event_t event);
 int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 static const char *version =
-"cs.c 1.145 1997/03/16 20:03:53 (David Hinds)";
+"cs.c 1.148 1997/06/01 05:03:16 (David Hinds)";
 #endif
 
 static const char *release =
@@ -60,6 +60,8 @@ static struct symbol_table cs_symtab = {
     X(unregister_ss_entry),
     X(CardServices),
     X(MTDHelperEntry),
+    X(find_mem_region),
+    X(release_mem_region),
 #include <linux/symtab_end.h>
 };
 
@@ -70,6 +72,8 @@ EXPORT_SYMBOL(register_ss_entry);
 EXPORT_SYMBOL(unregister_ss_entry);
 EXPORT_SYMBOL(CardServices);
 EXPORT_SYMBOL(MTDHelperEntry);
+EXPORT_SYMBOL(find_mem_region);
+EXPORT_SYMBOL(release_mem_region);
 
 #endif
 
@@ -265,11 +269,7 @@ int register_ss_entry(int nsock, ss_entry_t ss_entry)
     int i, ns;
     socket_info_t *s;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: register_ss_entry(%d, 0x%p)\n",
-	       nsock, ss_entry);
-#endif
+    DEBUG(0, "cs: register_ss_entry(%d, 0x%p)\n", nsock, ss_entry);
 
     MOD_INC_USE_COUNT;
 
@@ -353,10 +353,7 @@ static void shutdown_socket(u_long i)
     socket_info_t *s = socket_table[i];
     
     sti();
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: shutdown_socket(%ld)\n", i);
-#endif
+    DEBUG(1, "cs: shutdown_socket(%ld)\n", i);
 
     /* Blank out the socket state */
     s->state &= SOCKET_PRESENT|SOCKET_SETUP_PENDING;
@@ -381,10 +378,7 @@ static void setup_socket(u_long i)
     sti();
     s->ss_entry(s->sock, SS_GetStatus, &val);
     if (val & SS_DETECT) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "cs: setup_socket(%ld): applying power\n", i);
-#endif
+	DEBUG(1, "cs: setup_socket(%ld): applying power\n", i);
 	s->state |= SOCKET_PRESENT;
 	s->socket.flags = 0;
 	s->socket.Vcc = 50;
@@ -395,8 +389,7 @@ static void setup_socket(u_long i)
     }
 #ifdef PCMCIA_DEBUG
     else
-	if (pc_debug)
-	    printk(KERN_DEBUG "cs: setup_socket(%ld): no card!\n", i);
+	DEBUG(0, "cs: setup_socket(%ld): no card!\n", i);
 #endif
 } /* setup_socket */
 
@@ -413,10 +406,7 @@ static void reset_socket(u_long i)
 {
     socket_info_t *s = socket_table[i];
 
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "cs: resetting socket %ld\n", i);
-#endif
+    DEBUG(1, "cs: resetting socket %ld\n", i);
     s->socket.flags |= SS_OUTPUT_ENA | SS_RESET;
     s->ss_entry(s->sock, SS_SetSocket, &s->socket);
     udelay((long)reset_time);
@@ -439,10 +429,7 @@ static void unreset_socket(u_long i)
     sti();
     s->ss_entry(s->sock, SS_GetStatus, &val);
     if (val & SS_READY) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "cs: reset done on socket %ld\n", i);
-#endif
+	DEBUG(1, "cs: reset done on socket %ld\n", i);
 	if (s->state & SOCKET_SUSPEND) {
 	    s->state &= ~EVENT_MASK;
 	    if (verify_cis_cache(s) != 0)
@@ -461,10 +448,7 @@ static void unreset_socket(u_long i)
 	}
     }
     else {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug > 1)
-	    printk(KERN_DEBUG "cs: socket %ld not ready yet\n", i);
-#endif
+	DEBUG(2, "cs: socket %ld not ready yet\n", i);
 	if (s->unreset_timeout > unreset_limit) {
 	    printk(KERN_NOTICE "cs: socket %ld timed out during"
 		   " reset\n", i);
@@ -491,11 +475,8 @@ static int send_event(socket_info_t *s, event_t event, int priority)
 {
     client_t *client = s->clients;
     int ret;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "cs: send_event(sock %d, event %d, pri %d)\n",
-	       s->sock, event, priority);
-#endif
+    DEBUG(1, "cs: send_event(sock %d, event %d, pri %d)\n",
+	  s->sock, event, priority);
     ret = 0;
     for (; client; client = client->next) { 
 	if (client->state & (CLIENT_UNBOUND|CLIENT_STALE))
@@ -543,10 +524,7 @@ static void parse_events(void *info, u_long events)
 	    if (!(s->state & SOCKET_SETUP_PENDING))
 		s->state |= SOCKET_SETUP_PENDING;
 	    else {
-#ifdef PCMCIA_DEBUG
-		if (pc_debug)
-		    printk(KERN_DEBUG "cs: delaying pending setup\n");
-#endif
+		DEBUG(1, "cs: delaying pending setup\n");
 		del_timer(&s->setup);
 	    }
 	    s->setup.function = &setup_socket;
@@ -566,8 +544,7 @@ static void parse_events(void *info, u_long events)
 	if (!(s->state & SOCKET_RESET_PENDING))
 	    send_event(s, CS_EVENT_READY_CHANGE, CS_EVENT_PRI_LOW);
 #ifdef PCMCIA_DEBUG
-	else if (pc_debug)
-	    printk(KERN_DEBUG "cs: ready change during reset\n");
+	else DEBUG(1, "cs: ready change during reset\n");
 #endif
     }
 } /* parse_events */
@@ -591,10 +568,7 @@ static int handle_apm_event(apm_event_t event)
     switch (event) {
     case APM_SYS_SUSPEND:
     case APM_USER_SUSPEND:
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "cs: received suspend notification\n");
-#endif
+	DEBUG(1, "cs: received suspend notification\n");
 	if (down) {
 	    printk(KERN_DEBUG "cs: received extra suspend event\n");
 	    break;
@@ -612,10 +586,7 @@ static int handle_apm_event(apm_event_t event)
 	break;
     case APM_NORMAL_RESUME:
     case APM_CRITICAL_RESUME:
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "cs: received resume notification\n");
-#endif
+	DEBUG(1, "cs: received resume notification\n");
 	if (!down) {
 	    printk(KERN_DEBUG "cs: received bogus resume event\n");
 	    break;
@@ -793,12 +764,8 @@ static int bind_device(bind_req_t *req)
     init_waitqueue(&client->mtd_req);
     client->next = s->clients;
     s->clients = client;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: bind_device(): client 0x%p, sock %d, "
-	       "dev %s\n", client, client->Socket,
-	       (char *)client->dev_info);
-#endif
+    DEBUG(1, "cs: bind_device(): client 0x%p, sock %d, dev %s\n"
+	  client, client->Socket, (char *)client->dev_info);
     return CS_SUCCESS;
 } /* bind_device */
 
@@ -833,12 +800,8 @@ static int bind_mtd(mtd_bind_t *req)
 	return CS_BAD_OFFSET;
     region->dev_info = req->dev_info;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: bind_mtd(): attr 0x%lx, offset 0x%lx, "
-	       "dev %s\n", req->Attributes, req->CardOffset,
-	       (char *)req->dev_info);
-#endif
+    DEBUG(1, "cs: bind_mtd(): attr 0x%lx, offset 0x%lx, dev %s\n",
+	  req->Attributes, req->CardOffset, (char *)req->dev_info);
     return CS_SUCCESS;
 } /* bind_mtd */
 
@@ -852,10 +815,7 @@ static int deregister_client(client_handle_t handle)
     u_long flags;
     int i, sn;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: deregister_client(%p)\n", handle);
-#endif
+    DEBUG(1, "cs: deregister_client(%p)\n", handle);
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
     if (handle->state &
@@ -910,21 +870,18 @@ static int get_configuration_info(client_handle_t handle,
 	return CS_NO_CARD;
 
     if (handle->Function == BIND_FN_ALL) {
-	if ((s->functions > 0) && (config->Function >= s->functions))
+	if (config->Function && (config->Function >= s->functions))
 	    return CS_BAD_ARGS;
-	c = &s->config[config->Function];
-    }
-    else
+	c = (s->config != NULL) ? &s->config[config->Function] : NULL;
+    } else
 	c = CONFIG(handle);
     
-    if ((s->functions == 0) || !(c->state & CONFIG_LOCKED)) {
+    if ((c == NULL) || !(c->state & CONFIG_LOCKED)) {
 	config->Attributes = 0;
 	config->Vcc = s->socket.Vcc;
 	config->Vpp1 = config->Vpp2 = s->socket.Vpp;
-    }
-    
-    /* !!! This is a hack !!! */
-    if (c->state & CONFIG_LOCKED) {
+    } else {
+	/* !!! This is a hack !!! */
 	memcpy(&config->Attributes, &c->Attributes, 33);
 	config->Attributes |= CONF_VALID_CLIENT;
 	config->CardValues = c->CardValues;
@@ -1028,7 +985,7 @@ static int get_status(client_handle_t handle, status_t *status)
 
     /* Get info from the PRR, if necessary */
     if (handle->Function == BIND_FN_ALL) {
-	if ((s->functions > 0) && (status->Function > s->functions))
+	if (status->Function && (status->Function >= s->functions))
 	    return CS_BAD_ARGS;
 	c = (s->config != NULL) ? &s->config[status->Function] : NULL;
     } else
@@ -1216,12 +1173,8 @@ static int register_client(client_handle_t *handle, client_reg_t *req)
 	}
     }
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: register_client(): client 0x%p, sock "
-	       "%d, dev %s\n", client, client->Socket,
-	       (char *)client->dev_info);
-#endif
+    DEBUG(1, "cs: register_client(): client 0x%p, sock %d, dev %s\n",
+	  client, client->Socket, (char *)client->dev_info);
     if (client->EventMask & CS_EVENT_REGISTRATION_COMPLETE)
 	EVENT(client, CS_EVENT_REGISTRATION_COMPLETE, CS_EVENT_PRI_LOW);
     if ((socket_table[ns]->state & SOCKET_PRESENT) &&
@@ -1711,10 +1664,7 @@ static int reset_card(client_handle_t handle, client_req_t *req)
 	EVENT(handle, CS_EVENT_RESET_COMPLETE, CS_EVENT_PRI_LOW);
     }
     else {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "cs: resetting socket %d\n", i);
-#endif
+	DEBUG(1, "cs: resetting socket %d\n", i);
 	send_event(s, CS_EVENT_RESET_PHYSICAL, CS_EVENT_PRI_LOW);
 	s->reset_handle = handle;
 	reset_socket(i);
@@ -1742,10 +1692,7 @@ static int suspend_card(client_handle_t handle, client_req_t *req)
     if (s->state & SOCKET_SUSPEND)
 	return CS_IN_USE;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: suspending socket %d\n", i);
-#endif
+    DEBUG(1, "cs: suspending socket %d\n", i);
     send_event(s, CS_EVENT_PM_SUSPEND, CS_EVENT_PRI_LOW);
     s->ss_entry(s->sock, SS_SetSocket, &dead_socket);
     s->state |= SOCKET_SUSPEND;
@@ -1766,10 +1713,7 @@ static int resume_card(client_handle_t handle, client_req_t *req)
     if (!(s->state & SOCKET_SUSPEND))
 	return CS_IN_USE;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: waking up socket %d\n", i);
-#endif
+    DEBUG(1, "cs: waking up socket %d\n", i);
     setup_socket(i);
 
     return CS_SUCCESS;
@@ -1793,10 +1737,7 @@ static int eject_card(client_handle_t handle, client_req_t *req)
     if (!(s->state & SOCKET_PRESENT))
 	return CS_NO_CARD;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: user eject request on socket %d\n", i);
-#endif
+    DEBUG(1, "cs: user eject request on socket %d\n", i);
 
     ret = send_event(s, CS_EVENT_EJECTION_REQUEST, CS_EVENT_PRI_LOW);
     if (ret != 0)
@@ -1823,10 +1764,7 @@ static int insert_card(client_handle_t handle, client_req_t *req)
     if (s->state & SOCKET_PRESENT)
 	return CS_IN_USE;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "cs: user insert request on socket %d\n", i);
-#endif
+    DEBUG(1, "cs: user insert request on socket %d\n", i);
 
     save_flags(flags);
     cli();
@@ -2021,10 +1959,7 @@ int CardServices(int func, void *a1, void *a2, void *a3)
 int init_module(void)
 {
     printk(KERN_INFO "%s\n", release);
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
 #ifdef CONFIG_APM
     apm_register_callback(&handle_apm_event);
 #endif
