@@ -1,5 +1,5 @@
 /*
- *  cs_internal.h 1.10 1996/11/26 07:20:07 (David Hinds)
+ *  cs_internal.h 1.12 1997/06/01 05:01:55 (David Hinds)
  */
 
 #ifndef _LINUX_CS_INTERNAL_H
@@ -201,6 +201,9 @@ extern socket_info_t *socket_table[MAX_SOCK];
 
 #ifdef PCMCIA_DEBUG
 extern int pc_debug;
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
+#else
+#define DEBUG(n, args...)
 #endif
 
 #endif /* _LINUX_CS_INTERNAL_H */
