@@ -36,8 +36,11 @@
 #ifdef PCMCIA_DEBUG
 int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static const char *version =
-"ds.c 1.67 1997/03/16 20:04:07 (David Hinds)";
+"ds.c 1.70 1997/06/01 05:03:16 (David Hinds)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -56,10 +59,10 @@ static int ds_select(struct inode *inode, struct file *file,
 #else
 static unsigned int ds_poll(struct file *file, poll_table *poll);
 #endif
-static int ds_ioctl(struct inode * inode, struct file * file,
+static int ds_ioctl(struct inode *inode, struct file *file,
 		    u_int cmd, u_long arg);
-static int ds_open(struct inode * inode, struct file * file);
-static void ds_release(struct inode * inode, struct file * file);
+static int ds_open(struct inode *inode, struct file *file);
+static FS_RELEASE_T ds_release(struct inode *inode, struct file *file);
 
 typedef struct driver_info_t {
     dev_info_t *dev_info;
@@ -176,11 +179,7 @@ int register_pcmcia_driver(dev_info_t *dev_info,
 			   void (*detach)(dev_link_t *))
 {
     driver_info_t *driver;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ds: register_pcmcia_driver('%s')\n",
-	       (char *)dev_info);
-#endif
+    DEBUG(0, "ds: register_pcmcia_driver('%s')\n", (char *)dev_info);
     for (driver = root_driver; driver; driver = driver->next)
 	if (strcmp((char *)dev_info, (char *)driver->dev_info) == 0)
 	    break;
@@ -202,11 +201,8 @@ int register_pcmcia_driver(dev_info_t *dev_info,
 int unregister_pcmcia_driver(dev_info_t *dev_info)
 {
     driver_info_t *target, **driver = &root_driver;
-#ifdef PCMIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ds: unregister_pcmcia_driver('%s')\n",
-	       (char *)dev_info);
-#endif
+    DEBUG(0, "ds: unregister_pcmcia_driver('%s')\n",
+	  (char *)dev_info);
     while ((*driver) && ((*driver)->dev_info != dev_info))
 	driver = &(*driver)->next;
     if (*driver == NULL)
@@ -294,11 +290,8 @@ static int ds_event(event_t event, int priority,
     socket_info_t *s;
     int i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ds: ds_event(%d, %d, 0x%p)\n",
-	       event, priority, args->client_handle);
-#endif
+    DEBUG(1, "ds: ds_event(0x%06x, %d, 0x%p)\n",
+	  event, priority, args->client_handle);
     s = args->client_data;
     i = s - socket_table;
     
@@ -519,10 +512,7 @@ static int ds_open(struct inode *inode, struct file *file)
     socket_info_t *s;
     user_info_t *user;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ds_open(socket %d)\n", i);
-#endif
+    DEBUG(0, "ds_open(socket %d)\n", i);
     if ((i >= sockets) || (sockets == 0))
 	return -ENODEV;
     s = &socket_table[i];
@@ -548,22 +538,19 @@ static int ds_open(struct inode *inode, struct file *file)
 
 /*====================================================================*/
 
-static void ds_release(struct inode *inode, struct file *file)
+static FS_RELEASE_T ds_release(struct inode *inode, struct file *file)
 {
     socket_t i = MINOR(inode->i_rdev);
     socket_info_t *s;
     user_info_t *user, **link;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ds_release(socket %d)\n", i);
-#endif
+    DEBUG(0, "ds_release(socket %d)\n", i);
     if ((i >= sockets) || (sockets == 0))
-	return;
+	return (FS_RELEASE_T)0;
     s = &socket_table[i];
     user = file->private_data;
     if (CHECK_USER(user))
-	return;
+	return (FS_RELEASE_T)0;
 
     /* Unlink user data structure */
     if ((file->f_flags & O_ACCMODE) != O_RDONLY)
@@ -572,12 +559,13 @@ static void ds_release(struct inode *inode, struct file *file)
     for (link = &s->user; *link; link = &(*link)->next)
 	if (*link == user) break;
     if (link == NULL)
-	return;
+	return (FS_RELEASE_T)0;
     *link = user->next;
     user->user_magic = 0;
     kfree_s(user, sizeof(user_info_t));
     
     MOD_DEC_USE_COUNT;
+    return (FS_RELEASE_T)0;
 } /* ds_release */
 
 /*====================================================================*/
@@ -589,10 +577,7 @@ static FS_SIZE_T ds_read(struct inode *inode, struct file *file,
     socket_info_t *s;
     user_info_t *user;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ds_read(socket %d)\n", i);
-#endif
+    DEBUG(2, "ds_read(socket %d)\n", i);
     
     if ((i >= sockets) || (sockets == 0))
 	return -ENODEV;
@@ -621,10 +606,7 @@ static FS_SIZE_T ds_write(struct inode *inode, struct file *file,
     socket_info_t *s;
     user_info_t *user;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ds_write(socket %d)\n", i);
-#endif
+    DEBUG(2, "ds_write(socket %d)\n", i);
     
     if ((i >= sockets) || (sockets == 0))
 	return -ENODEV;
@@ -660,10 +642,7 @@ static int ds_select(struct inode *inode, struct file *file,
     socket_info_t *s;
     user_info_t *user;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "ds_select(socket %d)\n", i);
-#endif
+    DEBUG(2, "ds_select(socket %d)\n", i);
     
     if ((i >= sockets) || (sockets == 0))
 	return -ENODEV;
@@ -687,10 +666,7 @@ static unsigned int ds_poll(struct file *file, poll_table *wait)
     socket_info_t *s;
     user_info_t *user;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "ds_poll(socket %d)\n", i);
-#endif
+    DEBUG(2, "ds_poll(socket %d)\n", i);
     
     if ((i >= sockets) || (sockets == 0))
 	return POLLERR;
@@ -717,11 +693,7 @@ static int ds_ioctl(struct inode * inode, struct file * file,
     int ret, err;
     ds_ioctl_arg_t buf;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ds_ioctl(socket %d, %#x, %#lx)\n",
-	       i, cmd, arg);
-#endif
+    DEBUG(2, "ds_ioctl(socket %d, %#x, %#lx)\n", i, cmd, arg);
     
     if ((i >= sockets) || (sockets == 0))
 	return -ENODEV;
@@ -737,20 +709,14 @@ static int ds_ioctl(struct inode * inode, struct file * file,
     if (cmd & IOC_IN) {
 	err = verify_area(VERIFY_READ, (char *)arg, size);
 	if (err) {
-#ifdef PCMCIA_DEBUG
-	    if (pc_debug)
-		printk(KERN_DEBUG "ds_ioctl(): verify_read = %d\n", err);
-#endif
+	    DEBUG(3, "ds_ioctl(): verify_read = %d\n", err);
 	    return err;
 	}
     }
     if (cmd & IOC_OUT) {
 	err = verify_area(VERIFY_WRITE, (char *)arg, size);
 	if (err) {
-#ifdef PCMCIA_DEBUG
-	    if (pc_debug)
-		printk(KERN_DEBUG "ds_ioctl(): verify_write = %d\n", err);
-#endif
+	    DEBUG(3, "ds_ioctl(): verify_write = %d\n", err);
 	    return err;
 	}
     }
@@ -840,10 +806,7 @@ static int ds_ioctl(struct inode * inode, struct file * file,
     }
     
     if ((err == 0) && (ret != CS_SUCCESS)) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "ds_ioctl: ret = %d\n", ret);
-#endif
+	DEBUG(2, "ds_ioctl: ret = %d\n", ret);
 	switch (ret) {
 	case CS_BAD_SOCKET: case CS_NO_CARD:
 	    err = -ENODEV; break;
@@ -878,10 +841,7 @@ int init_module(void)
     socket_info_t *s;
     int i, ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
