@@ -40,6 +40,7 @@
 #include <pcmcia/cs_types.h>
 #include <pcmcia/cs.h>
 #include <pcmcia/cistpl.h>
+#include <pcmcia/ciscode.h>
 #include <pcmcia/ds.h>
 
 /*
@@ -52,6 +53,9 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*
@@ -120,7 +124,7 @@ static dev_link_t *dev_list = NULL;
 /*
     card type
  */
-typedef enum { MBH10302, MBH10304, TDK, CONTEC } cardtype_t;
+typedef enum { MBH10302, MBH10304, TDK, CONTEC, LA501 } cardtype_t;
 
 /*
     driver specific data structure
@@ -245,10 +249,7 @@ static dev_link_t *fmvj18x_attach(void)
     struct device *dev;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "fmvj18x_attach()\n");
-#endif
+    DEBUG(0, "fmvj18x_attach()\n");
 
     /* Initialize the dev_link_t structure */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -320,10 +321,7 @@ static void fmvj18x_detach(dev_link_t *link)
 {
     dev_link_t **linkp;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "fmvj18x_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "fmvj18x_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -364,7 +362,7 @@ static void fmvj18x_config(dev_link_t *link)
     tuple_t tuple;
     cisparse_t parse;
     struct device *dev;
-    u_char buf[64];
+    u_short buf[32];
     int i;
     short ioaddr;
     int ret;
@@ -374,10 +372,7 @@ static void fmvj18x_config(dev_link_t *link)
     handle = link->handle;
     dev =link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "fmvj18x_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "fmvj18x_config(0x%p)\n", link);
 
     /*
        This reads the card's CONFIG tuple to find its configuration
@@ -387,7 +382,7 @@ static void fmvj18x_config(dev_link_t *link)
 	tuple.DesiredTuple = CISTPL_CONFIG;
 	ret = CardServices(GetFirstTuple, handle, &tuple);
 	if (ret != CS_SUCCESS) break;
-	tuple.TupleData = buf;
+	tuple.TupleData = (u_char *)buf;
 	tuple.TupleDataMax = 64;
 	tuple.TupleOffset = 0;
 	ret = CardServices(GetTupleData, handle, &tuple);
@@ -411,26 +406,25 @@ static void fmvj18x_config(dev_link_t *link)
     if (CardServices(GetFirstTuple, handle, &tuple) == CS_SUCCESS) {
 	/* Yes, I have CISTPL_FUNCE. Let's check CISTPL_MANFID */
 	tuple.DesiredTuple = CISTPL_MANFID ;
-	tuple.TupleOffset = 0;
 	if (CardServices(GetFirstTuple, handle, &tuple) != CS_SUCCESS) {
 	    fmvj18x_release((u_long)link);
 	    return;
 	}
-	tuple.TupleOffset = 0;
 	if (CardServices(GetTupleData, handle, &tuple) != CS_SUCCESS) {
 	    fmvj18x_release((u_long)link);
 	    return;
 	}
-	if( tuple.TupleData[0] == 0x05 && tuple.TupleData[1] == 0x01 ) {
-	    /* Manufacture is TDK */
-	    cardtype = TDK;
-	    link->conf.ConfigIndex = 0x04;
-	} else if( tuple.TupleData[0] == 0x01 && tuple.TupleData[1] == 0xc0 ) {
-	    /* Manufacture is CONTEC */
-	    cardtype = CONTEC;
-	    link->conf.ConfigIndex = 1;
-	} else {
-	    /* FMV-J182(MBH10304) have MACID on CIS */
+	switch (buf[0]) {
+	case MANFID_TDK:
+	    cardtype = TDK; link->conf.ConfigIndex = 0x04;
+	    break;
+	case MANFID_CONTEC:
+	    cardtype = CONTEC; link->conf.ConfigIndex = 1;
+	    break;
+	case MANFID_FUJITSU:
+	    cardtype = LA501; link->conf.ConfigIndex = 1;
+	    break;
+	default:
 	    cardtype = MBH10304;
 	    link->conf.ConfigIndex = 0x60;
 	}
@@ -480,6 +474,7 @@ static void fmvj18x_config(dev_link_t *link)
     switch (cardtype) {
     case MBH10304:
     case TDK:
+    case LA501:
     case CONTEC:
 	do {
 	    u_char *node_id;
@@ -505,6 +500,8 @@ static void fmvj18x_config(dev_link_t *link)
 		node_id = &(tuple.TupleData[2]);
 		if( cardtype == TDK ) {
 		    card_name = "TDK LAK-CD021";
+		} else if( cardtype == LA501 ) {
+		    card_name = "LA501";
 		} else {
 		    card_name = "C-NET(PC)C";
 		}
@@ -550,21 +547,15 @@ static void fmvj18x_release(u_long arg)
     dev_link_t *link = (dev_link_t *)arg;
     struct device *dev = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "fmvj18x_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "fmvj18x_release(0x%p)\n", link);
 
     /*
        If the device is currently in use, we won't release until it
        is actually closed.
     */
     if (link->open) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "fmvj18x_cs: release postponed, '%s' "
-		   "still open\n", link->dev->dev_name);
-#endif 
+	DEBUG(1, "fmvj18x_cs: release postponed, '%s' "
+	      "still open\n", link->dev->dev_name);
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
@@ -594,18 +585,9 @@ static int fmvj18x_event(event_t event, int priority,
     dev_link_t *link = args->client_data;
     struct device *dev = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "fmvj18x_event()\n");
-#endif
+    DEBUG(1, "fmvj18x_event(0x%06x)\n", event);
     
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "fmvj18x_cs: registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -658,10 +640,7 @@ static int fmvj18x_init(struct device *dev)
 int init_module(void)
 {
     servinfo_t serv;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
 	printk(KERN_NOTICE "fmvj18x: Card Services release "
@@ -676,10 +655,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "fmvj18x_cs: unloading\n");
-#endif
+    DEBUG(0, "fmvj18x_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL) {
 	if (dev_list->state & DEV_CONFIG)
@@ -817,7 +793,7 @@ static int fjn_start_xmit(struct sk_buff *skb, struct device *dev)
     
     /* Block a timer-based transmit from overlapping.  This could better be
        done with atomic_swap(1, dev->tbusy), but set_bit() works as well. */
-    if (set_bit(0, (void*)&dev->tbusy) != 0)
+    if (test_and_set_bit(0, (void*)&dev->tbusy) != 0)
 	printk(KERN_NOTICE "%s: Transmitter access conflict.\n", dev->name);
     else {
 	short length = ETH_ZLEN < skb->len ? skb->len : ETH_ZLEN;
