@@ -59,8 +59,11 @@ static int major_dev = 0;
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"ftl_cs.c 1.27 1997/03/16 20:08:38 (David Hinds)";
+"ftl_cs.c 1.29 1997/06/01 05:03:16 (David Hinds)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 static int shuffle_freq = 50;
@@ -143,7 +146,7 @@ static int ftl_blocksizes[MINOR_NR(MAX_DEV, 0)] =
 static int ftl_ioctl(struct inode *inode, struct file *file,
 		     u_int cmd, u_long arg);
 static int ftl_open(struct inode *inode, struct file *file);
-static void ftl_close(struct inode *inode, struct file *file);
+static FS_RELEASE_T ftl_close(struct inode *inode, struct file *file);
 
 static struct file_operations ftl_blk_fops = {
     NULL,		/* lseek */
@@ -181,10 +184,7 @@ static dev_link_t *ftl_attach(void)
     eraseq_hdr_t eraseq_hdr;
     int i, ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: ftl_attach()\n");
-#endif
+    DEBUG(0, "ftl_cs: ftl_attach()\n");
 
     for (i = 0; i < MAX_DEV; i++)
 	if (dev_table[i] == NULL) break;
@@ -252,10 +252,7 @@ static void ftl_detach(dev_link_t *link)
     ftl_dev_t *dev;
     int i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: ftl_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "ftl_cs: ftl_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (i = 0; i < MAX_DEV; i++)
@@ -300,10 +297,7 @@ static void ftl_config(dev_link_t *link)
     dev_node_t **tail;
     int i, ret, nr;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: ftl_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "ftl_cs: ftl_config(0x%p)\n", link);
 
     /* Configure card */
     link->state |= DEV_CONFIG;
@@ -358,19 +352,12 @@ static void ftl_release(u_long arg)
     dev_link_t *link = (dev_link_t *)arg;
     int i;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: ftl_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "ftl_cs: ftl_release(0x%p)\n", link);
 
     for (i = 0; i < MAX_DEV; i++)
 	if (dev_table[i] == link) break;
     if (link->open) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "ftl_cs: release postponed, 'ftl%d'"
-		   " still open\n", i);
-#endif
+	DEBUG(1, "ftl_cs: release postponed, ftl%d still open\n", i);
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
@@ -400,18 +387,9 @@ static int ftl_event(event_t event, int priority,
     dev_link_t *link = args->client_data;
     ftl_dev_t *dev;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: ftl_event()\n");
-#endif
+    DEBUG(1, "ftl_cs: ftl_event()\n");
     
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "ftl_cs: registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -595,21 +573,14 @@ static int erase_xfer(ftl_dev_t *dev, partition_t *part,
     struct xfer_info_t *xfer;
 
     xfer = &part->XferInfo[xfernum];
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: erasing xfer unit at 0x%lx\n",
-	       xfer->Offset);
-#endif
+    DEBUG(1, "ftl_cs: erasing xfer unit at 0x%lx\n", xfer->Offset);
     xfer->state = XFER_ERASING;
     /* Is there a free erase slot? */
     for (;;) {
 	for (i = 0; i < MAX_ERASE; i++)
 	    if (!ERASE_IN_PROGRESS(dev->eraseq[i].State)) break;
 	if (i < MAX_ERASE) break;
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "ftl_cs: erase queue is full\n");
-#endif
+	DEBUG(0, "ftl_cs: erase queue is full\n");
 	sleep_on(&dev->erase_pending);
     }
 
@@ -671,11 +642,7 @@ static void prepare_xfer(partition_t *part, int i)
     xfer = &part->XferInfo[i];
     xfer->state = XFER_FAILED;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: preparing xfer unit at 0x%lx\n",
-	       xfer->Offset);
-#endif
+    DEBUG(1, "ftl_cs: preparing xfer unit at 0x%lx\n", xfer->Offset);
 
     /* Write the transfer unit header */
     header = part->header;
@@ -732,11 +699,8 @@ static int copy_erase_unit(partition_t *part, u_short srcunit,
 
     eun = &part->EUNInfo[srcunit];
     xfer = &part->XferInfo[xferunit];
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: copying block 0x%lx to 0x%lx\n",
-	       eun->Offset, xfer->Offset);
-#endif
+    DEBUG(2, "ftl_cs: copying block 0x%lx to 0x%lx\n",
+	  eun->Offset, xfer->Offset);
 	
     req.Attributes = MEM_OP_BUFFER_KERNEL;
     
@@ -846,10 +810,7 @@ static int reclaim_block(ftl_dev_t *dev, partition_t *part)
     u_long best;
     int queued, ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: reclaiming space...\n");
-#endif
+    DEBUG(0, "ftl_cs: reclaiming space...\n");
 	
     /* Pick the least erased transfer unit */
     best = 0xffffffff; xfer = 0xffff;
@@ -870,11 +831,8 @@ static int reclaim_block(ftl_dev_t *dev, partition_t *part)
 	}
 	if (xfer == 0xffff) {
 	    if (queued) {
-#ifdef PCMCIA_DEBUG
-		if (pc_debug)
-		    printk(KERN_DEBUG "ftl_cs: waiting for transfer "
-			   "unit to be prepared...\n");
-#endif
+		DEBUG(1, "ftl_cs: waiting for transfer "
+		      "unit to be prepared...\n");
 		sleep_on(&dev->erase_pending);
 	    }
 	    else {
@@ -889,10 +847,7 @@ static int reclaim_block(ftl_dev_t *dev, partition_t *part)
 
     eun = 0;
     if ((jiffies % shuffle_freq) == 0) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "ftl_cs: recycling freshest block...\n");
-#endif
+	DEBUG(1, "ftl_cs: recycling freshest block...\n");
 	best = 0xffffffff;
 	for (i = 0; i < part->DataUnits; i++)
 	    if (part->EUNInfo[i].EraseCount <= best) {
@@ -993,11 +948,7 @@ static u_long find_free(partition_t *part)
 	printk(KERN_NOTICE "ftl_cs: bad free list!\n");
 	return 0;
     }
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "ftl_cs: found free block at %ld in %d\n",
-	       blk, eun);
-#endif
+    DEBUG(2, "ftl_cs: found free block at %ld in %d\n", blk, eun);
     return blk;
     
 } /* find_free */
@@ -1018,10 +969,7 @@ static int ftl_open(struct inode *inode, struct file *file)
     open_mem_t open;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: ftl_open(%d)\n", minor);
-#endif
+    DEBUG(0, "ftl_cs: ftl_open(%d)\n", minor);
 
     link = dev_table[DEVICE_NR(minor)];
     if (!DEV_OK(link))
@@ -1062,7 +1010,7 @@ static int ftl_open(struct inode *inode, struct file *file)
 
 /*====================================================================*/
 
-static void ftl_close(struct inode *inode, struct file *file)
+static FS_RELEASE_T ftl_close(struct inode *inode, struct file *file)
 {
     dev_link_t *link;
     int minor = MINOR(inode->i_rdev);
@@ -1070,10 +1018,7 @@ static void ftl_close(struct inode *inode, struct file *file)
     partition_t *part;
     int i;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: ftl_close(%d)\n", minor);
-#endif
+    DEBUG(0, "ftl_cs: ftl_close(%d)\n", minor);
 
     /* Flush all writes */
     fsync_dev(inode->i_rdev);
@@ -1122,7 +1067,7 @@ static void ftl_close(struct inode *inode, struct file *file)
     }
     
     MOD_DEC_USE_COUNT;
-
+    return (FS_RELEASE_T)0;
 } /* ftl_close */
 
 /*======================================================================
@@ -1138,11 +1083,8 @@ static int ftl_read(partition_t *part, caddr_t buffer,
     u_long log_addr, bsize, i;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "ftl_cs: ftl_read(0x%p, 0x%lx, %ld)\n",
-	       part->handle, sector, nblocks);
-#endif
+    DEBUG(2, "ftl_cs: ftl_read(0x%p, 0x%lx, %ld)\n",
+	  part->handle, sector, nblocks);
     if (!(part->state & FTL_FORMATTED)) {
 	printk(KERN_NOTICE "ftl_cs: bad partition\n");
 	return -EIO;
@@ -1189,11 +1131,8 @@ static int set_bam_entry(partition_t *part, u_long log_addr,
     u_short eun;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "ftl_cs: set_bam_entry(0x%p, 0x%lx, 0x%lx)\n",
-	       part->handle, log_addr, virt_addr);
-#endif
+    DEBUG(2, "ftl_cs: set_bam_entry(0x%p, 0x%lx, 0x%lx)\n",
+	  part->handle, log_addr, virt_addr);
     bsize = part->region.BlockSize;
     eun = log_addr / bsize;
     blk = (log_addr % bsize) / SECTOR_SIZE;
@@ -1252,11 +1191,8 @@ static int ftl_write(ftl_dev_t *dev, partition_t *part, caddr_t buffer,
     u_long bsize, log_addr, virt_addr, old_addr, blk, i;
     int ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "ftl_cs: ftl_write(0x%p, %ld, %ld)\n",
-	       part->handle, sector, nblocks);
-#endif
+    DEBUG(2, "ftl_cs: ftl_write(0x%p, %ld, %ld)\n",
+	  part->handle, sector, nblocks);
     if (!(part->state & FTL_FORMATTED)) {
 	printk(KERN_NOTICE "ftl_cs: bad partition\n");
 	return -EIO;
@@ -1382,10 +1318,7 @@ static void do_ftl_request(void)
     ftl_dev_t *dev;
     partition_t *part;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: starting do_ftl_request()\n");
-#endif
+    DEBUG(2, "ftl_cs: starting do_ftl_request()\n");
     
     do {
 	sti();
@@ -1433,10 +1366,7 @@ int init_module(void)
     servinfo_t serv;
     int i;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
@@ -1465,10 +1395,7 @@ void cleanup_module(void)
     int i;
     dev_link_t *link;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ftl_cs: unloading\n");
-#endif
+    DEBUG(0, "ftl_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     if (major_dev != 0)
 	unregister_blkdev(major_dev, "ftl");
