@@ -40,8 +40,11 @@ extern struct device *irq2dev_map[16];
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
 "ibmtr_cs.c 1.10 1996/01/06 05:19:00 (Steve Kipisz)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -125,10 +128,7 @@ static dev_link_t *ibmtr_attach(void)
     struct tok_info *ti;
     int ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-        printk(KERN_DEBUG "ibmtr_attach()\n");
-#endif
+    DEBUG(0, "ibmtr_attach()\n");
 
     /* Create new token-ring device */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -151,6 +151,10 @@ static dev_link_t *ibmtr_attach(void)
     memset(info, 0, sizeof(struct ibmtr_dev_t));
     dev = &info->dev;
 
+#if (LINUX_KERNEL_VERSION > VERSION(2,1,16))
+    init_trdev(dev, 0);
+#endif
+    
     ti = kmalloc(sizeof(struct tok_info), GFP_KERNEL);
     memset(ti,0,sizeof(struct tok_info));
     dev->priv = ti;
@@ -196,10 +200,7 @@ static void ibmtr_detach(dev_link_t *link)
     dev_link_t **linkp;
     long flags;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-        printk(KERN_DEBUG "ibmtr_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "ibmtr_detach(0x%p)\n", link);
 
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -268,10 +269,7 @@ static void ibmtr_config(dev_link_t *link)
     dev = &info->dev;
     ti = dev->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-        printk(KERN_DEBUG "ibmtr_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "ibmtr_config(0x%p)\n", link);
 
     tuple.Attributes = 0;
     tuple.DesiredTuple = CISTPL_CONFIG;
@@ -365,9 +363,17 @@ static void ibmtr_config(dev_link_t *link)
 
     dev->tbusy = 0;
 
+#if (LINUX_KERNEL_VERSION <= VERSION(2,1,16))
     i = register_netdev(dev);
+#else
+    i = register_trdev(dev);
+#endif
     if (i != 0) {
+#if (LINUX_KERNEL_VERSION <= VERSION(2,1,16))
         printk(KERN_NOTICE "ibmtr_cs: register_netdev() failed\n");
+#else
+        printk(KERN_NOTICE "ibmtr_cs: register_trdev() failed\n");
+#endif
         goto config_failed;
     }
 
@@ -402,23 +408,21 @@ static void ibmtr_release(u_long arg)
     dev_link_t *link = (dev_link_t *)arg;
     ibmtr_dev_t *info = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-        printk(KERN_DEBUG "ibmtr_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "ibmtr_release(0x%p)\n", link);
 
     if (link->open) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "ibmtr_cs: release postponed, '%s' "
-		   "still open\n", info->node.dev_name);
-#endif
+	DEBUG(1, "ibmtr_cs: release postponed, '%s' "
+	      "still open\n", info->node.dev_name);
         link->state |= DEV_STALE_CONFIG;
         return;
     }
 
     if (link->dev)
+#if (LINUX_KERNEL_VERSION <= VERSION(2,1,16))
         unregister_netdev(&info->dev);
+#else
+        unregister_trdev(&info->dev);
+#endif
     link->dev = NULL;
 
     CardServices(ReleaseConfiguration, link->handle);
@@ -450,18 +454,9 @@ static int ibmtr_event(event_t event, int priority,
     dev_link_t *link = args->client_data;
     ibmtr_dev_t *info = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-        printk(KERN_DEBUG "ibmtr_event()\n");
-#endif
+    DEBUG(1, "ibmtr_event(0x%06x)\n", event);
 
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-        if (pc_debug)
-            printk(KERN_DEBUG "ibmtr_cs: registration complete\n");
-        break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
         link->state &= ~DEV_PRESENT;
         if (link->state & DEV_CONFIG) {
@@ -577,10 +572,7 @@ static void ibmtr_hw_setup(struct device *dev)
 int init_module(void)
 {
     servinfo_t serv;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-        printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
         printk(KERN_NOTICE "ibmtr_cs: Card Services release "
@@ -593,10 +585,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "ibmtr_cs: unloading\n");
-#endif
+    DEBUG(0, "ibmtr_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL)
         ibmtr_detach(dev_list);
