@@ -42,8 +42,11 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"iflash2+_mtd.c 1.30 1997/03/16 20:08:00 (David Hinds)";
+"iflash2+_mtd.c 1.31 1997/06/01 05:03:16 (David Hinds)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -316,10 +319,7 @@ static int suspend_erase(volatile u_short *esr)
 {
     u_short i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: suspending erase...\n");
-#endif
+    DEBUG(1, "iflash2+_mtd: suspending erase...\n");
     writew(IF_ERASE_SUSPEND, esr);
     writew(IF_READ_ESR, esr);
     for (i = 0; i < max_tries; i++)
@@ -335,10 +335,7 @@ static int suspend_erase(volatile u_short *esr)
 
 static void resume_erase(volatile u_short *esr)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: resuming erase...\n");
-#endif
+    DEBUG(1, "iflash2+_mtd: resuming erase...\n");
     writew(IF_READ_ESR, esr);
     /* Only give resume signal if the erase is really suspended */
     if (readw(esr+4) & GSR_OP_SUSPEND)
@@ -394,10 +391,7 @@ static int vpp_setup(dev_link_t *link, mtd_request_t *req)
 	    if (dev->vpp_timeout.expires)
 		del_timer(&dev->vpp_timeout);
 	    else {
-#ifdef PCMCIA_DEBUG
-		if (pc_debug > 1)
-		    printk(KERN_DEBUG "iflash2+_mtd: raising Vpp...\n");
-#endif
+		DEBUG(1, "iflash2+_mtd: raising Vpp...\n");
 		dev->vpp_start = jiffies;
 		vpp_req.Vpp1 = vpp_req.Vpp2 = 120;
 		MTDHelperEntry(MTDSetVpp, link->handle, &vpp_req);
@@ -419,10 +413,7 @@ static void vpp_off(u_long arg)
     flash_dev_t *dev;
     mtd_vpp_req_t req;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: lowering Vpp...\n");
-#endif
+    DEBUG(1, "iflash2+_mtd: lowering Vpp...\n");
     dev = (flash_dev_t *)link->priv;
     dev->vpp_timeout.expires = 0;
     req.Vpp1 = req.Vpp2 = 0;
@@ -455,10 +446,7 @@ static dev_link_t *flash_attach(void)
     flash_dev_t *dev;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: flash_attach()\n");
-#endif
+    DEBUG(0, "iflash2+_mtd: flash_attach()\n");
 
     /* Create new memory card device */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -510,10 +498,7 @@ static void flash_detach(dev_link_t *link)
     int ret;
     long flags;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: flash_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "iflash2+_mtd: flash_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -570,10 +555,7 @@ static void flash_config(dev_link_t *link)
     region_info_t region;
     int i, attr, ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: flash_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "iflash2+_mtd: flash_config(0x%p)\n", link);
 
     /* Allocate a 4K memory window */
     req.Attributes = WIN_DATA_WIDTH_16;
@@ -652,10 +634,7 @@ static void flash_release(u_long arg)
     flash_dev_t *dev;
     int i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: flash_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "iflash2+_mtd: flash_release(0x%p)\n", link);
 
     link->state &= ~DEV_CONFIG;
     if (link->win) {
@@ -702,12 +681,9 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     u_long time;
 #endif
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: flash_read(0x%p, 0x%ld, "
-	       "0x%p, 0x%lx, 0x%lx)\n", link, req->MediaID, buf,
-	       req->SrcCardOffset, req->TransferLength);
-#endif
+    DEBUG(2, "iflash2+_mtd: flash_read(0x%p, 0x%ld, 0x%p, 0x%lx, "
+	  "0x%lx)\n", link, req->MediaID, buf, req->SrcCardOffset,
+	  req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
     region = &flash->region;
@@ -725,10 +701,7 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     if (flash->cell[cell].state & FLASH_ERASING) {
 	if ((flash->cell[cell].erase_addr / region->BlockSize) ==
 	    (req->SrcCardOffset / region->BlockSize)) {
-#ifdef PCMCIA_DEBUG
-	    if (pc_debug > 1)
-		printk(KERN_DEBUG "iflash2+_mtd: delaying read...\n");
-#endif
+	    DEBUG(1, "iflash2+_mtd: delaying read...\n");
 	    req->Status = MTD_WAITREQ;
 	    return CS_BUSY;
 	}
@@ -768,11 +741,11 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     
 #ifdef PCMCIA_DEBUG
     time = uticks() - time;
-    if (pc_debug && (time < 10000000))
-	printk(KERN_DEBUG "iflash2+_mtd: read complete, time = %ld,"
-	       " avg = %ld ns/word, rate = %ld kb/sec\n", time,
-	       time*2000/req->TransferLength,
-	       req->TransferLength*977/time);
+    if (time < 10000000)
+	DEBUG(3, "iflash2+_mtd: read complete, time = %ld,"
+	      " avg = %ld ns/word, rate = %ld kb/sec\n", time,
+	      time*2000/req->TransferLength,
+	      req->TransferLength*977/time);
 #endif
     
 done:
@@ -804,11 +777,7 @@ static int basic_write(struct wait_queue **queue, char *esr, char *dest,
     
     /* Fix for mis-aligned writes */
     if ((u_long)dest & 1) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "iflash2+_mtd: odd address fixup "
-		   "at 0x%p...\n", dest);
-#endif
+	DEBUG(2, "iflash2+_mtd: odd address fixup at 0x%p\n", dest);
 	ret = page_setup(queue, (u_short *)esr, (u_short *)dest, 1);
 	if (ret != CS_SUCCESS) return ret;
 	if (is_krnl)
@@ -868,12 +837,9 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
     u_long time;
 #endif
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: flash_write(0x%p, 0x%lx, "
-	       "0x%p, 0x%lx, 0x%lx)\n", link, req->MediaID, buf,
-	       req->DestCardOffset, req->TransferLength);
-#endif
+    DEBUG(2, "iflash2+_mtd: flash_write(0x%p, 0x%lx, "
+	  "0x%p, 0x%lx, 0x%lx)\n", link, req->MediaID, buf,
+	  req->DestCardOffset, req->TransferLength);
 
     /* Check card write protect status */
     ret = CardServices(GetStatus, link->handle, &status);
@@ -896,10 +862,7 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
     /* Is this cell being erased or written? */
     cell = (req->DestCardOffset - region->CardOffset) / flash->cell_size;
     if (flash->cell[cell].state & FLASH_ERASING) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug > 1)
-	    printk(KERN_DEBUG "iflash2+_mtd: delaying write...\n");
-#endif
+	DEBUG(1, "iflash2+_mtd: delaying write...\n");
 	req->Status = MTD_WAITREQ;
 	return CS_BUSY;
     }
@@ -952,11 +915,11 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
 
 #ifdef PCMCIA_DEBUG
     time = uticks() - time;
-    if (pc_debug && (time < 10000000))
-	printk(KERN_DEBUG "iflash2+_mtd: write complete, time = %ld,"
-	       " avg = %ld us/word, rate = %ld kb/sec\n", time,
-	       time*2/req->TransferLength,
-	       req->TransferLength*977/time);
+    if (time < 10000000)
+	DEBUG(3, "iflash2+_mtd: write complete, time = %ld,"
+	      " avg = %ld us/word, rate = %ld kb/sec\n", time,
+	      time*2/req->TransferLength,
+	      req->TransferLength*977/time);
 #endif
     
 done:
@@ -985,12 +948,9 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     mtd_mod_win_t mod;
     int i, ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: flash_erase(0x%p, 0x%lx, "
-	       "0x%lx, 0x%lx)\n", link, req->MediaID,
-	       req->DestCardOffset, req->TransferLength);
-#endif
+    DEBUG(2, "iflash2+_mtd: flash_erase(0x%p, 0x%lx, "
+	  "0x%lx, 0x%lx)\n", link, req->MediaID,
+	  req->DestCardOffset, req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
     region = &flash->region;
@@ -1001,10 +961,7 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     
     if (!(req->Function & MTD_REQ_TIMEOUT)) {
 	if (flash->cell[i].state & (FLASH_ERASING|FLASH_PENDING)) {
-#ifdef PCMCIA_DEBUG
-	    if (pc_debug > 1)
-		printk(KERN_DEBUG "iflash2+_mtd: delaying erase...\n");
-#endif
+	    DEBUG(1, "iflash2+_mtd: delaying erase...\n");
 	    req->Status = MTD_WAITREQ;
 	    return CS_BUSY;
 	}
@@ -1076,11 +1033,8 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     return CS_BUSY;
     
 done:
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "iflash2+_mtd: erase complete, time = %ld\n",
-	       jiffies - flash->cell[i].erase_time);
-#endif
+    DEBUG(2, "iflash2+_mtd: erase complete, time = %ld\n",
+	  jiffies - flash->cell[i].erase_time);
     flash->cell[i].state &= ~(FLASH_ERASING|FLASH_PENDING);
     reset_block((u_short *)dev->ESRbase);
     set_global_lock(dev->ESRwin, dev->ESRbase, 1);
@@ -1143,10 +1097,7 @@ static int flash_event(event_t event, int priority,
 {
     dev_link_t *link = args->client_data;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2+_mtd: flash_event(%x)\n", event);
-#endif
+    DEBUG(1, "iflash2+_mtd: flash_event(0x%06x)\n", event);
     
     switch (event) {
 	
@@ -1193,10 +1144,7 @@ int init_module(void)
 {
     servinfo_t serv;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
@@ -1212,10 +1160,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "iflash2+_mtd: unloading\n");
-#endif
+    DEBUG(0, "iflash2+_mtd: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL)
 	flash_detach(dev_list);
