@@ -43,8 +43,11 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"iflash2_mtd.c 1.24 1997/03/16 20:08:00 (David Hinds)";
+"iflash2_mtd.c 1.25 1997/06/01 05:03:16 (David Hinds)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -128,10 +131,7 @@ static void abort_cmd(dev_link_t *link, volatile caddr_t base,
     u_char *addr = base + CISREG_SLEEP + ((cell>>3)<<1);
     /* Map the CIS register page into memory */
     mtd_mod_win_t mod = { WIN_MEMORY_TYPE_AM, 250, 0x4000 };
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "iflash2_mtd: abort\n");
-#endif
+    DEBUG(1, "iflash2_mtd: abort\n");
     MTDHelperEntry(MTDModifyWindow, link->win, &mod);
     /* Put the cell to sleep, then wake it up again */
     writeb(readb(addr) | (1 << (cell & 7)), addr);
@@ -309,10 +309,7 @@ static int vpp_setup(dev_link_t *link, mtd_request_t *req)
 	    if (dev->vpp_timeout.expires)
 		del_timer(&dev->vpp_timeout);
 	    else {
-#ifdef PCMCIA_DEBUG
-		if (pc_debug > 1)
-		    printk(KERN_DEBUG "iflash2_mtd: raising Vpp...\n");
-#endif
+		DEBUG(1, "iflash2_mtd: raising Vpp...\n");
 		dev->vpp_start = jiffies;
 		vpp_req.Vpp1 = vpp_req.Vpp2 = 120;
 		MTDHelperEntry(MTDSetVpp, link->handle, &vpp_req);
@@ -334,10 +331,7 @@ static void vpp_off(u_long arg)
     flash_dev_t *dev;
     mtd_vpp_req_t req;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2_mtd: lowering Vpp...\n");
-#endif
+    DEBUG(1, "iflash2_mtd: lowering Vpp...\n");
     dev = (flash_dev_t *)link->priv;
     dev->vpp_timeout.expires = 0;
     req.Vpp1 = req.Vpp2 = 0;
@@ -370,10 +364,7 @@ static dev_link_t *flash_attach(void)
     flash_dev_t *dev;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2_mtd: flash_attach()\n");
-#endif
+    DEBUG(0, "iflash2_mtd: flash_attach()\n");
 
     /* Create new memory card device */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -423,10 +414,7 @@ static void flash_detach(dev_link_t *link)
     int ret;
     long flags;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2_mtd: flash_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "iflash2_mtd: flash_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -483,10 +471,7 @@ static void flash_config(dev_link_t *link)
     region_info_t region;
     int i, attr, ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2_mtd: flash_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "iflash2_mtd: flash_config(0x%p)\n", link);
 
     /* Allocate a 4K memory window */
     req.Attributes = WIN_DATA_WIDTH_16;
@@ -553,10 +538,7 @@ static void flash_release(u_long arg)
     flash_dev_t *dev;
     int i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2_mtd: flash_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "iflash2_mtd: flash_release(0x%p)\n", link);
 
     link->state &= ~DEV_CONFIG;
     if (link->win) {
@@ -595,12 +577,9 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     u_long from, length, nb, time, cell;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2_mtd: flash_read(0x%p, 0x%ld, 0x%p,"
-	       " 0x%lx, 0x%lx)\n", link, req->MediaID, buf,
-	       req->SrcCardOffset, req->TransferLength);
-#endif
+    DEBUG(2, "iflash2_mtd: flash_read(0x%p, 0x%ld, 0x%p, 0x%lx, "
+	  "0x%lx)\n", link, req->MediaID, buf, req->SrcCardOffset,
+	  req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
     region = &flash->region;
@@ -655,11 +634,11 @@ static int flash_read(dev_link_t *link, char *buf, mtd_request_t *req)
     
 #ifdef PCMCIA_DEBUG
     time = jiffies - time;
-    if (pc_debug && (time > 1))
-	printk(KERN_DEBUG "iflash2_mtd: read complete, time = %ld, "
-	       "avg = %ld ns/word, rate = %ld kb/sec\n", time,
-	       time*20000000/req->TransferLength,
-	       req->TransferLength*100/(time*1024));
+    if (time > 1)
+	DEBUG(3, "iflash2_mtd: read complete, time = %ld, "
+	      "avg = %ld ns/word, rate = %ld kb/sec\n", time,
+	      time*20000000/req->TransferLength,
+	      req->TransferLength*100/(time*1024));
 #endif
     
 done:
@@ -735,12 +714,9 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
     status_t status;
     int ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2_mtd: flash_write(0x%p, 0x%lx, "
-	       "0x%p, 0x%lx, 0x%lx)\n", link, req->MediaID, buf,
-	       req->DestCardOffset, req->TransferLength);
-#endif
+    DEBUG(2, "iflash2_mtd: flash_write(0x%p, 0x%lx, 0x%p, 0x%lx, "
+	  "0x%lx)\n", link, req->MediaID, buf, req->DestCardOffset,
+	  req->TransferLength);
 
     /* Check card write protect status */
     ret = CardServices(GetStatus, link->handle, &status);
@@ -804,11 +780,11 @@ static int flash_write(dev_link_t *link, char *buf, mtd_request_t *req)
 
 #ifdef PCMCIA_DEBUG
     time = jiffies - time;
-    if (pc_debug && (time > 1))
-	printk(KERN_DEBUG "iflash2_mtd: write complete, time = %ld, "
-	       "avg = %ld us/word, rate = %ld kb/sec\n", time,
-	       time*20000/req->TransferLength,
-	       req->TransferLength*100/(time*1024));
+    if (time > 1)
+	DEBUG(3, "iflash2_mtd: write complete, time = %ld, "
+	      "avg = %ld us/word, rate = %ld kb/sec\n", time,
+	      time*20000/req->TransferLength,
+	      req->TransferLength*100/(time*1024));
 #endif
     
 done:
@@ -835,12 +811,9 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     mtd_mod_win_t mod;
     int i, ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2_mtd: flash_erase(0x%p, 0x%lx, "
-	       "0x%lx, 0x%lx)\n", link, req->MediaID,
-	       req->DestCardOffset, req->TransferLength);
-#endif
+    DEBUG(2, "iflash2_mtd: flash_erase(0x%p, 0x%lx, 0x%lx, 0x%lx)\n",
+	  link, req->MediaID, req->DestCardOffset,
+	  req->TransferLength);
 
     flash = (flash_region_t *)(req->MediaID);
     region = &flash->region;
@@ -919,11 +892,8 @@ static int flash_erase(dev_link_t *link, char *buf, mtd_request_t *req)
     return CS_BUSY;
     
 done:
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "iflash2_mtd: erase complete, time = %ld\n",
-	       jiffies - flash->cell[i].erase_time);
-#endif
+    DEBUG(2, "iflash2_mtd: erase complete, time = %ld\n",
+	  jiffies - flash->cell[i].erase_time);
     flash->cell[i].state &= ~(FLASH_ERASING|FLASH_PENDING);
     reset_block((u_short *)dev->Base);
     vpp_shutdown(link);
@@ -985,10 +955,7 @@ static int flash_event(event_t event, int priority,
 {
     dev_link_t *link = args->client_data;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "iflash2_mtd: flash_event(%x)\n", event);
-#endif
+    DEBUG(1, "iflash2_mtd: flash_event(0x%06x)\n", event);
     
     switch (event) {
 	
@@ -1031,10 +998,7 @@ int init_module(void)
 {
     servinfo_t serv;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
@@ -1050,10 +1014,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "iflash2_mtd: unloading\n");
-#endif
+    DEBUG(0, "iflash2_mtd: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL)
 	flash_detach(dev_list);
