@@ -55,8 +55,11 @@ static int major_dev = 0;
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"memory_cs.c 1.28 1997/03/16 20:10:53 (David Hinds)";
+"memory_cs.c 1.33 1997/06/01 05:03:16 (David Hinds)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -108,8 +111,10 @@ static FS_SIZE_T memory_read(struct inode *inode, struct file *file,
 static FS_SIZE_T memory_write(struct inode *inode, struct file *file,
 			      CONST char *buf, U_FS_SIZE_T count);
 static int memory_open(struct inode *inode, struct file *file);
-static void memory_close(struct inode *inode, struct file *file);
-static void memory_blk_close(struct inode *inode, struct file *file);
+static FS_RELEASE_T memory_close(struct inode *inode,
+				 struct file *file);
+static FS_RELEASE_T memory_blk_close(struct inode *inode,
+				     struct file *file);
 
 static struct file_operations memory_chr_fops = {
     NULL,		/* lseek */
@@ -160,10 +165,7 @@ static dev_link_t *memory_attach(void)
     eraseq_hdr_t eraseq_hdr;
     int i, ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_attach()\n");
-#endif
+    DEBUG(0, "memory_attach()\n");
 
     for (i = 0; i < MAX_DEV; i++)
 	if (dev_table[i] == NULL) break;
@@ -231,10 +233,7 @@ static void memory_detach(dev_link_t *link)
     memory_dev_t *dev;
     int i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "memory_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (i = 0; i < MAX_DEV; i++)
@@ -279,10 +278,7 @@ static void memory_config(dev_link_t *link)
     region_info_t region;
     int i, attr, ret, nr[2];
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "memory_config(0x%p)\n", link);
 
     /* Configure card */
     link->state |= DEV_CONFIG;
@@ -350,19 +346,13 @@ static void memory_release(u_long arg)
     dev_link_t *link = (dev_link_t *)arg;
     int i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "memory_release(0x%p)\n", link);
 
     for (i = 0; i < MAX_DEV; i++)
 	if (dev_table[i] == link) break;
     if (link->open) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "memory_cs: release postponed, 'mem%d'"
-		   " still open\n", i);
-#endif
+	DEBUG(0, "memory_cs: release postponed, 'mem%d'"
+	      " still open\n", i);
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
@@ -393,18 +383,9 @@ static int memory_event(event_t event, int priority,
     memory_dev_t *dev;
     eraseq_entry_t *erase;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_event()\n");
-#endif
+    DEBUG(1, "memory_event(0x%06x)\n", event);
     
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "memory_cs: registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -454,10 +435,7 @@ static int memory_open(struct inode *inode, struct file *file)
     open_mem_t open;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_open(%d)\n", minor);
-#endif
+    DEBUG(0, "memory_open(%d)\n", minor);
 
     link = dev_table[DEVICE_NR(minor)];
     if (!DEV_OK(link))
@@ -485,17 +463,15 @@ static int memory_open(struct inode *inode, struct file *file)
 
 /*====================================================================*/
 
-static void memory_close(struct inode *inode, struct file *file)
+static FS_RELEASE_T memory_close(struct inode *inode,
+				 struct file *file)
 {
     dev_link_t *link;
     int minor = MINOR(inode->i_rdev);
     memory_dev_t *dev;
     minor_dev_t *minor_dev;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_close(%d)\n", minor);
-#endif
+    DEBUG(0, "memory_close(%d)\n", minor);
 
     link = dev_table[DEVICE_NR(minor)];
     dev = (memory_dev_t *)link->priv;
@@ -508,17 +484,18 @@ static void memory_close(struct inode *inode, struct file *file)
     }
 
     MOD_DEC_USE_COUNT;
-
+    return (FS_RELEASE_T)0;
 } /* memory_close */
 
-static void memory_blk_close(struct inode *inode, struct file *file)
+static FS_RELEASE_T memory_blk_close(struct inode *inode,
+				     struct file *file)
 {
     fsync_dev(inode->i_rdev);
 #if (LINUX_VERSION_CODE >= VERSION(1,3,0))
     invalidate_inodes(inode->i_rdev);
 #endif
     invalidate_buffers(inode->i_rdev);
-    memory_close(inode, file);
+    return memory_close(inode, file);
 }
 
 /*======================================================================
@@ -535,11 +512,8 @@ static FS_SIZE_T memory_read(struct inode *inode, struct file *file,
     int ret;
 
     minor = file->private_data;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_read(0x%p, 0x%lx, %d)\n",
-	       minor->handle, (u_long)file->f_pos, count);
-#endif
+    DEBUG(2, "memory_read(0x%p, 0x%lx, %ld)\n", minor->handle,
+	  (u_long)file->f_pos, (u_long)count);
     req.Attributes = MEM_OP_BUFFER_USER;
     req.Offset = file->f_pos;
     req.Count = count;
@@ -547,8 +521,7 @@ static FS_SIZE_T memory_read(struct inode *inode, struct file *file,
     if (ret == CS_SUCCESS) {
 	file->f_pos += count;
 	return count;
-    }
-    else
+    } else
 	return -EIO;
 } /* memory_read */
 
@@ -570,10 +543,7 @@ static int memory_erase(memory_dev_t *dev, minor_dev_t *minor_dev,
 	for (i = 0; i < MAX_ERASE; i++)
 	    if (!ERASE_IN_PROGRESS(dev->eraseq[i].State)) break;
 	if (i < MAX_ERASE) break;
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "waiting for erase slot...\n");
-#endif
+	DEBUG(2, "waiting for erase slot...\n");
 	interruptible_sleep_on(&dev->erase_pending);
 	if (current->signal & ~current->blocked)
 	    return -ERESTARTSYS;
@@ -616,11 +586,8 @@ static FS_SIZE_T memory_write(struct inode *inode, struct file *file,
 
     dev = (memory_dev_t *)dev_table[DEVICE_NR(minor)]->priv;
     minor_dev = &dev->minor[REGION_NR(minor)];
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_write(0x%p, 0x%lx, %d)\n",
-	       minor_dev->handle, (u_long)file->f_pos, count);
-#endif
+    DEBUG(2, "memory_write(0x%p, 0x%lx, %ld)\n", minor_dev->handle,
+	  (u_long)file->f_pos, (u_long)count);
     if ((minor_dev->region.BlockSize > 1) &&
 	((file->f_pos & (minor_dev->region.BlockSize-1)) == 0) &&
 	((count & (minor_dev->region.BlockSize-1)) == 0)) {
@@ -767,10 +734,7 @@ int init_module(void)
     servinfo_t serv;
     int i;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
@@ -807,10 +771,7 @@ void cleanup_module(void)
     int i;
     dev_link_t *link;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "memory_cs: unloading\n");
-#endif
+    DEBUG(0, "memory_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     if (major_dev != 0) {
 	unregister_chrdev(major_dev, "memory");
