@@ -383,9 +383,9 @@ typedef struct _mace_private {
 Private Global Variables
 ---------------------------------------------------------------------------- */
 
+#ifdef PCMCIA_DEBUG
 static char rcsid[] =
 "nmclan_cs.c,v 0.16 1995/07/01 06:42:17 rpao Exp rpao";
-#ifdef PCMCIA_DEBUG
 static char *version =
 "nmclan_cs 0.16 (Roger C. Pao)";
 #endif
@@ -403,17 +403,11 @@ static char *if_names[]={
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
-#endif
-
-/* #define NMCLAN_DEBUG 6 */
-#ifdef NMCLAN_DEBUG
-static int nmclan_debug = NMCLAN_DEBUG;
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 #else
-static int nmclan_debug = 2;
+#define DEBUG(n, args...)
 #endif
 
-MODULE_PARM(nmclan_debug, "i");
-
 /* ----------------------------------------------------------------------------
 Parameters
 	These are the parameters that can be set during loading with
@@ -496,15 +490,8 @@ static dev_link_t *nmclan_attach(void)
   struct device *dev;
   int ret;
 
-
-#ifdef PCMCIA_DEBUG
-  if (pc_debug)
-    printk(KERN_DEBUG "nmclan_attach()\n");
-#endif
-
-  if (nmclan_debug > 3)
-    printk(KERN_DEBUG "%s\n", rcsid);
-
+  DEBUG(0, "nmclan_attach()\n");
+  DEBUG(1, "%s\n", rcsid);
 
   /* Create new ethernet device */
   link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -579,10 +566,7 @@ static void nmclan_detach(dev_link_t *link)
   dev_link_t **linkp;
 
 
-#ifdef PCMCIA_DEBUG
-  if (pc_debug)
-    printk(KERN_DEBUG "nmclan_detach(0x%p)\n", link);
-#endif
+  DEBUG(0, "nmclan_detach(0x%p)\n", link);
 
   /* Locate device structure */
   for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -758,10 +742,7 @@ static void nmclan_config(dev_link_t *link)
   dev = link->priv;
   phys_addr = (short *)dev->dev_addr;
 
-#ifdef PCMCIA_DEBUG
-  if (pc_debug)
-    printk(KERN_DEBUG "nmclan_config(0x%p)\n", link);
-#endif
+  DEBUG(0, "nmclan_config(0x%p)\n", link);
 
   do {
     tuple.Attributes = 0;
@@ -842,11 +823,8 @@ static void nmclan_config(dev_link_t *link)
     sig[0] = mace_read(ioaddr, MACE_CHIPIDL);
     sig[1] = mace_read(ioaddr, MACE_CHIPIDH);
     if ((sig[0] == 0x40) && ((sig[1] & 0x0F) == 0x09)) {
-#ifdef PCMCIA_DEBUG
-      if (pc_debug)
-	printk(KERN_INFO "nmclan_cs configured: mace id=%x %x\n",
-	       sig[0], sig[1]);
-#endif
+      DEBUG(0, "nmclan_cs configured: mace id=%x %x\n",
+	    sig[0], sig[1]);
     } else {
       printk(KERN_NOTICE "nmclan_cs: mace id not found: %x %x should"
 	     " be 0x40 0x?9\n", sig[0], sig[1]);
@@ -867,9 +845,8 @@ static void nmclan_config(dev_link_t *link)
   /* Determine which port we are using if auto is selected */
   if (if_port==0) {
     mace_write(ioaddr, MACE_MACCC, MACE_MACCC_ENXMT | MACE_MACCC_ENRCV);
-    if (nmclan_debug > 2)
-      printk(KERN_DEBUG "%s: mace_phycc 0x%X.\n", dev->name,
-	     mace_read(ioaddr, MACE_PHYCC));
+    DEBUG(2, "%s: mace_phycc 0x%X.\n", dev->name,
+	  mace_read(ioaddr, MACE_PHYCC));
     if (mace_read(ioaddr, MACE_PHYCC) & MACE_PHYCC_LNKFL)
       /* 10base-T receiver is in link fail, MACE is using AUI port. */
       dev->if_port = 2;
@@ -907,17 +884,11 @@ static void nmclan_release(u_long arg)
   dev_link_t *link = (dev_link_t *)arg;
   struct device *dev = link->priv;
 
-#ifdef PCMCIA_DEBUG
-  if (pc_debug)
-    printk(KERN_DEBUG "nmclan_release(0x%p)\n", link);
-#endif
+  DEBUG(0, "nmclan_release(0x%p)\n", link);
 
   if (link->open) {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-      printk(KERN_DEBUG "nmclan_cs: release postponed, '%s' "
-	     "still open\n", link->dev->dev_name);
-#endif
+    DEBUG(1, "nmclan_cs: release postponed, '%s' "
+	  "still open\n", link->dev->dev_name);
     link->state |= DEV_STALE_CONFIG;
     return;
   }
@@ -951,19 +922,9 @@ static int nmclan_event(event_t event, int priority,
   dev_link_t *link = args->client_data;
   struct device *dev = link->priv;
 
-    
-#ifdef PCMCIA_DEBUG
-  if (pc_debug)
-    printk(KERN_DEBUG "nmclan_event()\n");
-#endif
+  DEBUG(1 "nmclan_event(0x%06x)\n", event);
     
   switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-      if (pc_debug)
-	printk(KERN_DEBUG "nmclan_cs: registration complete\n");
-      break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
       link->state &= ~DEV_PRESENT;
       if (link->state & DEV_CONFIG) {
@@ -1041,12 +1002,11 @@ static void nmclan_reset(struct device *dev)
   /* Reset Xilinx */
   reg.Action = CS_WRITE;
   reg.Offset = CISREG_COR;
-  printk(KERN_DEBUG "nmclan_reset: OrigCorValue=0x%lX, resetting...",
-	 OrigCorValue);
+  DEBUG(1, "nmclan_reset: OrigCorValue=0x%lX, resetting...\n",
+	OrigCorValue);
   reg.Value = COR_SOFT_RESET;
   CardServices(AccessConfigurationRegister, link->handle, &reg);
   /* Need to wait for 20 ms for PCMCIA to finish reset. */
-  printk(" complete\n");
 
   /* Restore original COR configuration index */
   reg.Value = COR_LEVEL_REQ | (OrigCorValue & COR_CONFIG_MASK);
@@ -1136,8 +1096,7 @@ static int mace_close(struct device *dev)
   if (link == NULL)
     return -ENODEV;
 
-  if (nmclan_debug > 2)
-    printk(KERN_DEBUG "%s: shutting down ethercard.\n", dev->name);
+  DEBUG(2, "%s: shutting down ethercard.\n", dev->name);
   
   /* Mask off all interrupts from the MACE chip. */
   outb(0xFF, ioaddr + AM2150_MACE_BASE + MACE_IMR);
@@ -1206,15 +1165,11 @@ static int mace_start_xmit(struct sk_buff *skb, struct device *dev)
     return 0;
 #endif
  
-#ifdef NMCLAN_DEBUG
-  if (nmclan_debug > 4) {
-    printk(KERN_DEBUG "%s: mace_start_xmit(length = %ld) called.\n",
-	   dev->name, skb->len);
-  }
-#endif /* #ifdef NMCLAN_DEBUG */
-
+  DEBUG(3, "%s: mace_start_xmit(length = %ld) called.\n",
+	dev->name, skb->len);
+  
   /* Avoid timer-based retransmission conflicts. */
-  if (set_bit(TBUSY_UNSPECIFIED, (void*)&dev->tbusy) != 0) {
+  if (test_and_set_bit(TBUSY_UNSPECIFIED, (void*)&dev->tbusy) != 0) {
     printk(KERN_NOTICE "%s: transmitter access conflict.\n",
 	   dev->name);
     return 1;
@@ -1288,8 +1243,8 @@ static void mace_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 
 
   if (dev == NULL) {
-    printk(KERN_DEBUG "mace_interrupt(): irq 0x%X for "
-	   "unknown device.\n", irq);
+    DEBUG(2, "mace_interrupt(): irq 0x%X for unknown device.\n",
+	  irq);
     return;
   }
 
@@ -1312,8 +1267,7 @@ static void mace_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
   sti();
 
   if (dev->start == 0) {
-    if (nmclan_debug > 2)
-    printk(KERN_DEBUG "%s: interrupt from dead card\n", dev->name);
+    DEBUG(2, "%s: interrupt from dead card\n", dev->name);
     goto exception;
   }
 
@@ -1322,11 +1276,7 @@ static void mace_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
     /* WARNING: MACE_IR is a READ/CLEAR port! */
     status = inb(ioaddr + AM2150_MACE_BASE + MACE_IR);
  
-    if (nmclan_debug > 4) {
-      printk(KERN_DEBUG "mace_interrupt: irq 0x%X status 0x%X.\n",
-	     irq, status);
-    }
-
+    DEBUG(3, "mace_interrupt: irq 0x%X status 0x%X.\n", irq, status);
 
     if (status & MACE_IR_RCVINT) {
       mace_rx(dev, MACE_MAX_RX_ITERATIONS);
@@ -1453,10 +1403,8 @@ static int mace_rx(struct device *dev, unsigned char RxCnt)
   ) {
     rx_status = inw(ioaddr + AM2150_RCV);
 
-    if (nmclan_debug > 5) {
-      printk(KERN_DEBUG "%s: in mace_rx(), framecnt 0x%X, rx_status"
-	     " 0x%X.\n", dev->name, rx_framecnt, rx_status);
-    }
+    DEBUG(3, "%s: in mace_rx(), framecnt 0x%X, rx_status"
+	  " 0x%X.\n", dev->name, rx_framecnt, rx_status);
 
     if (rx_status & MACE_RCVFS_RCVSTS) { /* Error, update stats. */
       lp->linux_stats.rx_errors++;
@@ -1482,11 +1430,8 @@ static int mace_rx(struct device *dev, unsigned char RxCnt)
       lp->mace_stats.rfs_rcvcc += inb(ioaddr + AM2150_RCV);
         /* rcv collision count */
 
-#ifdef NMCLAN_DEBUG
-      if (nmclan_debug > 4)
-	printk(KERN_DEBUG "    receiving packet size 0x%X rx_status"
-	       " 0x%X.\n", pkt_len, rx_status);
-#endif /* #ifdef NMCLAN_DEBUG */
+      DEBUG(3, "    receiving packet size 0x%X rx_status"
+	    " 0x%X.\n", pkt_len, rx_status);
       
       skb = ALLOC_SKB(pkt_len);
       
@@ -1515,9 +1460,8 @@ static int mace_rx(struct device *dev, unsigned char RxCnt)
 	outb(0xFF, ioaddr + AM2150_RCV_NEXT); /* skip to next frame */
 	continue;
       } else {
-        if (nmclan_debug)
-	  printk(KERN_DEBUG "%s: couldn't allocate a sk_buff of size"
-		 " %d.\n", dev->name, pkt_len);
+	DEBUG(1, "%s: couldn't allocate a sk_buff of size"
+	      " %d.\n", dev->name, pkt_len);
 	lp->linux_stats.rx_dropped++;
       }
     }
@@ -1533,29 +1477,29 @@ pr_linux_stats
 ---------------------------------------------------------------------------- */
 static void pr_linux_stats(struct net_device_stats *pstats)
 {
-  printk(KERN_DEBUG "pr_linux_stats\n");
-  printk(KERN_DEBUG " rx_packets=%-7ld        tx_packets=%ld\n",
-    (long)pstats->rx_packets, (long)pstats->tx_packets);
-  printk(KERN_DEBUG " rx_errors=%-7ld         tx_errors=%ld\n",
-    (long)pstats->rx_errors, (long)pstats->tx_errors);
-  printk(KERN_DEBUG " rx_dropped=%-7ld        tx_dropped=%ld\n",
-    (long)pstats->rx_dropped, (long)pstats->tx_dropped);
-  printk(KERN_DEBUG " multicast=%-7ld         collisions=%ld\n",
-    (long)pstats->multicast, (long)pstats->collisions);
+  DEBUG(2, "pr_linux_stats\n");
+  DEBUG(2, " rx_packets=%-7ld        tx_packets=%ld\n",
+	(long)pstats->rx_packets, (long)pstats->tx_packets);
+  DEBUG(2, " rx_errors=%-7ld         tx_errors=%ld\n",
+	(long)pstats->rx_errors, (long)pstats->tx_errors);
+  DEBUG(2, " rx_dropped=%-7ld        tx_dropped=%ld\n",
+	(long)pstats->rx_dropped, (long)pstats->tx_dropped);
+  DEBUG(2, " multicast=%-7ld         collisions=%ld\n",
+	(long)pstats->multicast, (long)pstats->collisions);
 
-  printk(KERN_DEBUG " rx_length_errors=%-7ld  rx_over_errors=%ld\n",
-    (long)pstats->rx_length_errors, (long)pstats->rx_over_errors);
-  printk(KERN_DEBUG " rx_crc_errors=%-7ld     rx_frame_errors=%ld\n",
-    (long)pstats->rx_crc_errors, (long)pstats->rx_frame_errors);
-  printk(KERN_DEBUG " rx_fifo_errors=%-7ld    rx_missed_errors=%ld\n",
-    (long)pstats->rx_fifo_errors, (long)pstats->rx_missed_errors);
+  DEBUG(2, " rx_length_errors=%-7ld  rx_over_errors=%ld\n",
+	(long)pstats->rx_length_errors, (long)pstats->rx_over_errors);
+  DEBUG(2, " rx_crc_errors=%-7ld     rx_frame_errors=%ld\n",
+	(long)pstats->rx_crc_errors, (long)pstats->rx_frame_errors);
+  DEBUG(2, " rx_fifo_errors=%-7ld    rx_missed_errors=%ld\n",
+	(long)pstats->rx_fifo_errors, (long)pstats->rx_missed_errors);
 
-  printk(KERN_DEBUG " tx_aborted_errors=%-7ld tx_carrier_errors=%ld\n",
-    (long)pstats->tx_aborted_errors, (long)pstats->tx_carrier_errors);
-  printk(KERN_DEBUG " tx_fifo_errors=%-7ld    tx_heartbeat_errors=%ld\n",
-    (long)pstats->tx_fifo_errors, (long)pstats->tx_heartbeat_errors);
-  printk(KERN_DEBUG " tx_window_errors=%ld\n",
-    (long)pstats->tx_window_errors);
+  DEBUG(2, " tx_aborted_errors=%-7ld tx_carrier_errors=%ld\n",
+	(long)pstats->tx_aborted_errors, (long)pstats->tx_carrier_errors);
+  DEBUG(2, " tx_fifo_errors=%-7ld    tx_heartbeat_errors=%ld\n",
+	(long)pstats->tx_fifo_errors, (long)pstats->tx_heartbeat_errors);
+  DEBUG(2, " tx_window_errors=%ld\n",
+	(long)pstats->tx_window_errors);
 } /* pr_linux_stats */
 
 
@@ -1564,51 +1508,48 @@ pr_mace_stats
 ---------------------------------------------------------------------------- */
 static void pr_mace_stats(mace_statistics *pstats)
 {
-  printk(KERN_DEBUG "pr_mace_stats\n");
+  DEBUG(2, "pr_mace_stats\n");
 
-  printk(KERN_DEBUG " xmtsv=%-7d             uflo=%d\n",
-    pstats->xmtsv, pstats->uflo);
-  printk(KERN_DEBUG " lcol=%-7d              more=%d\n",
-    pstats->lcol, pstats->more);
-  printk(KERN_DEBUG " one=%-7d               defer=%d\n",
-    pstats->one, pstats->defer);
-  printk(KERN_DEBUG " lcar=%-7d              rtry=%d\n",
-    pstats->lcar, pstats->rtry);
+  DEBUG(2, " xmtsv=%-7d             uflo=%d\n",
+	pstats->xmtsv, pstats->uflo);
+  DEBUG(2, " lcol=%-7d              more=%d\n",
+	pstats->lcol, pstats->more);
+  DEBUG(2, " one=%-7d               defer=%d\n",
+	pstats->one, pstats->defer);
+  DEBUG(2, " lcar=%-7d              rtry=%d\n",
+	pstats->lcar, pstats->rtry);
 
   /* MACE_XMTRC */
-  printk(KERN_DEBUG " exdef=%-7d             xmtrc=%d\n",
-    pstats->exdef, pstats->xmtrc);
-
+  DEBUG(2, " exdef=%-7d             xmtrc=%d\n",
+	pstats->exdef, pstats->xmtrc);
+  
   /* RFS1--Receive Status (RCVSTS) */
-  printk(KERN_DEBUG " oflo=%-7d              clsn=%d\n",
-    pstats->oflo, pstats->clsn);
-  printk(KERN_DEBUG " fram=%-7d              fcs=%d\n",
-    pstats->fram, pstats->fcs);
+  DEBUG(2, " oflo=%-7d              clsn=%d\n",
+	pstats->oflo, pstats->clsn);
+  DEBUG(2, " fram=%-7d              fcs=%d\n",
+	pstats->fram, pstats->fcs);
 
   /* RFS2--Runt Packet Count (RNTPC) */
   /* RFS3--Receive Collision Count (RCVCC) */
-  printk(KERN_DEBUG " rfs_rntpc=%-7d         rfs_rcvcc=%d\n",
-    pstats->rfs_rntpc, pstats->rfs_rcvcc);
+  DEBUG(2, " rfs_rntpc=%-7d         rfs_rcvcc=%d\n",
+	pstats->rfs_rntpc, pstats->rfs_rcvcc);
 
   /* MACE_IR */
-  printk(KERN_DEBUG " jab=%-7d               babl=%d\n",
-    pstats->jab, pstats->babl);
-  printk(KERN_DEBUG " cerr=%-7d              rcvcco=%d\n",
-    pstats->cerr, pstats->rcvcco);
-  printk(KERN_DEBUG " rntpco=%-7d            mpco=%d\n",
-    pstats->rntpco, pstats->mpco);
+  DEBUG(2, " jab=%-7d               babl=%d\n",
+	pstats->jab, pstats->babl);
+  DEBUG(2, " cerr=%-7d              rcvcco=%d\n",
+	pstats->cerr, pstats->rcvcco);
+  DEBUG(2, " rntpco=%-7d            mpco=%d\n",
+	pstats->rntpco, pstats->mpco);
 
   /* MACE_MPC */
-  printk(KERN_DEBUG " mpc=%d\n",
-    pstats->mpc);
+  DEBUG(2, " mpc=%d\n", pstats->mpc);
 
   /* MACE_RNTPC */
-  printk(KERN_DEBUG " rntpc=%d\n",
-    pstats->rntpc);
+  DEBUG(2, " rntpc=%d\n", pstats->rntpc);
 
   /* MACE_RCVCC */
-  printk(KERN_DEBUG " rcvcc=%d\n",
-    pstats->rcvcc);
+  DEBUG(2, " rcvcc=%d\n", pstats->rcvcc);
 
 } /* pr_mace_stats */
 
@@ -1691,11 +1632,9 @@ static struct net_device_stats *mace_get_stats(struct device *dev)
 
   update_stats(dev->base_addr, dev);
 
-  if (nmclan_debug > 2 /* TBD: 5 */) {
-    printk(KERN_DEBUG "%s: updating the statistics.\n", dev->name);
-    pr_linux_stats(&lp->linux_stats);
-    pr_mace_stats(&lp->mace_stats);
-  }
+  DEBUG(1, "%s: updating the statistics.\n", dev->name);
+  pr_linux_stats(&lp->linux_stats);
+  pr_mace_stats(&lp->mace_stats);
 
   return &lp->linux_stats;
 } /* net_device_stats */
@@ -1763,7 +1702,8 @@ static void BuildLAF(int *ladrf, int *adr)
   byte = hashcode >> 3;
   ladrf[byte] |= (1 << (hashcode & 7));
 
-  if (nmclan_debug > 3) {
+#ifdef PCMCIA_DEBUG
+  if (pc_debug > 2) {
     printk(KERN_DEBUG "    adr =");
     for (i = 0; i < 6; i++)
       printk(" %02X", adr[i]);
@@ -1773,6 +1713,7 @@ static void BuildLAF(int *ladrf, int *adr)
       printk(" %02X", ladrf[i]);
     printk("\n");
   }
+#endif
 } /* BuildLAF */
 
 
@@ -1794,13 +1735,12 @@ static void restore_multicast_list(struct device *dev)
   int i;
 
 
-  if (nmclan_debug > 3)
-    printk(KERN_DEBUG "%s: restoring Rx mode to %d addresses.\n",
-	   dev->name, num_addrs);
+  DEBUG(2, "%s: restoring Rx mode to %d addresses.\n",
+	dev->name, num_addrs);
 
   if (num_addrs > 0) {
 
-    printk(KERN_INFO "Attempt to restore multicast list detected.\n");
+    DEBUG(1, "Attempt to restore multicast list detected.\n");
 
     mace_write(ioaddr, MACE_IAC, MACE_IAC_ADDRCHG | MACE_IAC_LOGADDR);
     /* Poll ADDRCHG bit */
@@ -1858,16 +1798,16 @@ static void set_multicast_list(struct device *dev, int num_addrs, void *addrs)
   struct dev_mc_list *dmi = dev->mc_list;
 #endif
 
-
-  if (nmclan_debug > 1) {
+#ifdef PCMCIA_DEBUG
+  if (pc_debug > 1) {
     static int old = 0;
     if (num_addrs != old) {
       old = num_addrs;
-      printk(KERN_INFO "%s: setting Rx mode to %d addresses.\n",
-	     dev->name, old);
+      DEBUG(0, "%s: setting Rx mode to %d addresses.\n",
+	    dev->name, old);
     }
   }
-
+#endif
 
   /* Set multicast_num_addrs. */
   lp->multicast_num_addrs = num_addrs;
@@ -1898,17 +1838,15 @@ static void set_multicast_list(struct device *dev, int num_addrs, void *addrs)
 static void restore_multicast_list(struct device *dev)
 {
   mace_private *lp = (mace_private *)dev->priv;
-  int num_addrs = lp->multicast_num_addrs;
   short ioaddr = dev->base_addr;
 
-  if (nmclan_debug > 3)
-    printk(KERN_DEBUG "%s: restoring Rx mode to %d addresses.\n",
-	   dev->name, num_addrs);
+  DEBUG(2, "%s: restoring Rx mode to %d addresses.\n",
+	dev->name, lp->multicast_num_addrs);
   
 #ifdef NEW_MULTICAST
   if (dev->flags & IFF_PROMISC) {
 #else
-  if (num_addrs < 0) {
+  if (lp->multicast_num_addrs < 0) {
 #endif
     /* Promiscuous mode: receive all packets */
     mace_write(ioaddr, MACE_UTR, MACE_UTR_LOOP_EXTERNAL);
@@ -1931,14 +1869,16 @@ static void set_multicast_list(struct device *dev, int num_addrs, void *addrs)
 {
   mace_private *lp = (mace_private *)dev->priv;
 
+#ifdef PCMCIA_DEBUG
   if (nmclan_debug > 1) {
     static int old = 0;
     if (num_addrs != old) {
       old = num_addrs;
-      printk(KERN_INFO "%s: setting Rx mode to %d addresses.\n",
-	     dev->name, old);
+      DEBUG(0, "%s: setting Rx mode to %d addresses.\n",
+	    dev->name, old);
     }
   }
+#endif
 
   lp->multicast_num_addrs = num_addrs;
   restore_multicast_list(dev);
@@ -1951,10 +1891,7 @@ init_module
 int init_module(void)
 {
   servinfo_t serv;
-#ifdef PCMCIA_DEBUG
-  if (pc_debug)
-    printk(KERN_INFO "%s\n", version);
-#endif
+  DEBUG(0, "%s\n", version);
   CardServices(GetCardServicesInfo, &serv);
   if (serv.Revision != CS_RELEASE_CODE) {
     printk(KERN_NOTICE "nmclan_cs: Card Services release does not match!\n");
@@ -1971,10 +1908,7 @@ cleanup_module
 
 void cleanup_module(void)
 {
-#ifdef PCMCIA_DEBUG
-  if (pc_debug)
-    printk(KERN_DEBUG "nmclan_cs: unloading\n");
-#endif
+  DEBUG(0, "nmclan_cs: unloading\n");
   unregister_pcmcia_driver(&dev_info);
   while (dev_list != NULL)
     nmclan_detach(dev_list);
