@@ -51,8 +51,11 @@ static int major_dev = 0;
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"pcmem_cs.c 1.70 1997/03/16 20:08:18 (David Hinds)";
+"pcmem_cs.c 1.72 1997/06/01 05:03:16 (David Hinds)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -114,8 +117,8 @@ static FS_SIZE_T pcmem_read(struct inode *inode, struct file *file,
 static FS_SIZE_T pcmem_write(struct inode *inode, struct file *file,
 			     CONST char *buf, U_FS_SIZE_T count);
 static int pcmem_open(struct inode *inode, struct file *file);
-static void pcmem_close(struct inode *inode, struct file *file);
-static void pcmem_blk_close(struct inode *inode, struct file *file);
+static FS_RELEASE_T pcmem_close(struct inode *inode, struct file *file);
+static FS_RELEASE_T pcmem_blk_close(struct inode *inode, struct file *file);
 
 static struct file_operations pcmem_chr_fops = {
     NULL,		/* lseek */
@@ -165,10 +168,7 @@ static dev_link_t *pcmem_attach(void)
     pcmem_dev_t *dev;
     int i, ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_attach()\n");
-#endif
+    DEBUG(0, "pcmem_attach()\n");
 
     for (i = 0; i < MAX_DEV; i++)
 	if (dev_table[i] == NULL) break;
@@ -221,10 +221,7 @@ static void pcmem_detach(dev_link_t *link)
 {
     int i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "pcmem_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (i = 0; i < MAX_DEV; i++)
@@ -320,10 +317,7 @@ static void pcmem_config(dev_link_t *link)
     win_req_t req;
     int i, ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "pcmem_config(0x%p)\n", link);
 
     /* Configure card */
     link->state |= DEV_CONFIG;
@@ -399,17 +393,11 @@ static void pcmem_release(u_long arg)
 {
     dev_link_t *link = (dev_link_t *)arg;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "pcmem_release(0x%p)\n", link);
 
     if (link->open) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "pcmem_cs: release postponed, '%s' "
-		   "still open\n", link->dev->dev_name);
-#endif
+	DEBUG(0, "pcmem_cs: release postponed, '%s' "
+	      "still open\n", link->dev->dev_name);
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
@@ -438,18 +426,9 @@ static int pcmem_event(event_t event, int priority,
 {
     dev_link_t *link = args->client_data;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_event()\n");
-#endif
+    DEBUG(1, "pcmem_event(0x%06x)\n", event);
     
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -487,10 +466,7 @@ static int pcmem_open(struct inode *inode, struct file *file)
     dev_link_t *link;
     pcmem_dev_t *dev;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_open(%d)\n", MINOR(inode->i_rdev));
-#endif
+    DEBUG(0, "pcmem_open(%d)\n", MINOR(inode->i_rdev));
 
     link = dev_table[DEVICE_NR(inode->i_rdev)];
     if (!DEV_OK(link))
@@ -507,30 +483,28 @@ static int pcmem_open(struct inode *inode, struct file *file)
 
 /*====================================================================*/
 
-static void pcmem_close(struct inode *inode, struct file *file)
+static FS_RELEASE_T pcmem_close(struct inode *inode, struct file *file)
 {
     dev_link_t *link;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_close(%d)\n", MINOR(inode->i_rdev));
-#endif
+    DEBUG(0, "pcmem_close(%d)\n", MINOR(inode->i_rdev));
 
     link = dev_table[DEVICE_NR(inode->i_rdev)];
-    if (link == NULL) return;
+    if (link == NULL) return (FS_RELEASE_T)0;
 
     MOD_DEC_USE_COUNT;
-
+    return (FS_RELEASE_T)0;
 } /* pcmem_close */
 
-static void pcmem_blk_close(struct inode *inode, struct file *file)
+static FS_RELEASE_T pcmem_blk_close(struct inode *inode,
+				    struct file *file)
 {
     fsync_dev(inode->i_rdev);
 #if (LINUX_VERSION_CODE >= VERSION(1,3,0))
     invalidate_inodes(inode->i_rdev);
 #endif
     invalidate_buffers(inode->i_rdev);
-    pcmem_close(inode, file);
+    return pcmem_close(inode, file);
 }
 
 /*======================================================================
@@ -550,10 +524,8 @@ static FS_SIZE_T pcmem_read(struct inode *inode, struct file *file,
     modwin_t mod;
     memreq_t mem;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_read(%d, %d)\n", minor, count);
-#endif
+    DEBUG(2, "pcmem_read(%d, 0x%lx, %ld)\n", minor,
+	  (u_long)file->f_pos, (u_long)count);
 
     link = dev_table[DEVICE_NR(inode->i_rdev)];
     
@@ -630,10 +602,8 @@ static FS_SIZE_T pcmem_write(struct inode *inode, struct file *file,
     modwin_t mod;
     memreq_t mem;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_write(%d, %d)\n", minor, count);
-#endif
+    DEBUG(2, "pcmem_write(%d, 0x%lx, %ld)\n", minor,
+	  (u_long)file->f_pos, (u_long)count);
     
     link = dev_table[DEVICE_NR(inode->i_rdev)];
     
@@ -823,10 +793,7 @@ int init_module(void)
     servinfo_t serv;
     int i;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
@@ -863,10 +830,7 @@ void cleanup_module(void)
     int i;
     dev_link_t *link;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcmem_cs: unloading\n");
-#endif
+    DEBUG(0, "pcmem_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     if (major_dev != 0) {
 	unregister_chrdev(major_dev, "pcmem");
