@@ -69,8 +69,11 @@ static char *if_names[] = { "Auto", "10baseT", "10base2"};
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"pcnet_cs.c 1.48 1997/03/16 20:09:13 (David Hinds)";
+"pcnet_cs.c 1.52 1997/06/01 05:03:16 (David Hinds)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -163,6 +166,7 @@ static hw_info_t hw_info[] = {
     { "Dayna CommuniCard E", 0x0110, 0x00, 0x80, 0x19, 0 },
     { "D-Link DE-650", 0x0040, 0x00, 0x80, 0xc8, 0 },
     { "EP-210 Ethernet", 0x0110, 0x00, 0x40, 0x33, 0 },
+    { "EP4000 Ethernet", 0x01c0, 0x00, 0x00, 0xb4, 0 },
     { "Epson EEN10B", 0x0ff0, 0x00, 0x00, 0x48,
       USE_SHMEM | HAS_MISC_REG | HAS_IBM_MISC },
     { "ELECOM Laneed LD-CDWA", 0xb8, 0x08, 0x00, 0x42, 0 },
@@ -247,10 +251,7 @@ static dev_link_t *pcnet_attach(void)
     struct device *dev;
     int ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcnet_attach()\n");
-#endif
+    DEBUG(0, "pcnet_attach()\n");
 
     /* Create new ethernet device */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -316,10 +317,7 @@ static void pcnet_detach(dev_link_t *link)
     dev_link_t **linkp;
     long flags;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcnet_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "pcnet_detach(0x%p)\n", link);
 
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -524,10 +522,7 @@ static void pcnet_config(dev_link_t *link)
     info = link->priv;
     dev = &info->dev;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcnet_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "pcnet_config(0x%p)\n", link);
 
     tuple.Attributes = 0;
     tuple.DesiredTuple = CISTPL_CONFIG;
@@ -588,9 +583,9 @@ static void pcnet_config(dev_link_t *link)
 		    link->io.BasePort2 = link->io.BasePort1 + 0x10;
 		i = CardServices(RequestIO, link->handle, &link->io);
 	    } else {
-		for (j = 0x300; j < 0x400; j += 0x20) {
-		    link->io.BasePort1 = j;
-		    link->io.BasePort2 = j+0x10;
+		for (j = 0; j < 0x400; j += 0x20) {
+		    link->io.BasePort1 = j ^ 0x300;
+		    link->io.BasePort2 = (j ^ 0x300) + 0x10;
 		    i = CardServices(RequestIO, link->handle, &link->io);
 		    if (i == CS_SUCCESS) break;
 		}
@@ -601,7 +596,7 @@ static void pcnet_config(dev_link_t *link)
     }
     if ((i != CS_SUCCESS) && !multi) {
 	link->conf.ConfigIndex = 1;
-	for (j = 0x300; j < 0x400; j += 0x20) {
+	for (j = 0x100; j < 0x400; j += 0x20) {
 	    link->io.BasePort1 = j;
 	    link->io.BasePort2 = j+0x10;
 	    i = CardServices(RequestIO, link->handle, &link->io);
@@ -714,17 +709,11 @@ static void pcnet_release(u_long arg)
     dev_link_t *link = (dev_link_t *)arg;
     pcnet_dev_t *info = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcnet_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "pcnet_release(0x%p)\n", link);
 
     if (link->open) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "pcnet_cs: release postponed, '%s' "
-		   "still open\n", info->node.dev_name);
-#endif
+	DEBUG(1, "pcnet_cs: release postponed, '%s' still open\n",
+	      info->node.dev_name);
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
@@ -762,18 +751,9 @@ static int pcnet_event(event_t event, int priority,
     dev_link_t *link = args->client_data;
     pcnet_dev_t *info = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcnet_event()\n");
-#endif
+    DEBUG(2, "pcnet_event(0x%06x)\n", event);
 
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "pcnet_cs: registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -842,10 +822,7 @@ static int pcnet_open(struct device *dev)
     pcnet_dev_t *info = (pcnet_dev_t *)dev;
     dev_link_t *link;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcnet_open('%s')\n", dev->name);
-#endif
+    DEBUG(2, "pcnet_open('%s')\n", dev->name);
 
     for (link = dev_list; link; link = link->next)
 	if (link->priv == dev) break;
@@ -868,10 +845,7 @@ static int pcnet_close(struct device *dev)
 {
     dev_link_t *link;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcnet_close('%s')\n", dev->name);
-#endif
+    DEBUG(2, "pcnet_close('%s')\n", dev->name);
 
     for (link = dev_list; link; link = link->next)
 	if (link->priv == dev) break;
@@ -926,6 +900,12 @@ static void pcnet_reset_8390(struct device *dev)
 
 static int set_config(struct device *dev, struct ifmap *map)
 {
+    pcnet_dev_t *info = (pcnet_dev_t *)dev;
+    if (!(info->flags & HAS_MISC_REG)) {
+	printk(KERN_NOTICE "%s: transceiver selection not "
+	       "implemented\n", dev->name);
+	return -EINVAL;
+    }
     if ((map->port != (u_char)(-1)) && (map->port != dev->if_port)) {
 	if ((map->port == 1) || (map->port == 2)) {
 	    dev->if_port = map->port;
@@ -1310,10 +1290,7 @@ config_failed:
 int init_module(void)
 {
     servinfo_t serv;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
 	printk(KERN_NOTICE "pcnet_cs: Card Services release "
@@ -1326,10 +1303,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "pcnet_cs: unloading\n");
-#endif
+    DEBUG(0, "pcnet_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL)
 	pcnet_detach(dev_list);
