@@ -1,5 +1,5 @@
 /*
- * pd6832.h 1.1 1996/11/20 07:49:24 (David Hinds)
+ * pd6832.h 1.2 1997/05/05 01:22:03 (David Hinds)
  */
 
 #ifndef _LINUX_PD6832_H
@@ -10,4 +10,7 @@
 /* Bridge Control Register */
 #define  PD6832_BCR_MGMT_IRQ_ENA	0x0800
 
+/* Socket Number Register */
+#define PD6832_SOCKET_NUMBER		0x004c	/* 8 bit */
+
 #endif /* _LINUX_PD6832_H */
