@@ -1,5 +1,5 @@
 /*
- * rl5c466.h 1.1 1996/11/20 07:49:24 (David Hinds)
+ * rl5c466.h 1.2 1997/05/05 01:22:17 (David Hinds)
  */
 
 #ifndef _LINUX_RL5C466_H
@@ -31,4 +31,14 @@
 #define  RL5C466_16CTL_IO_TIMING	0x0100
 #define  RL5C466_16CTL_MEM_TIMING	0x0200
 
+/* 16-bit IO and memory timing registers */
+#define RL5C466_16BIT_IO_0		0x0088	/* 16 bit */
+#define RL5C466_16BIT_MEM_0		0x0088	/* 16 bit */
+#define  RL5C466_SETUP_MASK		0x0007
+#define  RL5C466_SETUP_SHIFT		0
+#define  RL5C466_CMD_MASK		0x01f0
+#define  RL5C466_CMD_SHIFT		4
+#define  RL5C466_HOLD_MASK		0x1c00
+#define  RL5C466_HOLD_SHIFT		10
+
 #endif /* _LINUX_RL5C466_H */
