@@ -44,10 +44,10 @@ typedef struct resource_entry_t {
 } resource_entry_t;
 
 /* Memory resource database */
-resource_entry_t mem_db = { 0, 0, &mem_db };
+static resource_entry_t mem_db = { 0, 0, &mem_db };
 
 /* IO port resource database */
-resource_entry_t io_db = { 0, 0, &io_db };
+static resource_entry_t io_db = { 0, 0, &io_db };
 
 typedef struct irq_info_t {
     u_long		Attributes;
@@ -56,7 +56,7 @@ typedef struct irq_info_t {
 } irq_info_t;
 
 /* Table of IRQ assignments */
-irq_info_t irq_table[16] = { { 0, 0, 0 }, /* etc */ };
+static irq_info_t irq_table[16] = { { 0, 0, 0 }, /* etc */ };
 
 /*====================================================================*/
 
@@ -64,7 +64,7 @@ irq_info_t irq_table[16] = { { 0, 0, 0 }, /* etc */ };
 
 /* Should we probe resources for conflicts? */
 static int probe_io = 1;
-static int probe_mem = 0;
+static int probe_mem = 1;
 
 MODULE_PARM(probe_io, "i");
 MODULE_PARM(probe_mem, "i");
@@ -111,7 +111,7 @@ int register_mem_region(u_long base, u_long num, char *name)
     unsigned long flags;
     memory_entry_t *p, *entry;
 
-    entry = kmalloc(sizeof(memory_entry_t), GFP_KERNEL);
+    entry = kmalloc(sizeof(memory_entry_t), GFP_ATOMIC);
     entry->base = base;
     entry->num = num;
     entry->name = name;
@@ -255,8 +255,7 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
 		printk(" excluding");
 	    if (!bad)
 		bad = any = i;
-	}
-	else {
+	} else {
 	    if (bad) {
 		sub_interval(&io_db, bad, i-bad);
 		printk(" %#04x-%#04x", bad, i-1);
@@ -265,9 +264,10 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
 	}
     }
     if (bad) {
-	if ((num > 16) && (bad == base) && (i == base+num))
+	if ((num > 16) && (bad == base) && (i == base+num)) {
 	    printk(" nothing: probe failed.\n");
-	else {
+	    return;
+	} else {
 	    sub_interval(&io_db, bad, i-bad);
 	    printk(" %#04x-%#04x", bad, i-1);
 	}
@@ -276,40 +276,55 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
     printk(any ? "\n" : " clean.\n");
 }
 
-static void do_mem_probe(u_long base, u_long num)
+static int do_mem_probe(u_long base, u_long num, u_long step,
+			int (*is_valid)(u_long))
 {
-    u_long i, j, bad, any;
-    u_short match;
+    u_long i, bad, any;
 
-    printk(KERN_DEBUG "PCMCIA memory window probe:");
+    printk(KERN_INFO "cs: memory probe 0x%06lx-0x%06lx:",
+	   base, base+num-1);
     bad = any = 0;
-    for (i = base; i < base+num; i += 8192) {
-	if (check_mem_region(i, 8192) == 0) {
-	    match = *(u_short *)i;
-	    for (j = 0; j < 8192; j += 82)
-		if (match != *(u_short *)(i+j))
-		    break;
-	}
-	else j = 8192;
-	if (j < 8192) {
+    for (i = base; i < base+num; i += step) {
+	if (check_mem_region(i, step) != 0)
+	    continue;
+	if (!is_valid(i)) {
 	    if (!any)
 		printk(" excluding");
 	    if (!bad)
 		bad = any = i;
-	}
-	else {
+	} else {
 	    if (bad) {
-		sub_interval(&mem_db, bad, i-bad);
+		register_mem_region(bad, i-bad, "reserved");
 		printk(" %#05lx-%#05lx", bad, i-1);
 	    }
 	    bad = 0;
 	}
     }
     if (bad) {
-	sub_interval(&mem_db, bad, i-bad);
-	printk(" %#05lx-%#05lx", bad, i-1);
+	if ((bad == base) && (i == base+num)) {
+	    printk(" nothing: probe failed.\n");
+	    return -1;
+	} else {
+	    register_mem_region(bad, i-bad, "reserved");
+	    printk(" %#05lx-%#05lx", bad, i-1);
+	}
     }
     printk(any ? "\n" : " clean.\n");
+    return 0;
+}
+
+int validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long))
+{
+    if (!probe_mem)
+	return 0;
+    /* Make sure we have exactly one memory range resource */
+    if ((mem_db.next == &mem_db) || (mem_db.next->next != &mem_db))
+	return 0;
+    if (do_mem_probe(mem_db.next->base, mem_db.next->num, 4096,
+		     is_valid) == 0)
+	return 0;
+    return do_mem_probe(mem_db.next->base, mem_db.next->num, 8192,
+			do_cksum);
 }
 
 /*======================================================================
@@ -481,8 +496,6 @@ static int adjust_memory(adjust_t *adj)
     case ADD_MANAGED_RESOURCE:
 	if (add_interval(&mem_db, base, num) != 0)
 	    return CS_IN_USE;
-	if (probe_mem)
-	    do_mem_probe(base, num);
 	break;
     case REMOVE_MANAGED_RESOURCE:
 	sub_interval(&mem_db, base, num);
