@@ -32,8 +32,11 @@
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
-"serial_cs.c 1.81 1997/03/16 20:09:13 (David Hinds)";
+"serial_cs.c 1.85 1997/06/01 05:03:16 (David Hinds)";
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -56,7 +59,7 @@ MODULE_PARM(do_sound, "i");
 typedef struct {
     u_short	manfid;
     u_short	prodid;
-    int		multi;		/* 1 = multifunction, 2 = dual-port */
+    int		multi;		/* 1 = multifunction, > 1 = # ports */
     int		slave;		/* 1 = not compliant with MFC spec */
 } multi_id_t;
 
@@ -66,10 +69,13 @@ static multi_id_t multi_id[] = {
     { MANFID_LINKSYS, PRODID_LINKSYS_PCMLM28, 1, 1 },
     { MANFID_OSITECH, PRODID_OSITECH_JACK_144, 1, 1 },
     { MANFID_OSITECH, PRODID_OSITECH_JACK_288, 1, 1 },
+    { MANFID_OSITECH, PRODID_OSITECH_JACK_336, 1, 1 },
     { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_VARIOUS, 1, 1 },
     { MANFID_MEGAHERTZ, PRODID_MEGAHERTZ_EM3288, 1, 1 },
     { MANFID_QUATECH, PRODID_QUATECH_DUAL_RS232, 2, 0 },
-    { MANFID_SOCKET, PRODID_SOCKET_DUAL_RS232, 2, 0 }
+    { MANFID_QUATECH, PRODID_QUATECH_QUAD_RS232, 4, 0 },
+    { MANFID_SOCKET, PRODID_SOCKET_DUAL_RS232, 2, 0 },
+    { MANFID_INTEL, PRODID_INTEL_DUAL_RS232, 2, 0 }
 };
 #define MULTI_COUNT (sizeof(multi_id)/sizeof(multi_id_t))
 
@@ -78,8 +84,8 @@ typedef struct serial_info_t {
     int		multi;
     int		slave;
     int		manfid;
-    dev_node_t	node[2];
-    int		line[2];
+    dev_node_t	node[4];
+    int		line[4];
 } serial_info_t;
 
 static void serial_config(dev_link_t *link);
@@ -115,10 +121,7 @@ static dev_link_t *serial_attach(void)
     dev_link_t *link;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "serial_attach()\n");
-#endif
+    DEBUG(0, "serial_attach()\n");
 
     /* Create new serial device */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -178,10 +181,7 @@ static void serial_detach(dev_link_t *link)
     long flags;
     int ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "serial_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "serial_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -352,7 +352,7 @@ static int slave_config(dev_link_t *link)
 	return simple_config(link);
 }
 
-static int dual_config(dev_link_t *link)
+static int multi_config(dev_link_t *link)
 {
     client_handle_t handle = link->handle;
     serial_info_t *info = link->priv;
@@ -367,11 +367,13 @@ static int dual_config(dev_link_t *link)
     tuple.Attributes = 0;
     tuple.DesiredTuple = CISTPL_CFTABLE_ENTRY;
 
-    /* First, look for a generic 16-port window */
-    link->io.NumPorts1 = 16;
+    /* First, look for a generic full-sized window */
+    link->io.NumPorts1 = info->multi * 8;
     i = first_tuple(handle, &tuple, &parse);
     while (i == CS_SUCCESS) {
-	if ((cf->io.nwin == 1) && (cf->io.win[0].len == 16)) {
+	/* The quad port cards have bad CIS's, so just look for a
+	   window larger than 8 ports and assume it will be right */
+	if ((cf->io.nwin == 1) && (cf->io.win[0].len > 8)) {
 	    link->conf.ConfigIndex = cf->index;
 	    link->io.BasePort1 = cf->io.win[0].base;
 	    i = CardServices(RequestIO, link->handle, &link->io);
@@ -384,6 +386,7 @@ static int dual_config(dev_link_t *link)
     /* If that didn't work, look for two windows */
     if (i != CS_SUCCESS) {
 	link->io.NumPorts1 = link->io.NumPorts2 = 8;
+	info->multi = 2;
 	i = first_tuple(handle, &tuple, &parse);
 	while (i == CS_SUCCESS) {
 	    if (cf->io.nwin == 2) {
@@ -415,8 +418,9 @@ static int dual_config(dev_link_t *link)
     }
     
     setup_serial(info, link->io.BasePort1, link->irq.AssignedIRQ);
-    setup_serial(info, base2, link->irq.AssignedIRQ);
-
+    for (i = 0; i < info->multi-1; i++)
+	setup_serial(info, base2+(8*i), link->irq.AssignedIRQ);
+    
     return 0;
 }
 
@@ -445,10 +449,7 @@ void serial_config(dev_link_t *link)
     handle = link->handle;
     info = link->priv;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "serial_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "serial_config(0x%p)\n", link);
     
     tuple.TupleData = (cisdata_t *)buf;
     tuple.TupleOffset = 0; tuple.TupleDataMax = 255;
@@ -487,8 +488,8 @@ void serial_config(dev_link_t *link)
 	  (cf->io.win[1].len == 8))))
 	info->multi = 2;
 
-    if (info->multi == 2)
-	dual_config(link);
+    if (info->multi > 1)
+	multi_config(link);
     else if (info->slave)
 	slave_config(link);
     else
@@ -533,10 +534,7 @@ void serial_release(u_long arg)
     
     sti();
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "serial_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "serial_release(0x%p)\n", link);
 
     for (i = 0; i < info->ndev; i++) {
 	unregister_serial(info->line[i]);
@@ -566,18 +564,9 @@ static int serial_event(event_t event, int priority,
     dev_link_t *link = args->client_data;
     serial_info_t *info = link->priv;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "serial_event()\n");
-#endif
+    DEBUG(1, "serial_event(0x%06x)\n", event);
     
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "serial_cs: registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -613,10 +602,7 @@ static int serial_event(event_t event, int priority,
 int init_module(void)
 {
     servinfo_t serv;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
 	printk(KERN_NOTICE "serial_cs: Card Services release "
@@ -629,10 +615,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "serial_cs: unloading\n");
-#endif
+    DEBUG(0, "serial_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL)
 	serial_detach(dev_list);
