@@ -54,6 +54,9 @@ static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 static const char *version =
 "smc91c92_cs.c 0.09 1996/8/4 Donald Becker, becker@cesdis.gsfc.nasa.gov.\n";
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
+#else
+#define DEBUG(n, args...)
 #endif
 
 static char *if_names[] = { "Auto", "10baseT", "10base2"};
@@ -243,10 +246,7 @@ static void cs_error(int func, int ret)
 
 static int smc91c92_init(struct device *dev)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "%s: smc91c92_init called!\n", dev->name);
-#endif
+    DEBUG(0, "%s: smc91c92_init called!\n", dev->name);
     return 0;
 }
 
@@ -265,10 +265,7 @@ static dev_link_t *smc91c92_attach(void)
     struct device *dev;
     int ret;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_INFO "smc91c92_attach()\n");
-#endif
+    DEBUG(0, "smc91c92_attach()\n");
     
     /* Create new ethernet device */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -345,10 +342,7 @@ static void smc91c92_detach(dev_link_t *link)
     dev_link_t **linkp;
     long flags;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_INFO "smc91c92_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "smc91c92_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -498,8 +492,9 @@ static int mhz_mfc_config(dev_link_t *link)
     while (i == CS_SUCCESS) {
 	link->conf.ConfigIndex = cf->index;
 	link->io.BasePort2 = cf->io.win[0].base;
-	for (k = 0x300; k < 0x380; k += 0x20) {
-	    link->io.BasePort1 = k;
+	for (k = 0; k < 0x400; k += 0x10) {
+	    if (k & 0x80) continue;
+	    link->io.BasePort1 = k ^ 0x300;
 	    i = CardServices(RequestIO, link->handle, &link->io);
 	    if (i == CS_SUCCESS) break;
 	}
@@ -702,12 +697,9 @@ static void smc91c92_config(dev_link_t *link)
     tuple_t tuple;
     cisparse_t parse;
     u_short buf[32];
-    int i, j;
+    int i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 2)
-	printk(KERN_DEBUG "smc91c92_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "smc91c92_config(0x%p)\n", link);
     
     tuple.Attributes = 0;
     tuple.TupleData = (cisdata_t *)buf;
@@ -738,11 +730,7 @@ static void smc91c92_config(dev_link_t *link)
     else {
 	link->io.NumPorts1 = 16;
 	link->conf.ConfigIndex = 0x01;
-	for (j = 0x200; j < 0x400; j += 0x10) {
-	    link->io.BasePort1 = j;
-	    i = CardServices(RequestIO, link->handle, &link->io);
-	    if (i == CS_SUCCESS) break;
-	}
+	i = CardServices(RequestIO, link->handle, &link->io);
 	dev->base_addr = link->io.BasePort1;
     }
     CS_EXIT_TEST(i, RequestIO, config_failed);
@@ -769,6 +757,7 @@ static void smc91c92_config(dev_link_t *link)
     case MANFID_OSITECH:
 	i = osi_setup(link); break;
     case MANFID_SMC:
+    case MANFID_NEW_MEDIA:
 	i = smc_setup(link); break;
     case 0x128: /* For broken Megahertz cards */
     case MANFID_MEGAHERTZ:
@@ -815,17 +804,11 @@ static void smc91c92_release(u_long arg)
     dev_link_t *link = (dev_link_t *)arg;
     struct device *dev = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "smc91c92_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "smc91c92_release(0x%p)\n", link);
     
     if (link->open) {
-#ifdef PCMCIA_DEBUG
-	if (pc_debug)
-	    printk(KERN_DEBUG "smc91c92_cs: release postponed, "
-		   "'%s' still open\n", dev->name);
-#endif
+	DEBUG(1, "smc91c92_cs: release postponed, '%s' still open\n",
+	      dev->name);
 	link->state |= DEV_STALE_CONFIG;
 	return;
     }
@@ -863,18 +846,9 @@ static int smc91c92_event(event_t event, int priority,
     dev_link_t *link = args->client_data;
     struct device *dev = link->priv;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "smc91c92_event(0x%04x)\n", event);
-#endif
+    DEBUG(1, "smc91c92_event(0x%06x)\n", event);
 
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "smc91c92_cs: registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -939,9 +913,8 @@ static int smc91c92_open(struct device *dev)
     dev_link_t *link;
 
 #ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "%s: smc91c92_open(%p), ID/Window %4.4x.\n",
-	       dev->name, dev, inw(ioaddr + BANK_SELECT));
+    DEBUG(0, "%s: smc91c92_open(%p), ID/Window %4.4x.\n",
+	  dev->name, dev, inw(ioaddr + BANK_SELECT));
 
     if (pc_debug > 1) {
 	int i, window;
@@ -966,8 +939,10 @@ static int smc91c92_open(struct device *dev)
 	outw(0, ioaddr + CONTROL);
 	/* Wait 55 msec for startup */
 	busy_loop(HZ/18);
-	if (inb(ioaddr + BANK_SELECT + 1) != 0x33)
+	if (inb(ioaddr + BANK_SELECT + 1) != 0x33) {
+	    printk("smc91c92_cs: Yikes!  Bad chip signature!\n");
 	    return -ENODEV;
+	}
     }
     
     link->open++;
@@ -991,11 +966,8 @@ static int smc91c92_close(struct device *dev)
     int ioaddr = dev->base_addr;
     dev_link_t *link;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "%s: smc91c92_close(), status %4.4x.\n",
-	       dev->name, inw(ioaddr + BANK_SELECT));
-#endif
+    DEBUG(0, "%s: smc91c92_close(), status %4.4x.\n",
+	  dev->name, inw(ioaddr + BANK_SELECT));
     
     dev->tbusy = 1;
     dev->start = 0;
@@ -1075,11 +1047,8 @@ static void smc_hardware_send_packet( struct device * dev )
 	unsigned char *buf = skb->data;
 	int length = skb->len;	/* The chip will pad to ethernet min length. */
 
-#ifdef PCMCIA_DEBUG
-	if (pc_debug > 2)
-	    printk(KERN_DEBUG "%s: Trying to xmit packet of length %x.\n",
-		   dev->name, length);
-#endif
+	DEBUG(2, "%s: Trying to xmit packet of length %x.\n",
+	      dev->name, length);
 	
 #ifdef USE_32_BIT
 	outl((length+6) << 16, ioaddr + DATA_1);
@@ -1146,14 +1115,11 @@ static int smc_start_xmit(struct sk_buff *skb, struct device *dev)
 	return 0;
 #endif
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 2)
-	printk(KERN_DEBUG "%s: smc91c92_start_xmit(length = %ld) called,"
-	       " status %4.4x.\n", dev->name, skb->len, inw(ioaddr + 2));
-#endif
+    DEBUG(2, "%s: smc91c92_start_xmit(length = %d) called,"
+	  " status %4.4x.\n", dev->name, skb->len, inw(ioaddr + 2));
     
     /* Avoid timer-based retransmission conflicts. */
-    if (set_bit(0, (void*)&dev->tbusy) != 0) {
+    if (test_and_set_bit(0, (void*)&dev->tbusy) != 0) {
 	printk(KERN_ERR "%s: transmitter access conflict.\n", dev->name);
 	return 1;
     }
@@ -1252,11 +1218,8 @@ static void smc_eph_irq(struct device *dev)
     
     SMC_SELECT_BANK(0);
     ephs = inw(ioaddr + EPH);
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 2)
-	printk(KERN_NOTICE "%s: Ethernet protocol handler interrupt"
-	       ", status %4.4x.\n", dev->name, ephs);
-#endif
+    DEBUG(2, "%s: Ethernet protocol handler interrupt, status"
+	  " %4.4x.\n", dev->name, ephs);
     /* Could be a counter roll-over warning: update stats. */
     card_stats = inw( ioaddr + COUNTER );
     /* single collisions */
@@ -1309,14 +1272,14 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
     
 #ifdef PCMCIA_DEBUG
     if (dev->interrupt) {
-	printk(KERN_ERR "%s: re-entering the interrupt handler.\n", dev->name);
+	printk(KERN_ERR "%s: re-entering the interrupt handler.\n",
+	       dev->name);
 	return;
     }
     dev->interrupt = 1;
 
-    if (pc_debug > 5)
-	printk(KERN_DEBUG "%s: SMC91c92 interrupt %d at %#x.\n",
-	       dev->name, irq, ioaddr);
+    DEBUG(3, "%s: SMC91c92 interrupt %d at %#x.\n", dev->name,
+	  irq, ioaddr);
 #endif
     
     lp = (struct smc_private *)dev->priv;
@@ -1325,9 +1288,9 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
 	/* The device does not exist -- the card could be off-line, or
 	   maybe it has been ejected. */
 #ifdef PCMCIA_DEBUG
-	if (dev->start && (pc_debug > 4))
-	    printk(KERN_ERR "%s: SMC91c92 interrupt %d for non-existent"
-		   "/ejected device.\n", dev->name, irq);
+	if (dev->start)
+	    DEBUG(1, "%s: SMC91c92 interrupt %d for non-existent"
+		  "/ejected device.\n", dev->name, irq);
 	dev->interrupt = 0;
 #endif
 	goto irq_done;
@@ -1341,11 +1304,8 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
     
     do { /* read the status flag, and mask it */
 	int status = inb( ioaddr + INTERRUPT );
-#ifdef PCMCIA_DEBUG
-	if (pc_debug > 6)
-	    printk(KERN_DEBUG "%s: Status is %#2.2x (mask %#2.2x).\n",
-		   dev->name, status, mask);
-#endif
+	DEBUG(3, "%s: Status is %#2.2x (mask %#2.2x).\n", dev->name,
+	      status, mask);
 	if ((status & mask) == 0 )
 	    break;
 	
@@ -1385,11 +1345,8 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
 	    smc_eph_irq(dev);
     } while ( --bogus_cnt);
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 6)
-	printk(KERN_DEBUG "  Restoring saved registers mask %2.2x bank"
-	       " %d pointer %d.\n", mask, saved_bank, saved_pointer);
-#endif
+    DEBUG(3, "  Restoring saved registers mask %2.2x bank"
+	  " %d pointer %d.\n", mask, saved_bank, saved_pointer);
     
     /* restore state register */
     outb( mask, ioaddr + INT_MASK );
@@ -1399,9 +1356,7 @@ static void smc_interrupt IRQ(int irq, void *instance, struct pt_regs *regs)
 
 #ifdef PCMCIA_DEBUG
     dev->interrupt = 0;
-    if (pc_debug > 5)
-	printk(KERN_DEBUG "%s: Exiting interrupt IRQ%d.\n",
-	       dev->name, irq);
+    DEBUG(3, "%s: Exiting interrupt IRQ%d.\n", dev->name, irq);
 #endif
 
 irq_done:
@@ -1442,11 +1397,8 @@ static void smc_rx(struct device *dev)
     rx_status = inw(ioaddr + DATA_1);
     packet_length = inw(ioaddr + DATA_1) & 0x07ff;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 4)
-	printk(KERN_DEBUG "%s: Receive status %4.4x length %4d.\n",
-	       dev->name, rx_status, packet_length);
-#endif
+    DEBUG(2, "%s: Receive status %4.4x length %4d.\n",
+	  dev->name, rx_status, packet_length);
     
     if ( !(rx_status & RS_ERRORS )) {		
 	/* do stuff to make a new packet */
@@ -1456,11 +1408,7 @@ static void smc_rx(struct device *dev)
 	skb = ALLOC_SKB(packet_length);
 	
 	if ( skb == NULL ) {
-#ifdef PCMCIA_DEBUG
-	    if (pc_debug)
-		printk(KERN_NOTICE "%s: Low memory, packet dropped.\n",
-		       dev->name);
-#endif
+	    DEBUG(1, "%s: Low memory, packet dropped.\n", dev->name);
 	    lp->stats.rx_dropped++;
 	    outw( MC_RELEASE, ioaddr + MMU_CMD );
 	    return;
@@ -1627,10 +1575,7 @@ static void smc_reset(struct device *dev)
     struct smc_private *lp = dev->priv;
     int i;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "%s: smc91c92 reset called.\n", dev->name);
-#endif
+    DEBUG(0, "%s: smc91c92 reset called.\n", dev->name);
     
     /* The first interaction must be a write to bring the chip out
        of sleep mode. */
@@ -1682,10 +1627,7 @@ static void smc_reset(struct device *dev)
 int init_module(void)
 {
     servinfo_t serv;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
 	printk(KERN_ERR
@@ -1698,10 +1640,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "smc91c92_cs: unloading\n");
-#endif
+    DEBUG(0, "smc91c92_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL)
 	smc91c92_detach(dev_list);
