@@ -9,7 +9,7 @@
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
 
-#include <sys/types.h>
+#include <linux/types.h>
 #include <linux/fcntl.h>
 #include <linux/string.h>
 
@@ -33,8 +33,12 @@
 
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
+MODULE_PARM(pc_debug, "i");
 static const char *version =
-"tcic.c 1.80 1997/03/16 20:07:08 (David Hinds)";
+"tcic.c 1.82 1997/06/01 05:03:16 (David Hinds)";
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
+#else
+#define DEBUG(n, args...)
 #endif
 
 /*====================================================================*/
@@ -333,10 +337,7 @@ int tcic_init(void)
     int i, sock;
     u_long mask, scan;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_INFO "%s\n", version);
-#endif
+    DEBUG(0, "%s\n", version);
     
     printk(KERN_INFO "Databook TCIC-2 PCMCIA probe: ");
     sock = 0;
@@ -496,10 +497,7 @@ static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
     else
 	active = 1;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: tcic_interrupt()\n");
-#endif
+    DEBUG(2, "tcic: tcic_interrupt()\n");
     
     for (i = 0; i < sockets; i++) {
 	psock = socket_table[i].psock;
@@ -536,19 +534,13 @@ static void tcic_interrupt IRQ(int irq, void *dev, struct pt_regs *regs)
     }
     active = 0;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: done\n");
-#endif
+    DEBUG(2, "tcic: interrupt done\n");
     
 } /* tcic_interrupt */
 
 void tcic_timer(u_long data)
 {
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: tcic_timer()\n");
-#endif
+    DEBUG(2, "tcic: tcic_timer()\n");
     timer_pending = 0;
     tcic_interrupt IRQ(0, NULL, NULL);
 } /* tcic_timer */
@@ -592,11 +584,7 @@ static int tcic_get_status(u_short lsock, u_int *value)
     reg = tcic_getb(TCIC_PWR);
     if (reg & (TCIC_PWR_VCC(psock)|TCIC_PWR_VPP(psock)))
 	*value |= SS_POWERON;
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: GetStatus(%d) = %#2.2x\n",
-	       lsock, *value);
-#endif
+    DEBUG(1, "tcic: GetStatus(%d) = %#2.2x\n", lsock, *value);
     return 0;
 } /* tcic_get_status */
   
@@ -655,13 +643,9 @@ static int tcic_get_socket(u_short lsock, socket_state_t *state)
 	state->csc_mask |= (scf2 & TCIC_SCF2_MRDY) ? 0 : SS_READY;
     }
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: GetSocket(%d) = flags %#3.3lx, "
-	       "Vcc %d, Vpp %d, io_irq %d, csc_mask %#2.2lx\n", lsock,
-	       state->flags, state->Vcc, state->Vpp, state->io_irq,
-	       state->csc_mask);
-#endif
+    DEBUG(1, "tcic: GetSocket(%d) = flags %#3.3lx, Vcc %d, Vpp %d, "
+	  "io_irq %d, csc_mask %#2.2lx\n", lsock, state->flags,
+	  state->Vcc, state->Vpp, state->io_irq, state->csc_mask);
     return 0;
 } /* tcic_get_socket */
 
@@ -673,13 +657,9 @@ static int tcic_set_socket(u_short lsock, socket_state_t *state)
     u_char reg;
     u_short scf1, scf2;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: SetSocket(%d, flags %#3.3lx, "
-	       "Vcc %d, Vpp %d, io_irq %d, csc_mask %#2.2lx)\n",
-	       lsock, state->flags, state->Vcc, state->Vpp,
-	       state->io_irq, state->csc_mask);
-#endif
+    DEBUG(1, "tcic: SetSocket(%d, flags %#3.3lx, Vcc %d, Vpp %d, "
+	  "io_irq %d, csc_mask %#2.2lx)\n", lsock, state->flags,
+	  state->Vcc, state->Vpp, state->io_irq, state->csc_mask);
     tcic_setw(TCIC_ADDR+2, (psock << TCIC_SS_SHFT) | TCIC_ADR2_INDREG);
 
     reg = tcic_getb(TCIC_PWR);
@@ -782,12 +762,9 @@ static int tcic_get_io_map(u_short lsock, struct pcmcia_io_map *io)
     default:
 	break;
     }
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: GetIOMap(%d, %d) = %#2.2x, %d ns, "
-	       "%#4.4x-%#4.4x\n", lsock, io->map, io->flags,
-	       io->speed, io->start, io->stop);
-#endif
+    DEBUG(1, "tcic: GetIOMap(%d, %d) = %#2.2x, %d ns, "
+	  "%#4.4x-%#4.4x\n", lsock, io->map, io->flags,
+	  io->speed, io->start, io->stop);
     return 0;
 } /* tcic_get_io_map */
 
@@ -799,12 +776,9 @@ static int tcic_set_io_map(u_short lsock, struct pcmcia_io_map *io)
     u_long addr;
     u_short base, len, ioctl;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: SetIOMap(%d, %d, %#2.2x, %d ns, "
-	       "%#4.4x-%#4.4x)\n", lsock, io->map, io->flags,
-	       io->speed, io->start, io->stop);
-#endif
+    DEBUG(1, "tcic: SetIOMap(%d, %d, %#2.2x, %d ns, "
+	  "%#4.4x-%#4.4x)\n", lsock, io->map, io->flags,
+	  io->speed, io->start, io->stop);
     if ((io->map > 1) || (io->start > 0xffff) || (io->stop > 0xffff) ||
 	(io->stop < io->start)) return -EINVAL;
     tcic_setw(TCIC_ADDR+2, TCIC_ADR2_INDREG | (psock << TCIC_SS_SHFT));
@@ -869,13 +843,9 @@ static int tcic_get_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
     mem->flags |= (ctl & TCIC_MCTL_WP) ? MAP_WRPROT : 0;
     mem->speed = to_ns(ctl & TCIC_MCTL_WSCNT_MASK);
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: GetMemMap(%d, %d) = %#2.2x, %d ns, "
-	       "%#5.5lx-%#5.5lx, %#5.5lx\n", lsock, mem->map,
-	       mem->flags, mem->speed, mem->sys_start, mem->sys_stop,
-	       mem->card_start);
-#endif
+    DEBUG(1, "tcic: GetMemMap(%d, %d) = %#2.2x, %d ns, "
+	  "%#5.5lx-%#5.5lx, %#5.5lx\n", lsock, mem->map, mem->flags,
+	  mem->speed, mem->sys_start, mem->sys_stop, mem->card_start);
     return 0;
 } /* tcic_get_mem_map */
 
@@ -887,13 +857,9 @@ static int tcic_set_mem_map(u_short lsock, struct pcmcia_mem_map *mem)
     u_short addr, ctl;
     u_long base, len, mmap;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug > 1)
-	printk(KERN_DEBUG "tcic: SetMemMap(%d, %d, %#2.2x, %d ns, "
-	       "%#5.5lx-%#5.5lx, %#5.5lx)\n", lsock, mem->map,
-	       mem->flags, mem->speed, mem->sys_start, mem->sys_stop,
-	       mem->card_start);
-#endif
+    DEBUG(1, "tcic: SetMemMap(%d, %d, %#2.2x, %d ns, "
+	  "%#5.5lx-%#5.5lx, %#5.5lx)\n", lsock, mem->map, mem->flags,
+	  mem->speed, mem->sys_start, mem->sys_stop, mem->card_start);
     if ((mem->map > 3) || (mem->card_start > 0x3ffffff) ||
 	(mem->sys_start > 0xffffff) || (mem->sys_stop > 0xffffff) ||
 	(mem->sys_start > mem->sys_stop) || (mem->speed > 1000))
@@ -949,10 +915,7 @@ static int tcic_service(u_int lsock, u_int cmd, void *arg)
 {
     int err;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug) 
-	printk(KERN_DEBUG "tcic_ioctl(%d, %d, 0x%p)\n", lsock, cmd, arg);
-#endif
+    DEBUG(2, "tcic_ioctl(%d, %d, 0x%p)\n", lsock, cmd, arg);
 
     if (cmd < NFUNC)
 	err = service_table[cmd](lsock, arg);
