@@ -18,13 +18,7 @@
  * This driver provide a Linux interface to the Wavelan Pcmcia hardware
  * The Wavelan is a product of Lucent ("http://wavelan.netland.nl/").
  * This division was formerly part of NCR and then AT&T.
- * Wavelan are also distributed by DEC (RoamAbout), Digital Ocean and
- * Aironet (Arlan). If you have one of those product, you will need to
- * make some changes below...
- *
- * This driver is still a beta software. A lot of bugs have been corrected,
- * a lot of functionalities are implemented, the whole appear pretty stable,
- * but there is still some area of improvement (encryption, pcmcia events...).
+ * Wavelan are also distributed by DEC (RoamAbout DS)...
  *
  * To know how to use this driver, read the PCMCIA HOWTO.
  * If you want to exploit the many other fonctionalities, look comments
@@ -35,6 +29,12 @@
 
 /* ------------------------ SPECIFIC NOTES ------------------------ */
 /*
+ * wavelan_cs.o is darn too big
+ * -------------------------
+ *	That's true ! There is a very simple way to reduce the driver
+ *	object by 33% (yes !). Comment out the following line :
+ *		#include <linux/wireless.h>
+ *
  * MAC address and hardware detection :
  * ----------------------------------
  *	The detection code of the wavelan chech that the first 3
@@ -63,14 +63,12 @@
  * caracteristics of the hardware in a standard way and support for
  * applications for taking advantage of it (like Mobile IP).
  *
- * By default, these wireless extensions are disabled, because they
- * need a patch to the Linux Kernel. This simple patch may be found
- * with the driver + some utilities to access those wireless
- * extensions (iwconfig...). Hopefully, those wireless extensions will
- * make their way in the kernel someday.
+ * You will need to enable the CONFIG_NET_RADIO define in the kernel
+ * configuration to enable the wireless extensions (this is the one
+ * giving access to the radio network device choice).
  *
- * You also will need to enable the CONFIG_NET_RADIO in the kernel
- * configuration to enable the wireless extensions.
+ * It might also be a good idea as well to fetch the wireless tools to
+ * configure the device and play a bit.
  */
 
 /* ---------------------------- FILES ---------------------------- */
@@ -150,7 +148,7 @@
  *
  *    This software was originally developed under Linux 1.2.3
  *	(Slackware 2.0 distribution).
- *	and then under Linux 2.0.x (Debian 1.1 - pcmcia 2.8.18-23) with
+ *    And then under Linux 2.0.x (Debian 1.1 - pcmcia 2.8.18-23) with
  *	HP OmniBook 4000 & 5500.
  *
  *    It is based on other device drivers and information either written
@@ -161,6 +159,7 @@
  *	Jim Binkley <jrb@cs.pdx.edu>,
  *	Loeke Brederveld <lbrederv@wavelan.com>,
  *	Allan Creighton (allanc@cs.su.oz.au),
+ *	Brent Elphick <belphick@uwaterloo.ca>,
  *	Joe Finney <joe@comp.lancs.ac.uk>,
  *	Matthew Geier (matthew@cs.su.oz.au),
  *	Remo di Giovanni (remo@cs.su.oz.au),
@@ -178,6 +177,7 @@
  *	John Rosenberg (johnr@cs.su.oz.au),
  *	George Rossi (george@phm.gov.au),
  *	Arthur Scott (arthur@cs.su.oz.au),
+ *	Stanislav Sinyagin <stas@isf.ru>
  *	Peter Storey,
  *	Jean Tourrilhes <jt@hplb.hpl.hp.com>,
  *	Girish Welling (welling@paul.rutgers.edu)
@@ -270,15 +270,29 @@
  *	- Small bug in debug code (probably not the last one...)
  *	- 1.2.13 support (thanks to Clark Woodworth)
  *
- * Changes made in this release (2.9.2b) :
- * -------------------------------------
+ * Changes made for release in 2.9.2b :
+ * ----------------------------------
  *	- Level threshold is now a standard wireless extension (version 4 !)
  *	- modules parameters types for kernel > 2.1.17
  *	- updated man page
+ *	- Others cleanup from David Hinds
+ *
+ * Changes made for release in 2.9.5 :
+ * ---------------------------------
+ *	- byte count stats (courtesy of David Hinds)
+ *	- Remove dev_tint stuff (courtesy of David Hinds)
+ *	- Others cleanup from David Hinds
+ *	- Encryption setting from Brent Elphick (thanks a lot !)
+ *	- 'base' to 'u_long' for the Alpha (thanks to Stanislav Sinyagin)
+ *
+ * Changes made for release in 2.9.6 :
+ * ---------------------------------
+ *	- fix bug : no longuer disable watchdog in case of bogus interrupt
+ *	- increase timeout in config code for picky hardware
+ *	- mask unused bits in status (Wireless Extensions)
  *
  * Wishes & dreams :
  * ---------------
- *	- Encryption stuff
  *	- Roaming
  */
 
@@ -310,10 +324,6 @@
 
 #ifdef HAS_WIRELESS_EXTENSIONS
 #include <linux/wireless.h>		/* Wireless extensions */
-#if WIRELESS_EXT < 4
-#warning "You need to upgrade your version of Wireless Extensions."
-#undef WIRELESS_EXT
-#endif
 #endif
 
 /* Pcmcia headers that we need */
@@ -344,7 +354,7 @@
 #define DEBUG_TX_ERROR		/* unexpected conditions */
 #undef DEBUG_RX_TRACE		/* Transmission calls */
 #undef DEBUG_RX_INFO		/* Header of the transmited packet */
-#define DEBUG_RX_ERROR		/* unexpected conditions */
+#undef DEBUG_RX_ERROR		/* unexpected conditions */
 #undef DEBUG_PACKET_DUMP	16	/* Dump packet on the screen */
 #undef DEBUG_IOCTL_TRACE	/* Misc call by Linux */
 #undef DEBUG_IOCTL_INFO		/* Various debug info */
@@ -376,7 +386,7 @@
 /************************ CONSTANTS & MACROS ************************/
 
 #ifdef DEBUG_VERSION_SHOW
-static const char *version = "wavelan_cs.c : v15 (wireless extensions) 12/2/97\n";
+static const char *version = "wavelan_cs.c : v17 (wireless extensions) 20/5/97\n";
 #endif
 
 /* Watchdog temporisation */
@@ -462,11 +472,11 @@ static inline void
 		 int);
 /* ----------------- MODEM MANAGEMENT SUBROUTINES ----------------- */
 static inline u_char		/* data */
-	hasr_read(u_short);	/* Read the host interface : base address */
+	hasr_read(u_long);	/* Read the host interface : base address */
 static inline void
-	hacr_write(u_short,	/* Write to host interface : base address */
+	hacr_write(u_long,	/* Write to host interface : base address */
 		   u_char),	/* data */
-	hacr_write_slow(u_short,
+	hacr_write_slow(u_long,
 		   u_char);
 static void
 	psa_read(device *,	/* Read the Parameter Storage Area */
@@ -478,33 +488,29 @@ static void
 		  u_char *,	/* Buffer in memory */
 		  int);		/* Length of buffer */
 static inline void
-	mmc_out(u_short,	/* Write 1 byte to the Modem Manag Control */
+	mmc_out(u_long,		/* Write 1 byte to the Modem Manag Control */
 		u_short,
 		u_char),
-	mmc_write(u_short,	/* Write n bytes to the MMC */
+	mmc_write(u_long,	/* Write n bytes to the MMC */
 		  u_char,
 		  u_char *,
 		  int);
 static inline u_char		/* Read 1 byte from the MMC */
-	mmc_in(u_short,
+	mmc_in(u_long,
 	       u_short);
 static inline void
-	mmc_read(u_short,	/* Read n bytes from the MMC */
+	mmc_read(u_long,	/* Read n bytes from the MMC */
 		 u_char,
 		 u_char *,
 		 int),
-	fee_wait(u_short,	/* Wait for frequency EEprom : base address */
+	fee_wait(u_long,	/* Wait for frequency EEprom : base address */
 		 int,		/* Base delay to wait for */
 		 int);		/* Number of time to wait */
 static void
-	fee_read(u_short,	/* Read the frequency EEprom : base address */
+	fee_read(u_long,	/* Read the frequency EEprom : base address */
 		 u_short,	/* destination offset */
 		 u_short *,	/* data buffer */
-		 int),		/* number of registers */
-	fee_write(u_short,	/* Write to frequency EEprom : base address */
-		  u_short,	/* destination offset */
-		  u_short *,	/* data buffer */
-		  int);		/* number of registers */
+		 int);		/* number of registers */
 /* ---------------------- I82593 SUBROUTINES ----------------------- */
 static int
 	wv_82593_cmd(device *,	/* synchronously send a command to i82593 */ 
