@@ -11,7 +11,6 @@
 
 /* To have statistics (just packets sent) define this */
 #undef XIRCNW_STATS
-#define PCMCIA_DEBUG 0
 
 #include <pcmcia/config.h>
 #include <pcmcia/k_compat.h>
@@ -118,8 +117,6 @@ static const unsigned int txConfEUD    = 0x10; /* Enable Uni-Data packets */
 static const unsigned int txConfKey    = 0x02; /* Scramble data packets */
 static const unsigned int txConfLoop   = 0x01; /* Loopback mode */
 
-int xinw_debug = 0;
-
 /*
    All the PCMCIA modules use PCMCIA_DEBUG to control debugging.  If
    you do not define PCMCIA_DEBUG at all, all the debug code will be
@@ -130,8 +127,11 @@ int xinw_debug = 0;
 #ifdef PCMCIA_DEBUG
 static int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
+#define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static char *version =
 "xircnw_cs.c 0.2.0 1996/05/22 15:05:51 (John Markus Bjrndalen)\n";
+#else
+#define DEBUG(n, args...)
 #endif
 
 static dev_info_t dev_info = "xircnw_cs";
@@ -313,10 +313,7 @@ static dev_link_t *xircnw_attach(void)
     struct device *dev;
     int ret;
     
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "xircnw_attach()\n");
-#endif
+    DEBUG(0, "xircnw_attach()\n");
 
     /* Initialize the dev_link_t structure */
     link = kmalloc(sizeof(struct dev_link_t), GFP_KERNEL);
@@ -399,10 +396,7 @@ static void xircnw_detach(dev_link_t *link)
 {
     dev_link_t **linkp;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "xircnw_detach(0x%p)\n", link);
-#endif
+    DEBUG(0, "xircnw_detach(0x%p)\n", link);
     
     /* Locate device structure */
     for (linkp = &dev_list; *linkp; linkp = &(*linkp)->next)
@@ -419,9 +413,8 @@ static void xircnw_detach(dev_link_t *link)
     if (link->state & DEV_CONFIG) {
 	xircnw_release((u_long) link);
 	if (link->state & DEV_STALE_CONFIG) {
-	    if (xinw_debug)
-	        printk(KERN_DEBUG "xircnw_cs: detach postponed, '%s' still locked\n",
-	               link->dev->dev_name);
+	    DEBUG(1, "xircnw_cs: detach postponed, '%s' still "
+		  "locked\n", link->dev->dev_name);
 	    link->state |= DEV_STALE_LINK;
 	    return;
 	}
@@ -469,10 +462,7 @@ static void xircnw_config(dev_link_t *link)
     handle = link->handle;
     dev = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug) 
-	printk(KERN_DEBUG "xircnw_config(0x%p)\n", link);
-#endif
+    DEBUG(0, "xircnw_config(0x%p)\n", link);
 
     /*
        This reads the card's CONFIG tuple to find its configuration
@@ -509,9 +499,8 @@ static void xircnw_config(dev_link_t *link)
 	 *  If you want, you can also read the card's config table to
 	 *  pick addresses -- see the serial driver for an example.
 	 */
-	for (j = 0x300; j < 0x400; j += 0x20) {
-	    link->io.BasePort1 = j;
-/*	    link->io.BasePort2 = j+0x10; */
+	for (j = 0; j < 0x400; j += 0x20) {
+	    link->io.BasePort1 = j ^ 0x300;
 	    i = CardServices(RequestIO, link->handle, &link->io);
 	    if (i == CS_SUCCESS) break;
 	}
@@ -546,8 +535,7 @@ static void xircnw_config(dev_link_t *link)
 	 *  device needs several windows, you'll need to keep track of
 	 *  the handles in your private data structure, link->priv.
 	 */
-	if (xinw_debug)
-	    printk(KERN_DEBUG "Setting mem speed of %d\n", mem_speed);
+	DEBUG(1, "Setting mem speed of %d\n", mem_speed);
 	req.Attributes = WIN_DATA_WIDTH_8|WIN_MEMORY_TYPE_CM|WIN_ENABLE;
 	req.Base = NULL;
 	req.Size = 0x8000;
@@ -597,16 +585,13 @@ static void xircnw_config(dev_link_t *link)
     for (i = 0; i < 6; i++) 
 	dev->dev_addr[i] = readb(ramBase + XIRCOM_EREG_PA + i);
 
-    if (xinw_debug) {
-        printk(KERN_DEBUG "xircnw_config: %s irq %d, io %lx\n", 
-	       dev->name, dev->irq, dev->base_addr);
-        printk(KERN_DEBUG "xircnw_config: memory base is %lx  id %c%c\n",
-	       (u_long) ramBase, 
-	       readb(ramBase+XIRCOM_EREG_NI),
-	       readb(ramBase+XIRCOM_EREG_NI+1));
-    }
+    DEBUG(1, "xircnw_config: %s irq %d, io %lx\n", 
+	  dev->name, dev->irq, dev->base_addr);
+    DEBUG(1, "xircnw_config: memory base is %lx  id %c%c\n",
+	  (u_long) ramBase, readb(ramBase+XIRCOM_EREG_NI),
+	  readb(ramBase+XIRCOM_EREG_NI+1));
 
-    printk(KERN_DEBUG "xircnw_config: phys addr: %02x:%02x:%02x:%02x:%02x:%02x\n", 
+    printk(KERN_INFO "xircnw_config: phys addr: %02x:%02x:%02x:%02x:%02x:%02x\n", 
 	   readb(ramBase+XIRCOM_EREG_PA+0),
 	   readb(ramBase+XIRCOM_EREG_PA+1),
 	   readb(ramBase+XIRCOM_EREG_PA+2),
@@ -635,10 +620,7 @@ static void xircnw_release(u_long arg)
     dev_link_t *link = (dev_link_t *)arg;
     struct device *dev = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "xircnw_release(0x%p)\n", link);
-#endif
+    DEBUG(0, "xircnw_release(0x%p)\n", link);
 
     /*
        If the device is currently in use, we won't release until it
@@ -692,18 +674,9 @@ static int xircnw_event(event_t event, int priority,
     dev_link_t *link = args->client_data;
     struct device *dev = link->priv;
 
-#ifdef PCMCIA_DEBUG
-    if (pc_debug)
-	printk(KERN_DEBUG "xircnw_event()\n");
-#endif
+    DEBUG(1, "xircnw_event(0x%06x)\n", event);
     
     switch (event) {
-#ifdef PCMCIA_DEBUG
-    case CS_EVENT_REGISTRATION_COMPLETE:
-	if (pc_debug)
-	    printk(KERN_DEBUG "xircnw_cs: registration complete\n");
-	break;
-#endif
     case CS_EVENT_CARD_REMOVAL:
 	link->state &= ~DEV_PRESENT;
 	if (link->state & DEV_CONFIG) {
@@ -859,13 +832,11 @@ static int xircnw_hw_xmit(unsigned char* data, int len, struct device* dev)
     priv->stats.tx_bytes += len;
 #endif
 
-    if (xinw_debug > 4) {
-	printk(KERN_DEBUG "Transmitting with SPCQ %x SPU %x LIF %x ISPLQ %x\n",
-	       readb(ramBase + XIRCOM_EREG_SPCQ),
-	       readb(ramBase + XIRCOM_EREG_SPU),
-	       readb(ramBase + XIRCOM_EREG_LIF),
-	       readb(ramBase + XIRCOM_EREG_ISPLQ));
-    }
+    DEBUG(3, "Transmitting with SPCQ %x SPU %x LIF %x ISPLQ %x\n",
+	  readb(ramBase + XIRCOM_EREG_SPCQ),
+	  readb(ramBase + XIRCOM_EREG_SPU),
+	  readb(ramBase + XIRCOM_EREG_LIF),
+	  readb(ramBase + XIRCOM_EREG_ISPLQ));
 
     wait_WOC(iobase);
 
@@ -876,9 +847,8 @@ static int xircnw_hw_xmit(unsigned char* data, int len, struct device* dev)
     nextBuff   = get_uint16(ramBase + TxFreeList);
     curBuff = TxFreeList;
 
-    if (xinw_debug > 4) 
-	printk(KERN_DEBUG "TxFreeList %x, MaxData %x, DataOffset %x, nextBuff %x\n",
-	       TxFreeList, MaxData, DataOffset, nextBuff);
+    DEBUG(3, "TxFreeList %x, MaxData %x, DataOffset %x, nextBuff %x\n",
+	  TxFreeList, MaxData, DataOffset, nextBuff);
 
 
     if (len <= MaxData)
@@ -923,8 +893,7 @@ static int xinw_start_xmit(struct sk_buff *skb, struct device *dev)
 	 * on a stopped card. 
 	 */
        
-	if (xinw_debug > 0)
-	   printk(KERN_DEBUG "%s timed out.\n", dev->name);
+	DEBUG(0, "%s timed out.\n", dev->name);
         xircnw_reset(dev); 
 	dev->trans_start = jiffies;
 	dev->tbusy = 0;
@@ -942,7 +911,7 @@ static int xinw_start_xmit(struct sk_buff *skb, struct device *dev)
     /* Block a timer-based transmit from overlapping. This could 
        better be done with atomic_swap(1, dev->tbusy, but set_bit()
        works as well */
-    if (set_bit(0, (void*)&dev->tbusy) != 0) {
+    if (test_and_set_bit(0, (void*)&dev->tbusy) != 0) {
 	printk("%s: Transmitter access conflict.\n", dev->name);
     } else {
 	short length = ETH_ZLEN < skb->len ? skb->len : ETH_ZLEN;
@@ -1004,9 +973,8 @@ static void xinw_interrupt IRQ(int irq, void* dev_id, struct pt_regs *regs)
        
 	if ( ! (link->state & DEV_PRESENT) ||
 	     link->state & DEV_SUSPEND ) {
-	    if (xinw_debug)
-		printk("xinw_interupt: Interrupt with status 0x%x from removed or suspended card!\n",
-		       status);
+	    DEBUG(1, "xinw_interupt: Interrupt with status 0x%x "
+		  "from removed or suspended card!\n", status);
 	    break;
 	}
        
@@ -1051,9 +1019,8 @@ static void xinw_interrupt IRQ(int irq, void* dev_id, struct pt_regs *regs)
 	    int txStatus;
 	    wait_WOC(iobase);
 	    txStatus = readb(ramBase + XIRCOM_EREG_TSER);
-	    if (xinw_debug > 4) 
-		printk(KERN_DEBUG "Transmit done. TSER = %x id %x\n", 
-		       txStatus, readb(ramBase + XIRCOM_EREG_TSER + 1));
+	    DEBUG(3, "Transmit done. TSER = %x id %x\n", 
+		  txStatus, readb(ramBase + XIRCOM_EREG_TSER + 1));
 
 	    if (txStatus & 0x20) {
 		/* Transmitting was okay, clear bits */
@@ -1076,9 +1043,8 @@ static void xinw_interrupt IRQ(int irq, void* dev_id, struct pt_regs *regs)
 		 *      TxGU and TxNOAP is set. (Those are the only ones
 		 *      to set TxErr).
 		 */
-		if (xinw_debug > 4) 
-		    printk(KERN_DEBUG "xircnw_interrupt: TxDN with error status %x\n", 
-			   txStatus);
+		DEBUG(3, "xircnw_interrupt: TxDN with error status %x\n", 
+		      txStatus);
 
 		/* Clear out TxGU, TxNOAP, TxErr and TxTrys */
 		wait_WOC(iobase);
@@ -1087,18 +1053,16 @@ static void xinw_interrupt IRQ(int irq, void* dev_id, struct pt_regs *regs)
 	    }
 	    /* wait_WOC(iobase); */
 	    
-	    if (xinw_debug > 4) 
-		printk(KERN_DEBUG "New status is TSER %x ASR %x\n",
-		       readb(ramBase + XIRCOM_EREG_TSER),
-		       inb(iobase + XIRCOM_REG_ASR));
+	    DEBUG(3, "New status is TSER %x ASR %x\n",
+		  readb(ramBase + XIRCOM_EREG_TSER),
+		  inb(iobase + XIRCOM_REG_ASR));
 
 	    dev->tbusy = 0;
 	    mark_bh(NET_BH);
 	}
 	/* TxBA, this would trigger on all error packets received */
 /*	if (status & 0x01) {
-	    if (xinw_debug > 3) 
-		printk(KERN_DEBUG "Transmit buffers available, %x\n", status); 
+	    DEBUG(3, "Transmit buffers available, %x\n", status); 
 	} */
 	   
     }
@@ -1119,20 +1083,19 @@ static struct net_device_stats *xinw_get_stats(struct device *dev)
     sti();
     restore_flags(flags);
 
-    if (xinw_debug > 1) 
-	printk(KERN_DEBUG "xircom: SPCQ %x SPU %x LIF %x ISPLQ %x MHS %x rxtx %x %x tx %x %x %x %x\n", 
-	       readb(priv->ramBase + XIRCOM_EREG_SPCQ),
-	       readb(priv->ramBase + XIRCOM_EREG_SPU),
-	       readb(priv->ramBase + XIRCOM_EREG_LIF),
-	       readb(priv->ramBase + XIRCOM_EREG_ISPLQ),
-	       readb(priv->ramBase + XIRCOM_EREG_MHS),
-	       readb(priv->ramBase + XIRCOM_EREG_EC + 0xe),
-	       readb(priv->ramBase + XIRCOM_EREG_EC + 0xf),
-	       readb(priv->ramBase + XIRCOM_EREG_EC + 0x18),
-	       readb(priv->ramBase + XIRCOM_EREG_EC + 0x19),
-	       readb(priv->ramBase + XIRCOM_EREG_EC + 0x1a),
-	       readb(priv->ramBase + XIRCOM_EREG_EC + 0x1b)
-	    );
+    DEBUG(2, "xircom: SPCQ %x SPU %x LIF %x ISPLQ %x MHS %x rxtx %x"
+	  " %x tx %x %x %x %x\n", 
+	  readb(priv->ramBase + XIRCOM_EREG_SPCQ),
+	  readb(priv->ramBase + XIRCOM_EREG_SPU),
+	  readb(priv->ramBase + XIRCOM_EREG_LIF),
+	  readb(priv->ramBase + XIRCOM_EREG_ISPLQ),
+	  readb(priv->ramBase + XIRCOM_EREG_MHS),
+	  readb(priv->ramBase + XIRCOM_EREG_EC + 0xe),
+	  readb(priv->ramBase + XIRCOM_EREG_EC + 0xf),
+	  readb(priv->ramBase + XIRCOM_EREG_EC + 0x18),
+	  readb(priv->ramBase + XIRCOM_EREG_EC + 0x19),
+	  readb(priv->ramBase + XIRCOM_EREG_EC + 0x1a),
+	  readb(priv->ramBase + XIRCOM_EREG_EC + 0x1b));
     return &priv->stats;
 }
 
@@ -1160,8 +1123,7 @@ static int xinw_rx(struct device *dev)
     int i;
     u_char *ptr;
 
-    if (xinw_debug > 2) 
-	printk(KERN_DEBUG "xinw_rx: Receiving ... \n");
+    DEBUG(3, "xinw_rx: Receiving ... \n");
 
     wait_WOC(iobase);
     /* Data offset and maxData */
@@ -1197,15 +1159,15 @@ static int xinw_rx(struct device *dev)
 	}
    
 	 
-	if (xinw_debug > 2) 
-	    printk(KERN_DEBUG "Getting packet of length %d at %x, offset %x\n", 
-		   rcvLen, rcvList, dataOffset);
+	DEBUG(3, "Getting packet of length %d at %x, offset %x\n", 
+	      rcvLen, rcvList, dataOffset);
       
 	/* Use the pcmcia_cs k_compat.h version */
 	skb = ALLOC_SKB(rcvLen+3);
       
 	if (skb == NULL) {
-	    printk(KERN_DEBUG "xircnw_rx: Could not allocate an sk_buff of length %d\n", rcvLen);
+	    DEBUG(1, "xircnw_rx: Could not allocate an sk_buff of "
+		  "length %d\n", rcvLen);
 	    ++priv->stats.rx_dropped; 
 	    return 0;
 	}
@@ -1255,8 +1217,7 @@ static int xinw_rx(struct device *dev)
 	writeb(XIRCOM_CMD_SRP, ramBase + XIRCOM_EREG_CB + 0);
 	writeb(XIRCOM_CMD_EOC, ramBase + XIRCOM_EREG_CB + 1);
       
-	if (xinw_debug > 2) 
-	    printk(KERN_DEBUG "Packet reception ok\n");
+	DEBUG(3, "Packet reception ok\n");
       
 	priv->stats.rx_packets++;
 #if (LINUX_VERSION_CODE >= VERSION(2,1,25))
@@ -1270,7 +1231,7 @@ static int xinw_rx(struct device *dev)
 static int xinw_open(struct device *dev)
 {
     dev_link_t *link;
-    printk(KERN_DEBUG "xinw_open: starting.\n");
+    DEBUG(2, "xinw_open: starting.\n");
 
     for (link = dev_list; link; link = link->next)
 	if (link->priv == dev) break;
@@ -1315,13 +1276,12 @@ int init_module(void)
 {
     servinfo_t serv;
 	
+    DEBUG(0, "%s\n", version);
     CardServices(GetCardServicesInfo, &serv);
     if (serv.Revision != CS_RELEASE_CODE) {
 	printk("xircnw_cs: Card Services release does not match!\n");
 	return -1;
     }
-    printk("xircnw_cs: Card services version = %d\n", CS_RELEASE_CODE);
-    printk(version);
     register_pcmcia_driver(&dev_info, &xircnw_attach, &xircnw_detach);
   
     return 0;
@@ -1329,7 +1289,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-    printk("xircnw_cs: unloading\n");
+    DEBUG(0, "xircnw_cs: unloading\n");
     unregister_pcmcia_driver(&dev_info);
     while (dev_list != NULL) {
 	if (dev_list->state & DEV_CONFIG)
@@ -1351,16 +1311,18 @@ static void set_multicast_list(struct device *dev)
     short iobase = dev->base_addr;
     u_char* ramBase = ((xircom_private*) dev->priv)->ramBase;
     u_char  rcvMode = 0;
-   
-    if (xinw_debug > 3) {
+
+#ifdef PCMCIA_DEBUG
+    if (pc_debug > 2) {
 	static int old = 0;
 	if (old != dev->mc_count) {
 	    old = dev->mc_count;
-	    printk("%s: setting Rx mode to %d addresses.\n",
-		   dev->name, old);
+	    DEBUG(0, "%s: setting Rx mode to %d addresses.\n",
+		  dev->name, old);
 	}
     }
-   
+#endif
+    
     if (dev->mc_count || (dev->flags & IFF_ALLMULTI)) {
 	/* Multicast Mode */
 	rcvMode = rxConfRxEna + rxConfAMP + rxConfBcast;
@@ -1372,7 +1334,6 @@ static void set_multicast_list(struct device *dev)
 	rcvMode = rxConfRxEna + rxConfBcast;
     }
    
-    printk("xircom set_multicast_list: rcvMode to %x\n", rcvMode);
     /* Now set receive mode */
     writeb(XIRCOM_CMD_SRC, ramBase + XIRCOM_EREG_CB + 0);
     writeb(rcvMode, ramBase + XIRCOM_EREG_CB + 1);
@@ -1386,15 +1347,17 @@ set_multicast_list(struct device *dev, int num_addrs, void *addrs)
     short iobase = dev->base_addr;
     u_char* ramBase = ((xircom_private*) dev->priv)->ramBase;
     u_char  rcvMode = 0;
-   
-    if (xinw_debug > 3) {
+
+#ifdef PCMCIA_DEBUG
+    if (pc_debug > 2) {
 	static int old = 0;
 	if (old != num_addrs) {
 	    old = num_addrs;
-	    printk("%s: setting Rx mode to %d addresses.\n",
-		   dev->name, num_addrs);
+	    DEBUG(0, "%s: setting Rx mode to %d addresses.\n",
+		  dev->name, num_addrs);
 	}
     }
+#endif
    
     if ((num_addrs > 0) || (num_addrs == -2)) {
 	/* Multicast Mode */
@@ -1407,7 +1370,6 @@ set_multicast_list(struct device *dev, int num_addrs, void *addrs)
 	rcvMode = rxConfRxEna + rxConfBcast;
     }
 
-    printk("xircom set_multicast_list: rcvMode to %x\n", rcvMode);
     /* Now set receive mode */
     writeb(XIRCOM_CMD_SRC, ramBase + XIRCOM_EREG_CB + 0);
     writeb(rcvMode, ramBase + XIRCOM_EREG_CB + 1);
